      subroutine arc

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  arc      Updated     1999 November 3 12:10.
c               arc      Originated  1997 November 6 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find, at a point on a quadric surface:  the normal vector;
c               the number of curves passing through that point, and lying in
c               the surface and in a plane containing the normal vector, having
c               zero or extreme values of the curvature; and for each such
c               curve, the corresponding radius of curvature, center of
c               curvature, unit direction vector at that point, and the normal
c               vector of the plane containing the curve.
c
c     Logic:    Entry points:  arc.
c arc           Called by:  helpall, main.
c
c arc           Calls:  aptcris, aptvxun, ptfind, quadfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      dimension rcurve(4)             ! Radius of curvature.
      dimension cx(4)                 ! Coordinate x of center of curvature.
      dimension cy(4)                 ! Coordinate y of center of curvature.
      dimension cz(4)                 ! Coordinate z of center of curvature.
      dimension ux(4)                 ! Component x of unit direction vector.
      dimension uy(4)                 ! Component y of unit direction vector.
      dimension uz(4)                 ! Component z of unit direction vector.

c=======================================================================********

c.... Input line:  arc [help]

 8110 format ('  Command "arc":  acceptable input lines include:' /
     &  '  "arc [help]"        Display this message.' //
     &  '  "arc PNAME QNAME"   Find, at point PNAME:' /
     &  '     the normal vector of the family of quadric surfaces',
     &  ' QNAME;' /
     &  '     the number of curves in the surface, passing through',
     &  ' that point,' /
     &  '     and lying in a plane containing the normal vector,' /
     &  '     having zero or extreme values of the curvature;' /
     &  '     and for each such curve, the corresponding curvature,' /
     &  '     radius of curvature, center of curvature, and unit',
     &  ' direction vector.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  arc PNAME QNAME

c.... Test input for validity.

      nerr = 0

 8120 format ('  arc ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 3 words on input line:  ',i3)
      if (nword .ne. 3) then
        nerr = 1
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
      endif

c.... Find the point.

      call ptfind (2, np)

      if (np .eq. 0) then
        nerr = 1
      endif

c.... Find the quadric surface.

      call quadfind (3, nq)

      if (nq .eq. 0) then
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c.... Find the curves with zero and extreme curvature, and their
c....   radii, centers of curvature, and direction vectors.

      call aptcris (x(np), y(np), z(np), qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol, bx, by, bz,
     &              nrad, rcurve, cx, cy, cz, ux, uy, uz, nerr)

 8210 format (/ 'Normal vector is zero.  Curvatures are indeterminate.')
 8220 format (/ 'Normal       (xyz)',1p3e20.12)
 8230 format (/ 'Number of curves with zero or extreme',
     &          ' curvature = ',i3,'.' //
     &          'Curvature, radius and center of curvature, unit',
     &  ' direction vector along curve,' /
     &         '  and normal vector of plane containing center and',
     &  ' curve:' )
 8240 format (/ 'Curvature         ',1p3e20.12)    
 8250 format (  'Radius            ',1p3e20.12)    
 8260 format (  'Center       (xyz)',1p3e20.12)    
 8270 format (  'Direction    (xyz)',1p3e20.12)    
 8280 format (  'Plane normal (xyz)',1p3e20.12)    

      if (nrad .eq. 0) then
        write (   6, 8210)
        write (iout, 8210)
        go to 999
      else
        write (   6, 8220) bx, by, bz
        write (iout, 8220) bx, by, bz
        write (   6, 8230) nrad
        write (iout, 8230) nrad
        do 240 n = 1, nrad
          curve = 1.0 / rcurve(n)
          if (abs (curve) .le. tol) curve = 0.0
          write (   6, 8240) curve
          write (iout, 8240) curve
          write (   6, 8250) rcurve(n)
          write (iout, 8250) rcurve(n)
          write (   6, 8260) cx(n), cy(n), cz(n)
          write (iout, 8260) cx(n), cy(n), cz(n)
          write (   6, 8270) ux(n), uy(n), uz(n)
          write (iout, 8270) ux(n), uy(n), uz(n)

          call aptvxun (bx, by, bz, ux(n), uy(n), uz(n), 1, tol,
     &                  pnx, pny, pnz, pnlen, nerra)

          write (   6, 8280) pnx, pny, pnz
          write (iout, 8280) pnx, pny, pnz

  240   continue
      endif
 
c=======================================================================********

  999 return

c.... End of subroutine arc.          (+1 line)
      end
