      subroutine bigint

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  bigint   Updated     2006 July 6 13:45.
c               bigint   Originated  2005 June 6 13:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To define and do mathematical operations on big integers
c               stored as an array of single digits in base 10 (decimal).
c
c               The operations include addition, subtraction, multiplication,
c               division, modulus, integer powers, integer roots (accurate to
c               one digit) and factorials.
c
c     Logic:    Entry points:  bigint.
c bigint        Called by:  helpall, main.
c
c bigint        Calls:  aptatob, aptbadd, aptbasb, aptbdiv, aptbfac, aptbmul,
c               aptbpac, aptbpow, aptbrtn, aptbsub, aptbtod, aptchia, aptchmv,
c               aptltob, bigcopy, bigfind, bigout, bigtran, bigwork, bigzero,
c               incrsubs, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8  amount             ! Amount of output from bigout.

      character*80 awordx             ! Temporary command word.
      character*80 awordy             ! Temporary command word.
      character*80 aword4             ! Temporary command word.
      character*80 aword6             ! Temporary command word.
      character*26 atoz               ! Upper case ASCII characters A to Z.

      character*32 abigtagx           ! Descriptive tag for big integer.
      character*1  acharx             ! Single or double quote.
c***DEBUG begins.
      character*64 abug
 9330 format ('bigint DEBUG.  ',a)
c***DEBUG ends.

c=======================================================================********

c.... Input line:  big [help]

 8110 format ('  Command "big":  acceptable input lines include:' /
     &  '  "big [help]"          Display this message.' //
     &  '  "big all"             Display all big integers.' /
     &  '  "big all 0"           Zero out all big integers.' /
     &  '  "big BIGNAME"         Display big integer BIGNAME and its',
     &  ' one-word' /
     &  '     integer and floating point values, if any,',
     &  ' and log10, if any.' /
     &  '     BIGNAME must be a single capital letter from A to Z.' //
     &  '  "big BIGNAME [binary,octal,hex]"' /
     &  '     Display big integer BIGNAME in number base [2, 8, 16].' //
     &  '  "big BIGNAME base NBASE"' /
     &  '     Display big integer BIGNAME in number base NBASE.' //
     &  '  "big BIGNAME [N|1] = INT1 INT2 INT3 ..."' /
     &  '     Replace digits N (default 1), N+1, N+2, ... of big',
     &  ' integer BIGNAME' /
     &  '     with integer groups INT1, INT2, INT3, ...' /
     &  '     If BIGNAME is not zero, and N exceeds the initial length',
     &  ' of BIGNAME,' /
     &  '     all digits in between will be zeros.' /
     &  '     INT1, INT2, INT3, ... may be positive integers or',
     &  ' integer variables,' /
     &  '     but not big integers.' //
     &  '  "big BIGNAME tag ''BIGTAG''"', /
     &  '     Give big integer BIGNAME the tag BIGTAG (up to 32',
     &  ' characters).' /
     &  '  "big BIGNAME = big BIGNAME2"' /
     &  '     Replace big integer BIGNAME with big integer BIGNAME2.' //
     &  '  "big BIGNAME = INT1 [+,-,*,/,] INT2"' /
     &  '     Replace big integer BIGNAME with the [sum, difference,',
     &  ' product, quotient]' /
     &  '     of integers INT1 and INT2.' /
     &  '     In this and the following commands INT1 and INT2 may be' /
     &  '     positive integers or integer variables or big integers.'//
     &  '  Type <return> for more help:  ',$)

 8112 format (/
     &  '  "big BIGNAME = INT1 // INT2"' /
     &  '     Replace big integer BIGNAME with the result of dividing',
     &  ' INT1 by the' /
     &  '     highest possible power of INT2, if the latter is an',
     &  ' exact factor of INT1.' /
     &  '  "big BIGNAME = INT1 mod INT2"' /
     &  '     Replace big integer BIGNAME with the modulus of',
     &  ' integer' /
     &  '     INT1 with respect to integer INT2.' /
     &  '  "big BIGNAME = INT1 cat INT2"' /
     &  '     Replace big integer BIGNAME with the concatenation',
     &  ' of INT1 and INT2.' /
     &  '     If either is zero, do not concatenate it.' /
     &  '  "big BIGNAME = INT1 perm INT2"' /
     &  '     Replace big integer BIGNAME with the number of',
     &  ' permutations' /
     &  '     of INT1 things taken INT2 at a time.' /
     &  '  "big BIGNAME = INT1 comb INT2"' /
     &  '     Replace big integer BIGNAME with the number of',
     &  ' combinations' /
     &  '     of INT1 things taken INT2 at a time.' /
     &  '  "big BIGNAME = INT1 ^ NPOW"' /
     &  '     Replace big integer BIGNAME with the NPOW''th power of',
     &  ' integer INT1.' /
     &  '  "big BIGNAME = INT1 root NROOT"' /
     &  '     Replace big integer BIGNAME with the NROOT''th root of',
     &  ' integer INT1.' /
     &  '  "big BIGNAME = INT1 fact"' /
     &  '     Replace big integer BIGNAME with the factorial of',
     &  ' integer INT1.' /
     &  '  "big BIGNAME = INT1 exp"' /
     &  '     Replace big integer BIGNAME with the exponential',
     &  ' function of integer INT1.' /
     &  '  "big BIGNAME ran N"' /
     &  '     Replace big integer BIGNAME with N random digits.' //
     &  '  Note:  use +/- for BIGNAME to incr/decr the last big',
     &  ' integer name:  ',a1 //
     &  '  Hint:  multiply non-integers by a power of 10 to make',
     &  ' into integers' /
     &  '    before using any of these commands, and locate decimal',
     &  ' point later.' //
     &  '  Synonyms:  [0, zero], [all, list], [binary, bin],',
     &  ' [help, h], [octal, oct],' /
     &  '    [^, **].' )

 8114 format (a)

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        if (ainput .eq. 'tty') then
          read  (   5, 8114) ans
        endif
        write (   6, 8112) abigs
        write (iout, 8112) abigs
        go to 999
      endif

c=======================================================================********

c.... Initialize all 26 big integers, with names A to Z.

      if (nword .eq. 0) then

        atoz = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
c....            2 4 6 8(1)2 4 6 8(2)2 4 6

        call aptchmv (atoz, 1, 26, abig, 1, nerra)

        do nb = 1, nbigm
          call bigzero (nb, nerrzero)
        enddo

cbugc***DEBUG begins.
cbug        abigtag(7) = 'debug test'
cbug        ndig(7) = 77
cbug        idig(1,7) = 3
cbug        idig(77,7) = 5
cbug        amount = 'maximum'
cbug        call bigout (1, nbigm, amount)
cbug        abigtag(7) = ' '
cbug        ndig(7) = 0
cbug        idig(1,7) = 0
cbug        idig(77,7) = 5
cbugc***DEBUG ends.

        go to 999

      endif                           ! Initialize at beginning of run.

c=======================================================================********
cbugc***DEBUG begins.
cbug 9737 format ('bigint DEBUG. abig = "',a,'", abigtag = "',a,
cbug     &  '", ndig = ',i5)
cbug      write (   6, 9737) (abig(n), abigtag(n), ndig(n), n = 1, nbigm)
cbug      write (iout, 9737) (abig(n), abigtag(n), ndig(n), n = 1, nbigm)
cbugc***DEBUG ends.

c.... Input line:  big all

c.... Display all big integers.

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        amount = 'minimum'
        call bigout (1, nbigm, amount)

        go to 999

      endif                           ! Input line:  big all

c=======================================================================********

c.... Input line:  big all zero

c.... Zero out all big integers.

      if ((nword .eq. 3) .and. (aword(2) .eq. 'all') .and.
     &   ((aword(3) .eq. '0') .or. (aword(3) .eq. 'zero'))) then

        do nb = 1, nbigm
          call bigzero (nb, nerrzero)
        enddo

 9112   format ('Zeroed out all big integers.')

        write (   6, 9112)
        write (iout, 9112)

        go to 999

      endif                           ! Input line:  big all 0

c=======================================================================********

c.... Input line:  big BIGNAME

c.... Display big integer BIGNAME.
cbugc***DEBUG begins.
cbug        abigtag(7) = 'debug test'
cbug        ndig(7) = 77
cbug        idig(1,7) = 3
cbug        idig(77,7) = 5
cbug        amount = 'maximum'
cbug        call bigout (7, 7, amount)
cbugc***DEBUG ends.

 8450   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  Named big integer does not exist:  ',a)

      if (nword .eq. 2) then
        call bigfind (2, nb)
        if (nb .ne. 0) then
          amount = 'maximum'
          call bigout (nb, nb, amount)
        else
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
        endif
        go to 999
      endif                           ! Input line:  big BIGNAME

c=======================================================================********

c.... Input line:  big BIGNAME [bin,oct,hex]

      if (nword .eq. 3) then
        if ((aword(3) .eq. 'bin') .or. (aword(3) .eq. 'binary')) then
          aword(3) = 'base'
          nword = 4
          mtype(4) = 2
          aword(4) = '2'
          iword(4) = 2
        elseif ((aword(3) .eq. 'oct') .or. (aword(3) .eq. 'octal')) then
          aword(3) = 'base'
          nword = 4
          aword(4) = '8'
          iword(4) = 8
          mtype(4) = 2
        elseif (aword(3) .eq. 'hex') then
          aword(3) = 'base'
          nword = 4
          mtype(4) = 2
          aword(4) = '16'
          iword(4) = 8
          iword(4) = 16
        endif
      endif                           ! Input line:  big BIGNAME [bin,oct,hex]

c=======================================================================********

c.... Input line:  big BIGNAME base NBASE

c.... Find big integer "awb" = big integer BIGNAME in base NBASE.

      if ((nword .eq. 4) .and. (aword(3) .eq. 'base')) then

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        if (abigtag(nb1) .eq. ' ') abigtag(nb1) = 'base NBASE'

c....   Find NBASE.  See if it is 2 or more.

        call varfind (4, 4)

        if (mtype(4) .ne. 2) then
          write (   6, 8253) ainput, ninput, aword(4)
          write (iout, 8253) ainput, ninput, aword(4)
          go to 999
        endif

 8258   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  NBASE is not 2 or more:  ',a )

        if (iword(4) .le. 1) then
          write (   6, 8258) ainput, ninput, aword(4)
          write (iout, 8258) ainput, ninput, aword(4)
          go to 999
        endif

 8255   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  NBASE is already 10.')

        if (iword(4) .eq. 10) then
          write (   6, 8255) ainput, ninput
          write (iout, 8255) ainput, ninput
          go to 999
        endif

        nbasb = iword(4)

c....   Find working arrays.

        call bigwork (nwb)           ! Display.
        if (nwb .eq. 0) go to 999
        call bigwork (nwq)           ! Temporary.
        if (nwq .eq. 0) go to 999
        call bigwork (nwr)           ! Temporary.
        if (nwr .eq. 0) go to 999
        call bigwork (nwu)           ! Temporary.
        if (nwu .eq. 0) go to 999
        call bigwork (nwv)           ! Temporary.
        if (nwv .eq. 0) go to 999
        call bigwork (nww)           ! Temporary.
        if (nww .eq. 0) go to 999
        call bigwork (nwx)           ! Temporary.
        if (nwx .eq. 0) go to 999
        call bigwork (nwy)           ! Temporary.
        if (nwy .eq. 0) go to 999
        call bigwork (nwz)           ! Temporary.
        if (nwz .eq. 0) go to 999

c....   Find big integer abig(nwb), BIGNAME in number base NBASE format.

        abigtag(nwb) = '? in base ?'
        call aptchmv (abig(nb1), 1, 1,  abigtag(nwb), 1,  nerrch)
        call aptchmv (aword(4),  1, 20, abigtag(nwb), 11, nerrch)

        nerrbase = 0
cbugc***DEBUG begins.
cbug 9333 format ('DEBUG.  nbug=',i3,'  nbasb=',i3,'  nba=',i5,'  nwb=',i5)
cbug      nbug = 1
cbug      write (   6, 9333) nbug, nbasb, nb1, nwb
cbug      write (iout, 9333) nbug, nbasb, nb1, nwb
cbugc***DEBUG ends.

        call aptbasb (10, nbasb, idig(1,nb1), ndig(nb1),
     &                idig(1,nwq), idig(1,nwr),
     &                idig(1,nwu), idig(1,nwv),
     &                idig(1,nww), idig(1,nwx),
     &                idig(1,nwy), idig(1,nwz), ndigm,
     &                idig(1,nwb), ndig(nwb), nerrbase)
cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (   6, 9333) nbug, nbasb, nb1, nwb
cbug      write (iout, 9333) nbug, nbasb, nb1, nwb
cbug      amount     = 'Maximum'                      ! DEBUG.
cbug      call bigout (nb1, nb1, amount)                      ! DEBUG.
cbug      call bigout (nwb, nwb, amount)                      ! DEBUG.
cbugc***DEBUG ends.

c....   Display big integer BIGNAME in base 10 and in base nbasb.

        if (nerrbase .eq. 0) then
          amount     = 'minimum'
          call bigout (nb1, nb1, amount)
          amount     = 'Maximum'
          call bigout (nwb, nwb, amount)
        endif
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 9333) nbug, nbasb, nb1, nwb
cbug      write (iout, 9333) nbug, nbasb, nb1, nwb
cbug      amount     = 'Maximum'                      ! DEBUG.
cbug      call bigout (nb1, nb1, amount)                      ! DEBUG.
cbug      call bigout (nwb, nwb, amount)                      ! DEBUG.
cbugc***DEBUG ends.

c....   Zero out working arrays, including base nbasb version of BIGNAME.

        call bigzero (nwb, nerrzero)
        call bigzero (nwq, nerrzero)
        call bigzero (nwr, nerrzero)
        call bigzero (nwu, nerrzero)
        call bigzero (nwv, nerrzero)
        call bigzero (nww, nerrzero)
        call bigzero (nwx, nerrzero)
        call bigzero (nwy, nerrzero)
        call bigzero (nwz, nerrzero)

 8458   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not translate to number base NBASE.')

        if (nerrbase .ne. 0) then
          write (   6, 8458) ainput, ninput
          write (iout, 8458) ainput, ninput
        endif

        go to 999

      endif                           ! Input line:  big BIGNAME base NBASE

c=======================================================================********

c.... Input line:  big BIGNAME tag 'BIGTAG'

      if (aword(3) .eq. 'tag') then

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

c....   Find BIGTAG.

        ibeg = lword(1) + lword(2) + lword(3) + 3
        acharx = '"'
        ibeg1  = 81
        do 152 n = ibeg, 80

          if (achar(n) .eq. '"') then
            ibeg1  =  n + 1
            acharx = '"'
            go to 154
          endif

          if (achar(n) .eq. '''') then
            ibeg1  =  n + 1
            acharx = ''''
            go to 154
          endif

  152   continue

  154   iend1 = 0
        do 156 n = 80, ibeg1 + 1, -1

          if (achar(n) .eq. acharx) then
            iend1 = n - 1
            go to 158
          endif

  156   continue

 8158   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Tag not properly quoted.')

  158   if (ibeg1 .ge. iend1) then
          write (   6, 8158) ainput, ninput
          write (iout, 8158) ainput, ninput
          go to 999
        endif

 8160   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Tag > 32 characters:  ',i3)

        ltag = iend1 - ibeg1 + 1
        if (ltag .gt. 32) then
          write (   6, 8160) ainput, ninput, ltag
          write (iout, 8160) ainput, ninput, ltag
          go to 999
        endif

        abigtagx = ' '
        call aptchmv (achar, ibeg1, ltag, abigtagx, 1, nerra)

c....   Display BIGNAME and BIGTAG.

        abigtag(nb1) = abigtagx

        amount = 'maximum'
        call bigout (nb1, nb1, amount)

        go to 999

      endif                           ! Input line:  big BIGNAME tag "BIGTAG"

c=======================================================================********

c.... See if last big integer name should be incremented and used.

 8128 format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last big integer can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = abigs
        call incrsubs (1, aword(2), 1, 1, nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, abigs
          write (iout, 8128) ainput, ninput, abigs
          go to 999
        endif
      endif

 8129 format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last big integer can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = abigs
        call incrsubs (-1, aword(2), 1, 1, nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, abigs
          write (iout, 8129) ainput, ninput, abigs
          go to 999
        endif
      endif

c=======================================================================********

c.... Input line:  big BIGNAME = big BIGNAME2

c.... Replace a big integer with another big integer.

      if ((nword .eq. 5) .and. (aword(3) .eq. '=') .and.
     &   (aword(4) .eq. 'big')) then

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        call bigfind (5, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(5)
          write (iout, 8450) ainput, ninput, aword(5)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)

        if (abigtag(nb2) .eq. ' ') then
          abigtag(nb2) = abig(nb2)
        endif

        call bigout (nb2, nb2, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = abig(nb1)
        endif

c....   Copy BIGNAME2 into BIGNAME.

        call bigcopy (nb2, nb1, nerrcopy)

c....   Display big integer BIGNAME.

        if (nerrcopy .eq. 0) then
          abigs  = abig(nb1)
          amount = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

        go to 999

      endif                           ! Input line:  big BIGNAME = big BIGNAME2

c=======================================================================********

c.... Input line:  big BIGNAME ran N

c.... Replace a big integer with a random big integer.

      if ((nword .eq. 4) .and. (aword(3) .eq. 'ran')) then

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = abig(nb1)
        endif

c....   Find N.  See if it is 1 or more.

        call varfind (4, 4)

        if ((mtype(4) .ne. 2) .or. (iword(4) .le. 0)) then
          write (   6, 8250) ainput, ninput, aword(4)
          write (iout, 8250) ainput, ninput, aword(4)
          go to 999
        endif

        if (iword(4) .gt. ndigm) then
          write (   6, 8259) ainput, ninput, ndigm, aword(4)
          write (iout, 8259) ainput, ninput, ndigm, aword(4)
          go to 999
        endif

c....   Copy N random digits into BIGNAME.

        ndig(nb1) = iword(4)
        do n = 1, ndig(nb1)
          iran = 10.0 * ranf ()
          idig(n,nb1) = iran
        enddo

c....   Display big integer BIGNAME.

        abigs  = abig(nb1)
        amount = 'maximum'
        call bigout (nb1, nb1, amount)

        go to 999

      endif                           ! Input line:  big BIGNAME ran N

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 + INT2
c.... Input line:  big BIGNAME = INT1 - INT2
c.... Input line:  big BIGNAME = INT1 * INT2
c.... Input line:  big BIGNAME = INT1 / INT2
c.... Input line:  big BIGNAME = INT1 // INT2
c.... Input line:  big BIGNAME = INT1 & INT2
c.... Input line:  big BIGNAME = INT1 mod  INT2

      if ((nword .eq. 6)          .and.
     &    (aword(3) .eq. '=')     .and.
     &   ((aword(5) .eq. '+')     .or.
     &    (aword(5) .eq. '-')     .or.
     &    (aword(5) .eq. '*')     .or.
     &    (aword(5) .eq. '/')     .or.
     &    (aword(5) .eq. '//')    .or.
     &    (aword(5) .eq. 'cat')   .or.
     &    (aword(5) .eq. 'mod'))) then

        aword4 = aword(4)
        lword4 = lword(4)
        aword6 = aword(6)
        lword6 = lword(6)

c....   Find big integer BIGNAME = abig(nb1).

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif
cbugc***DEBUG begins.
cbug      abug = 'Found big integer BIGNAME line 598.'
cbug      write (iout, 9330) abug
cbugc***DEBUG ends.

 8210   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Copying over existing big integer:  ',a )

        if ((ndig(nb1) .gt. 1) .or.
     &      ((ndig(nb1) .eq. 1) .and. (idig(1,nb1) .ne. 0))) then
          write (   6, 8210) ainput, ninput, abig(nb1)
          write (iout, 8210) ainput, ninput, abig(nb1)
        endif                         ! Copying over existing big integer.

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 [+,-,*,/,//,mod] INT2'
        endif

 8460   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
     &   '  Named big integer does not exist:  ',a)

c....   Find INT1 = abig(nb2).

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(4)
          write (iout, 8460) ainput, ninput, aword(4)
        endif

        nbwork2 = 0
        if (nb2 .eq. 0) then
          call bigtran (4, nb2, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork2 = nb2
          abigtag(nb2) = 'INT1'
        endif
cbugc***DEBUG begins.
cbug 9010 format ('bigint DEBUG.  nb =',i5,'  abig(nb) = "',a,'"')
cbug      write (   6, 9010) nbwork2, abig(nbwork2)
cbug      write (iout, 9010) nbwork2, abig(nbwork2)
cbug      amount = 'minimum'
cbug      call bigout (nbwork2, nbwork2, amount)
cbugc***DEBUG ends.

c....   Find INT2 = abig(nb3).

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(6)
          write (iout, 8460) ainput, ninput, aword(6)
        endif

        nbwork3 = 0
        if (nb3 .eq. 0) then
          call bigtran (6, nb3, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork3 = nb3
          abigtag(nb3) = 'INT2'
        endif
cbugc***DEBUG begins.
cbug      write (   6, 9010) nbwork3, abig(nbwork3)
cbug      write (iout, 9010) nbwork3, abig(nbwork3)
cbug      amount = 'minimum'
cbug      call bigout (nbwork3, nbwork3, amount)
cbugc***DEBUG ends.

      endif                 ! Input line:  big BIGNAME = INT1 [+,-,*,/,mod] INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 + INT2

c.... Find big integer BIGNAME = the sum of big integers INT1 and INT2.

      if ((nword .eq. 6) .and. (aword(5) .eq. '+') .and.
     &    (aword(3) .eq. '=')) then

c....   Find BIGNAME = abig(nb1).

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

c....   Find INT1 = abig(nb2).

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

c....   Find INT2 = abig(nb3).

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(6)
          write (iout, 8450) ainput, ninput, aword(6)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 + INT2'
        endif

c....   Find a working array to store result in.

        call bigwork (nw1)
        if (nw1 .eq. 0) go to 999

        abigtag(nb1) = '? + ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 5, nerrch)

c....   Find abig(nb1), the sum of big integers INT1 and INT2.

        call aptbadd (10, idig(1,nb2), ndig(nb2),
     &                    idig(1,nb3), ndig(nb3),
     &                    idig(1,nw1), ndigm,
     &                    idig(1,nb1), ndig(nb1), nerradd)

c....   Display big integer BIGNAME.

        if (nerradd .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

cbugc***DEBUG begins.
cbug      write (   6, 9010) nb1, abig(nw1)
cbug      write (iout, 9010) nb1, abig(nw1)
cbug      write (   6, 9010) nbwork2, abig(nbwork2)
cbug      write (iout, 9010) nbwork2, abig(nbwork2)
cbug      write (   6, 9010) nbwork3, abig(nbwork2)
cbug      write (iout, 9010) nbwork3, abig(nbwork2)
cbugc***DEBUG ends.

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nw1, nerrzero)

 8260   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Bad data or result size exceeds memory space.')

        if (nerradd .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                    ! Input line:  big BIGNAME = INT1 + INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 cat INT2

c.... Find big integer BIGNAME = the concatenation of INT1 and INT2.

      if ((nword .eq. 6) .and. (aword(5) .eq. 'cat') .and.
     &    (aword(3) .eq. '=')) then
cbugc***DEBUG begins.
cbug      abug = 'Found command "big BIGNAME = INT1 cat INT2"'
cbug      write (iout, 9330) abug
cbugc***DEBUG ends

c....   Find BIGNAME = abig(nb1).

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

c....   Find INT1 = abig(nb2).

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

c....   Find INT2 = abig(nb3).

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(6)
          write (iout, 8450) ainput, ninput, aword(6)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)
cbugc***DEBUG begins.
cbug      abug = 'Found big integers nb1,nb2, nb3.  Write below.'
cbug      write (iout, 9330) abug
cbug      amount = 'maximum'
cbug      call bigout (nb1, nb1, amount)
cbug      call bigout (nb2, nb2, amount)
cbug      call bigout (nb3, nb3, amount)
cbugc***DEBUG ends

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 cat INT2'
        endif

c....   Find a working array to store result in.

        call bigwork (nw1)
        if (nw1 .eq. 0) go to 999
cbugc***DEBUG begins.
cbug      abug = 'Found working array nw1'
cbug      write (iout, 9330) abug
cbug      amount = 'maximum'
cbug      call bigout (nw1, nw1, amount)
cbugc***DEBUG ends

        abigtag(nb1) = '? cat ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 7, nerrch)
cbugc***DEBUG begins.
cbug      abug = 'Gave nb2 and nb3 tags.'
cbug      write (iout, 9330) abug
cbug        amount = 'maximum'
cbug        call bigout (nb2, nb2, amount)
cbug        call bigout (nb3, nb3, amount)
cbugc***DEBUG ends

c....   Find abig(nb1), the concatenation of big integers INT1 and INT2.
c....     Do not concatenate INT1 = 0 or INT2 = 0.

        call aptbcat (10, idig(1,nb2), ndig(nb2),
     &                    idig(1,nb3), ndig(nb3),
     &                    idig(1,nw1), ndigm,
     &                    idig(1,nb1), ndig(nb1), nerrcat)

c....   Display big integer BIGNAME.

        if (nerrcat .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

cbugc***DEBUG begins.
cbug      write (   6, 9010) nb1, abig(nw1)
cbug      write (iout, 9010) nb1, abig(nw1)
cbug      write (   6, 9010) nbwork2, abig(nbwork2)
cbug      write (iout, 9010) nbwork2, abig(nbwork2)
cbug      write (   6, 9010) nbwork3, abig(nbwork2)
cbug      write (iout, 9010) nbwork3, abig(nbwork2)
cbugc***DEBUG ends.

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nw1, nerrzero)

        if (nerrcat .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                    ! Input line:  big BIGNAME = INT1 cat INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 - INT2

c.... Find big integer BIGNAME = the difference of big integers INT1 and INT2.

      if ((nword .eq. 6) .and. (aword(5) .eq. '-') .and.
     &    (aword(3) .eq. '=')) then

c....   Find BIGNAME = abig(nb1).

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

c....   Find INT1 = abig(nb2).

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

c....   Find INT1 = abig(nb2).

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(6)
          write (iout, 8450) ainput, ninput, aword(6)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 - INT2'
        endif

c....   Find a working array to store result in.

        call bigwork (nw1)
        if (nw1 .eq. 0) go to 999

        abigtag(nb1) = '? - ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 5, nerrch)

c....   Find abig(nb1), the difference of big integers INT1 and INT2.

        call aptbsub (10, idig(1,nb2), ndig(nb2),
     &                    idig(1,nb3), ndig(nb3),
     &                    idig(1,nw1), ndigm,
     &                    idig(1,nb1), ndig(nb1), isign, nerrsub)

 8372   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Big digit     INT1 - INT2 is negative.' /
     &    '  Replaced with INT2 - INT1.' )

        asign(nb1) = ' '
        if (isign .eq. -1) then
          asign(nb1) = '-'
          write (   6, 8372) ainput, ninput
          write (iout, 8372) ainput, ninput
          call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 4, nerrch)
          call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 8, nerrch)
        endif

c....   Display big integer BIGNAME.

        if (nerrsub .eq. 0) then
          abigs  = abig(nb1)
          amount = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nw1, nerrzero)

        if (nerrsub .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                    ! Input line:  big BIGNAME = INT1 - INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 * INT2

c.... Find big integer BIGNAME = the product of big integers INT1 and INT2.

      if ((nword .eq. 6) .and. (aword(5) .eq. '*') .and.
     &    (aword(3) .eq. '=')) then

c....   Find BIGNAME = abig(nb1).

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

c....   Find INT1 = abig(nb2).

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

c....   Find INT2 = abig(nb3).

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(6)
          write (iout, 8450) ainput, ninput, aword(6)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 * INT2'
        endif

c....   Find a working array to store result in.

        call bigwork (nw1)
        if (nw1 .eq. 0) go to 999

        abigtag(nb1) = '? * ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 5, nerrch)

c....   Find abig(nb1), the product of big integers INT1 and INT2.

        call aptbmul (10, idig(1,nb2), ndig(nb2),
     &                    idig(1,nb3), ndig(nb3),
     &                    idig(1,nw1), ndigm,
     &                    idig(1,nb1), ndig(nb1), nerrmul)

c....   Display big integer BIGNAME.

        if (nerrmul .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nw1, nerrzero)

        if (nerrmul .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                    ! Input line:  big BIGNAME = INT1 * INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 [/,//] INT2

c.... Find big integer BIGNAME = the quotient of big integers INT1 and INT2
c....   (option '/') or dividing INT1 by INT2 exactly, as many times as
c....   possible (option '//').

      if ((nword .eq. 6) .and. (aword(3) .eq. '=') .and.
     &   ((aword(5) .eq. '/') .or. (aword(5) .eq. '//'))) then

c....   Find and display big integers BIGNAME, INT1 and INT2.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif
cbugc***DEBUG begins.
cbug      abug = 'Found big integer BIGNAME line 1011.'
cbug      write (iout, 9330) abug
cbugc***DEBUG ends.

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(6)
          write (iout, 8450) ainput, ninput, aword(6)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 / INT2'
        endif
cbugc***DEBUG begins.
cbug      abug = 'Tagged big integer BIGNAME line 1054.'
cbug      write (iout, 9330) abug
cbugc***DEBUG ends.

c....   See if INT2 = 0.

 8380   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Division by INT2 = 0 is not allowed.' )

        if ((ndig(nb3) .le. 0) .or.
     &     ((ndig(nb3) .eq. 1) .and. (idig(1,nb3) .eq. 0))) then
          write (   6, 8380) ainput, ninput
          write (iout, 8380) ainput, ninput
          call bigzero (nbwork2, nerrzero)
          call bigzero (nbwork3, nerrzero)
          call bigzero (nb1, nerrzero)
          go to 999
        endif

c....   See if INT2 = 1.

 8382   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Division by INT2 = 1 has no effect.' )

        if ((ndig(nb3) .eq. 1) .and.  (idig(1,nb3) .eq. 1)) then
          write (   6, 8382) ainput, ninput
          write (iout, 8382) ainput, ninput
          abigtag(nb1) = 'INT1 / 1 = INT1'
          ndig(nb1) = ndig(nb2)
          do n = 1, ndigm
            idig(n,nb1) = idig(n,nb2)
          enddo
          amount = 'maximum'
          call bigout (nb1, nb1, amount)
          call bigzero (nbwork2, nerrzero)
          call bigzero (nbwork3, nerrzero)
          go to 999
        endif
cold
coldc....   See if INT1 = 0 or 1.  Can not remove a factor.
cold
cold 8384   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
cold     &    '  INT2 cannot be an exact factor of INT1.')
cold
cold        if (aword(5) .eq. '//') then
cold          if ((ndig(nb2) .eq. 1) .and. (idig(1,nb2) .le. 1)) then
cold            write (   6, 8384) ainput, ninput
cold            write (iout, 8384) ainput, ninput
cold          endif
cold        endif

cbugc***DEBUG begins.
cbugc....   Find the floating point value of the quotient.
cbug
cbug 8406   format (2x,a1,' / ',a1,' (dec) =',1pe22.14)
cbug
cbug        call aptbtod (10, idig(1,nb2), ndig(nb2), idmax, iemax,
cbug     &                ideca2, fdeca2, floga2, nerra)
cbug        call aptbtod (10, idig(1,nb3), ndig(nb3), idmax, iemax,
cbug     &                ideca3, fdeca3, floga3, nerra)
cbug
cbug        if ((floga2 .ne. -99.0) .and. (floga3 .ne. -99.0)) then
cbug
cbug          flogq = floga2 - floga3
cbug          if (flogq .le. iemax) then
cbug            fnorm = 10.0 ** flogq
cbug            write (   6, 8406) abig(nb2), abig(nb3), fnorm
cbug            write (iout, 8406) abig(nb2), abig(nb3), fnorm
cbug          endif
cbug        endif
cbugc***DEBUG ends.

c....   Find working arrays.

        call bigwork (nwr)           ! Remainder.
        if (nwr .eq. 0) go to 999
        call bigwork (nwu)           ! Temporary.
        if (nwu .eq. 0) go to 999
        call bigwork (nwv)           ! Temporary.
        if (nwv .eq. 0) go to 999
        call bigwork (nww)           ! Temporary.
        if (nww .eq. 0) go to 999
        call bigwork (nwx)           ! Temporary.
        if (nwx .eq. 0) go to 999
        call bigwork (nwy)           ! Temporary.
        if (nwy .eq. 0) go to 999
        call bigwork (nwz)           ! Temporary.
        if (nwz .eq. 0) go to 999

c....   Find abig(nb1), the quotient of big integers INT1 and INT2.

        abigtag(nb1) = 'Quotient ? / ?'
        abigtag(nwr) = 'Remainder or modulus (?, ?)'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 10, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 14, nerrch)
        call aptchmv (abig(nb2), 1, 1, abigtag(nwr), 23, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nwr), 26, nerrch)

c....   Find working numerator, for option "//".

        ndig(nwu)    = ndig(nb2)
        abigtag(nwu) = abigtag(nb2)
        do n = 1, ndig(nwu)
          idig(n,nwu) = idig(n,nb2)
        enddo

c....   Return here to divide by same factor again (option "//').

        ipow = 0

  360   ipow = ipow + 1
cbugc***DEBUG begins.
cbug 8363 format ('bigint DEBUG.  ipow =',i4)
cbug      abug = 'Will call aptbdiv.  Numerator nwu, denomenator nb3.'
cbug      write (iout, 9330) abug
cbug      write (iout, 8363) ipow
cbug      amount = 'minimum'
cbug      call bigout (nwu, nwu, amount)
cbug      call bigout (nb3, nb3, amount)
cbugc***DEBUG ends.

        call aptbdiv (10, idig(1,nwu), ndig(nwu),
     &                    idig(1,nb3), ndig(nb3),
     &                    idig(1,nwv), idig(1,nww), idig(1,nwx),
     &                    idig(1,nwy), idig(1,nwz), ndigm,
     &                    idig(1,nb1), ndig(nb1),
     &                    idig(1,nwr), ndig(nwr), nerrdiv)

cbugc***DEBUG begins.
cbug      abug = 'Called aptbdiv.  Quotient nb1, Remainder nwr.'
cbug      write (iout, 9330) abug
cbug      write (iout, 8363) ipow
cbug      amount = 'minimum'
cbug      call bigout (nb1, nb1, amount)
cbug      call bigout (nwr, nwr, amount)
cbug      if (ipow .ge. 10) go to 999
cbugc***DEBUG ends.

c....   Display big integers BIGNAME and remainder or mod.

 8364 format (/ a8,'^(',i4,') is an exact factor of ',a8)
 8365 format (/ a24,'^(',i4,') is an exact factor of ',a24)
 8366 format (/ 'big ',a,' = ',a72 / 'raised to power ',i4,' is an',
     &  ' exact factor of' /,'big ',a,' = ',a72)
 8369 format (/ a8,' is not an exact factor of ',a8)
 8370 format (/ a24,' is not an exact factor of ',a24)
 8371 format (/ 'big ',a,' = ',a72 / ' is not an exact factor of' /
     &  'big ',a,' = ',a72)

        if (nerrdiv .eq. 0) then      ! No error in aptbdiv.

c....     Input line:  big BIGNAME = INT1 / INT2

          if (aword(5) .eq. '/') then
            asign(nb1) = ' '
            abigs      = abig(nb1)
            amount     = 'maximum'
            call bigout (nb1, nb1, amount)

            if ((ndig(nwr) .eq. 1) .and. (idig(1,nwr) .eq. 0)) then
              if ((ndig(nb2) .gt. 1) .or. (idig(1,nb2) .ge. 1)) then
                if ((lword4 .le. 8) .and. (lword6 .le. 8)) then
                  write (   6, 8364) aword6, ipow, aword4
                  write (iout, 8364) aword6, ipow, aword4
                elseif ((lword4 .le. 24) .and. (lword6 .le. 24)) then
                  write (   6, 8365) aword6, ipow, aword4
                  write (iout, 8365) aword6, ipow, aword4
                else
                  write (   6, 8366) abig(nb3), aword6, ipow,
     &              abig(nb2), aword4
                  write (iout, 8366) abig(nb3), aword6, ipow,
     &              abig(nb2), aword4
                endif
              endif
            else
              if ((lword4 .le. 8) .and. (lword6 .le. 8)) then
                write (   6, 8369) aword6, aword4
                write (iout, 8369) aword6, aword4
              elseif ((lword4 .le. 24) .and. (lword6 .le. 24)) then
                write (   6, 8370) aword6, aword4
                write (iout, 8370) aword6, aword4
              else
                write (   6, 8371) abig(nb3), aword6, abig(nb2), aword4
                write (iout, 8371) abig(nb3), aword6, abig(nb2), aword4
              endif
              call bigout (nwr, nwr, amount)
            endif
            go to 380
          endif                       ! Input line:  big BIGNAME = INT1 / INT2

c....     Input line:  big BIGNAME = INT1 // INT2

          if (aword(5) .eq. '//') then

c....       See if quotient nb1 = 0 or remainder nwr > 0.

            if (((ndig(nb1) .le. 1) .and. (idig(1,nb1) .le. 0)) .or.
     &          ((ndig(nwr) .gt. 1) .or. (idig(1,nwr) .ge. 1))) then
cbugc***DEBUG begins. AAA
cbug      abug = 'Quotient = 0, remainder = ?.'
cbug      write (iout, 9330) abug
cbug      write (iout, 8363) ipow
cbugc***DEBUG ends.
              ipow = ipow - 1
                ndig(nb1) = ndig(nwu)
                do n = 1, ndig(nb1)
                  idig(n,nb1) = idig(n,nwu)
                enddo
              abigtag(nb1) = 'INT1 / INT2^IPOW'
              asign(nb1) = ' '
              abigs      = abig(nb1)
              amount     = 'maximum'
              call bigout (nb1, nb1, amount)
              if (ipow .gt. 0) then
                if ((lword4 .le. 8) .and. (lword6 .le. 8)) then
                  write (   6, 8364) aword6, ipow, aword4
                  write (iout, 8364) aword6, ipow, aword4
                elseif ((lword4 .le. 24) .and. (lword6 .le. 24)) then
                  write (   6, 8365) aword6, ipow, aword4
                  write (iout, 8365) aword6, ipow, aword4
                else
                  write (   6, 8366) abig(nb3), aword6, ipow,
     &              abig(nb2), aword4
                  write (iout, 8366) abig(nb3), aword6, ipow,
     &              abig(nb2), aword4
                endif
              else
                if ((lword4 .le. 8) .and. (lword6 .le. 8)) then
                  write (   6, 8369) aword6, aword4
                  write (iout, 8369) aword6, aword4
                elseif ((lword4 .le. 24) .and. (lword6 .le. 24)) then
                  write (   6, 8370) aword6, aword4
                  write (iout, 8370) aword6, aword4
                else
                write (   6, 8371) abig(nb3), aword6, abig(nb2), aword4
                write (iout, 8371) abig(nb3), aword6, abig(nb2), aword4
                endif
              endif
              go to 380
            endif                     ! Quotient nb1 = 0 or remainder nwr > 0.

c....       See if quotient nb1 = 1 and remainder nwr = 0.

            if (((ndig(nb1) .eq. 1) .and. (idig(1,nb1) .eq. 1))  .and.
     &          ((ndig(nwr) .le. 1) .and. (idig(1,nwr) .le. 0))) then
cbugc***DEBUG begins. BBB
cbug      abug = 'Quotient = 1, remainder = 0.'
cbug      write (iout, 9330) abug
cbug      write (iout, 8363) ipow
cbugc***DEBUG ends.
              abigtag(nb1) = 'INT1 / INT2^IPOW'
              asign(nb1) = ' '
              abigs      = abig(nb1)
              amount     = 'maximum'
              call bigout (nb1, nb1, amount)
              if ((lword4 .le. 8) .and. (lword6 .le. 8)) then
                write (   6, 8364) aword6, ipow, aword4
                write (iout, 8364) aword6, ipow, aword4
              elseif ((lword4 .le. 24) .and. (lword6 .le. 24)) then
                write (   6, 8365) aword6, ipow, aword4
                write (iout, 8365) aword6, ipow, aword4
              else
                  write (   6, 8366) abig(nb3), aword6, ipow,
     &              abig(nb2), aword4
                  write (iout, 8366) abig(nb3), aword6, ipow,
     &              abig(nb2), aword4
              endif
              go to 380
            endif                     ! Quotient nb1 = 1, remainder nwr = 0.

c....       See if quotient nb1 > 1 and remainder nwr = 0.

            if  (((ndig(nb1) .gt. 1) .or. (idig(1, nb1) .gt. 1))  .and.
     &           ((ndig(nwr) .le. 1) .and. (idig(1,nwr) .le. 0))) then
cbugc***DEBUG begins. DDD
cbug      abug = 'Quotient > 1, remainder = 0.'
cbug      write (iout, 9330) abug
cbug      write (iout, 8363) ipow
cbugc***DEBUG ends.
              ndig(nwu)    = ndig(nb1)
              abigtag(nwu) = 'INT1 / INT2^NPOW'
              do n = 1, ndig(nwu)
                idig(n,nwu) = idig(n,nb1)
              enddo
              go to 360
            endif                     ! Quotient nb1 > 1, remainder nwr = 0.

          endif                       ! Input line:  big BIGNAME = INT1 // INT2

        endif                         ! No error.

c....   Zero out working arrays.

  380   call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nwr, nerrzero)
        call bigzero (nwu, nerrzero)
        call bigzero (nwv, nerrzero)
        call bigzero (nww, nerrzero)
        call bigzero (nwx, nerrzero)
        call bigzero (nwy, nerrzero)
        call bigzero (nwz, nerrzero)

 8385   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not divide.')

        if (nerrdiv .ne. 0) then
          write (   6, 8385) ainput, ninput
          write (iout, 8385) ainput, ninput
        endif

        go to 999

      endif                    ! Input line:  big BIGNAME = INT1 [/,//] INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 mod INT2

c.... Find big integer BIGNAME = modulus big integer INT1 with respect to INT2.

      if ((nword .eq. 6) .and. (aword(5) .eq. 'mod') .and.
     &    (aword(3) .eq. '=')) then

c....   Find and display big integers BIGNAME, INT1 and INT2.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(6)
          write (iout, 8450) ainput, ninput, aword(6)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 mod INT2'
        endif

c....   Find working arrays.

        call bigwork (nwc)           ! Temporary.
        if (nwc .eq. 0) go to 999
        call bigwork (nwg)           ! Temporary.
        if (nwg .eq. 0) go to 999
        call bigwork (nwq)           ! Quotient.
        if (nwq .eq. 0) go to 999
        call bigwork (nwv)           ! Temporary.
        if (nwv .eq. 0) go to 999
        call bigwork (nww)           ! Temporary.
        if (nww .eq. 0) go to 999
        call bigwork (nwx)           ! Temporary.
        if (nwx .eq. 0) go to 999
        call bigwork (nwy)           ! Temporary.
        if (nwy .eq. 0) go to 999
        call bigwork (nwz)           ! Temporary.
        if (nwz .eq. 0) go to 999

c....   See if INT2 is zero.

        if ((ndig(nb3) .le. 0) .or.
     &     ((ndig(nb3) .eq. 1) .and. (idig(1,nb3) .eq. 0))) then
          write (   6, 8380) ainput, ninput
          write (iout, 8380) ainput, ninput
          go to 999
        endif

c....   Find abig(nb1), the modulus of big integer INT1 rel to INT2.

        abigtag(nb1) = 'INT1 mod INT2'

        abigtag(nb1) = 'Modulus (?, ?)'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 10, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 13, nerrch)

        call aptbdiv (10, idig(1,nb2), ndig(nb2),
     &                    idig(1,nb3), ndig(nb3),
     &                    idig(1,nwv), idig(1,nww), idig(1,nwx),
     &                    idig(1,nwy), idig(1,nwz), ndigm,
     &                    idig(1,nwq), ndig(nwq),
     &                    idig(1,nb1), ndig(nb1), nerrdiv)

c....   Display big integer BIGNAME.

        if (nerrdiv .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nwq, nerrzero)
        call bigzero (nwv, nerrzero)
        call bigzero (nww, nerrzero)
        call bigzero (nwx, nerrzero)
        call bigzero (nwy, nerrzero)
        call bigzero (nwz, nerrzero)

 8387   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not divide.')

        if (nerrdiv .ne. 0) then
          write (   6, 8387) ainput, ninput
          write (iout, 8387) ainput, ninput
        endif

        go to 999

      endif                  ! Input line:  big BIGNAME = INT1 mod INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 ^ NPOW
c.... Input line:  big BIGNAME = INT1 ** NPOW
c.... Input line:  big BIGNAME = INT1 root NROOT

c.... Find the big integers on the right side.

      if ((nword .eq. 6)           .and.
     &    (aword(3) .eq. '=')      .and.
     &   ((aword(5) .eq. '^')      .or.
     &    (aword(5) .eq. '**')     .or.
     &    (aword(5) .eq. 'root'))) then

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        if ((ndig(nb1) .gt. 1) .or.
     &      ((ndig(nb1) .eq. 1) .and. (idig(1,nb1) .ne. 0))) then
          write (   6, 8210) ainput, ninput, abig(nb1)
          write (iout, 8210) ainput, ninput, abig(nb1)
        endif                         ! Copying over existing big integer.

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 [^,**] NPOW or INT1 root NROOT'
        endif

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(4)
          write (iout, 8460) ainput, ninput, aword(4)
        endif

        nbwork2 = 0
        if (nb2 .eq. 0) then
          call bigtran (4, nb2, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork2 = nb2
          abigtag(nb2) = 'INT1'
        endif

      endif

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 ^ NPOW

c.... Find big integer BIGNAME = big integer INT1 to the NPOW'th power.

      if ((nword .eq. 6) .and. (aword(3) .eq. '=') .and.
     &   ((aword(5) .eq. '^') .or. (aword(5) .eq. '**'))) then

c....   Find and display big integers BIGNAME and INT1.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

        abigtag(nb2) = 'INT1'

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 ^ NPOW'
        endif

c....   Find power.  See if it is a non-negative integer.

        call varfind (6, 6)

        if (mtype(6) .ne. 2) then
          write (   6, 8250) ainput, ninput, aword(6)
          write (iout, 8250) ainput, ninput, aword(6)
          go to 999
        endif

 8390   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not find a negative power:  ',a )

        if (iword(6) .lt. 0) then
          write (   6, 8390) ainput, ninput, aword(6)
          write (iout, 8390) ainput, ninput, aword(6)
          go to 999
        endif

 8392   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Zero to the zero power is undefined.' )

        if ((iword(6) .eq. 0) .and.
     &      (ndig(nb1)  .le. 1) .and. (idig(1,nb1) .eq. 0)) then
          write (   6, 8392) ainput, ninput
          write (iout, 8392) ainput, ninput
          go to 999
        endif

c....   Display power.

 8391   format (/ 'Power =',i7,'.')

        write (   6, 8391) iword(6)
        write (iout, 8391) iword(6)

c....   Find working arrays to store results in.

        call bigwork (nw1)
        if (nw1 .eq. 0) go to 999
        call bigwork (nw2)
        if (nw2 .eq. 0) go to 999

c....   Find abig(nb1), big integer INT1 to the NPOW'th power.

        abigtag(nb1) = '? ^ ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchia (iword(6), 10, awordx, 80, ibeg, nchar, nerrch)
        call aptchmv (awordx, ibeg, nchar, abigtag(nb1), 5, nerrch)

        call aptbpow (iword(6), 10, idig(1,nb2), ndig(nb2),
     &                              idig(1,nw1), idig(1,nw2), ndigm,
     &                              idig(1,nb1), ndig(nb1), nerrpow)

c....   Display big integer BIGNAME.

        if (nerrpow .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nw1, nerrzero)
        call bigzero (nw2, nerrzero)

        if (nerrpow .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                        ! Input line:  big BIGNAME = INT1 ^ NPOW

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 exp

c.... Find big integer BIGNAME = exp (INT1).

      if ((nword .eq. 5) .and. (aword(3) .eq. '=') .and.
     &    (aword(5) .eq. 'exp')) then

c....   Find and display big integers BIGNAME and INT1.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        if ((ndig(nb1) .gt. 1) .or.
     &      ((ndig(nb1) .eq. 1) .and. (idig(1,nb1) .ne. 0))) then
          write (   6, 8210) ainput, ninput, abig(nb1)
          write (iout, 8210) ainput, ninput, abig(nb1)
        endif                         ! Copying over existing big integer.

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'exp (INT1)'
        endif

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(4)
          write (iout, 8460) ainput, ninput, aword(4)
        endif

        nbwork2 = 0
        if (nb2 .eq. 0) then
          call bigtran (4, nb2, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork2 = nb2
          abigtag(nb2) = 'INT1'
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)

c....   Tag result.

        abigtag(nb1) = 'exp (?)'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 6, nerrch)
 
        call aptbexp (10, idig(1,nb2), ndig(nb2), ndigm,
     &                idmax, iemax, idig(1,nb1), ndig(nb1), nerrl)

c....   Display big integer BIGNAME.

        if (nerrl .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)

        if (nerrl .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                           ! Input line:  big BIGNAME = INT1 exp

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 root NROOT

c.... Find big integer BIGNAME = the NROOT'th root of big integer INT1,
c....    store in big integer BIGNAME.

      if ((nword .eq. 6) .and. (aword(3) .eq. '=') .and.
     &    (aword(5) .eq. 'root')) then

c....   Find and display big integers BIGNAME and INT1.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 root NROOT'
        endif

        abigtag(nb2) = 'INT1'

        call bigout (nb2, nb2, amount)

c....   See if BIGNAME is positive and NROOT is not negative.

        call varfind (6, 6)

        if (mtype(6) .ne. 2) then
          write (   6, 8250) ainput, ninput, aword(6)
          write (iout, 8250) ainput, ninput, aword(6)
          go to 999
        endif

 8395   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Big integer not + or root -:  root  = ',a )

        if (((ndig(nb2) .eq. 1) .and. (idig(1,nb2) .eq. 0)) .or.
     &      (iword(6) .lt. 0)) then
          write (   6, 8395) ainput, ninput, aword(6)
          write (iout, 8395) ainput, ninput, aword(6)
          amount = 'minimum'
          call bigout (nb2, nb2, amount)
          go to 999
        endif

c....   Find the floating point value of the root.

 8405   format (2x,a1,' = ',a1,'^(1/',i4,') (dec) =',f18.15,
     &    ' * 10^(',i5,') (to',i5,' digits in ',a1,')')

        call aptbtod (10, idig(1,nb2), ndig(nb2), idmax, iemax,
     &                ideca, fdeca, floga, nerra)

        if (floga .ne. -99.0) then
          froot = iword(6)
          flogr = floga / froot
          ngood = flogr - log10 (froot)
          ipowr = flogr
          fpowr = ipowr
          fmant = flogr - fpowr
          fnorm = 10.0 ** fmant
          write (   6, 8405) abig(nb1), abig(nb2), iword(6),
     &      fnorm, ipowr, ngood, abig(nb2)
          write (iout, 8405) abig(nb1), abig(nb2), iword(6),
     &      fnorm, ipowr, ngood, abig(nb2)
        endif

c....   Find working arrays to store results in.

        call bigwork (nw1)           ! Temporary BIGNAME.
        if (nw1 .eq. 0) go to 999
        call bigwork (nw2)           ! Temporary BIGNAME^NROOT.
        if (nw2 .eq. 0) go to 999
        call bigwork (nw3)           ! Temporary INT1 - BIGNAME^NROOT.
        if (nw3 .eq. 0) go to 999
        call bigwork (nw4)                         
        if (nw4 .eq. 0) go to 999
        call bigwork (nw5)                         
        if (nw5 .eq. 0) go to 999
        call bigwork (nw6)                         
        if (nw6 .eq. 0) go to 999
        call bigwork (nw7)                         
        if (nw7 .eq. 0) go to 999
        call bigwork (nw8)                         
        if (nw8 .eq. 0) go to 999
        call bigwork (nw9)                         
        if (nw9 .eq. 0) go to 999
        call bigwork (nw10)                         
        if (nw10 .eq. 0) go to 999
        call bigwork (nw11)                         
        if (nw11 .eq. 0) go to 999
        call bigwork (nw12)                         
        if (nw12 .eq. 0) go to 999

c....   Find big integer BIGNAME = the NROOT'th root of big integer INT1.

        abigtag(nb1) = '? root ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchia (iword(6), 10, awordx, 80, ibeg, nchar, nerrch)
        call aptchmv (awordx, ibeg, nchar, abigtag(nb1), 8, nerrch)

        abigtag(nw2) = 'Check ? ^ ?'
        call aptchmv (abig(nb1), 1, 1, abigtag(nw2), 7, nerrch)
        call aptchia (iword(6), 10, awordx, 80, ibeg, nchar, nerrch)
        call aptchmv (awordx, ibeg, nchar, abigtag(nw2), 11, nerrch)

        abigtag(nw3) = 'Error ? - ? ^ ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nw3), 7, nerrch)
        call aptchmv (abig(nb1), 1, 1, abigtag(nw3), 11, nerrch)
        call aptchia (iword(6), 10, awordx, 80, ibeg, nchar, nerrch)
        call aptchmv (awordx, ibeg, nchar, abigtag(nw3), 15, nerrch)

        call aptbrtn (iword(6), 10, idig(1,nb2), ndig(nb2),
     &                idmax, iemax,
     &                idig(1,nw2), idig(1,nw3), idig(1,nw4),
     &                idig(1,nw5), idig(1,nw6), idig(1,nw7),
     &                idig(1,nw8), idig(1,nw9), idig(1,nw10),
     &                idig(1,nw11), idig(1,nw12),
     &                idig(1,nw1), ndigm,
     &                idig(1,nb1), ndig(nb1), isign, nerrroot)

cbugc***DEBUG begins.
cbug 8764 format ('big DEBUG.  ndig(nw2) =',i5)
cbug      write (   6, 8764) ndig(nw2)
cbug      write (iout, 8764) ndig(nw2)
cbugc***DEBUG ends.

c....   Display big integers BIGNAME, nw2 and nw3.

        if (nerrroot .eq. 0) then

cbugc***DEBUG begins.
cbug      write (   6, 8764) ndig(nw2)
cbug      write (iout, 8764) ndig(nw2)
cbugc***DEBUG ends.

          ndig(nw2) = idig(1,nw4)
          if ((ndig(nw2) .gt. 1) .or. (idig(1, nw2) .gt. 0)) then
            amount = 'maximum'
            call bigout (nw2, nw2, amount)
          endif

          ndig(nw3) = idig(2,nw4)
          if ((ndig(nw3) .gt. 1) .or. (idig(1, nw3) .gt. 0)) then
            amount = 'maximum'
            call bigout (nw3, nw3, amount)
          endif

          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nw1, nerrzero)
        call bigzero (nw2, nerrzero)
        call bigzero (nw3, nerrzero)
        call bigzero (nw4, nerrzero)
        call bigzero (nw5, nerrzero)
        call bigzero (nw6, nerrzero)
        call bigzero (nw7, nerrzero)
        call bigzero (nw8, nerrzero)
        call bigzero (nw9, nerrzero)
        call bigzero (nw10, nerrzero)
        call bigzero (nw11, nerrzero)
        call bigzero (nw12, nerrzero)

cbugc***DEBUG begins.
cbug 8470   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
cbug     &    '  Big digit "|INT1 - BIGNAME^NROOT|" is negative.' )
cbug
cbug        if (isign .eq. -1) then
cbug          write (   6, 8470) ainput, ninput
cbug          write (iout, 8470) ainput, ninput
cbug        endif
cbugc***DEBUG ends.

 8468   format ('  bigint WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Root can not be found accurately by aptbrtn.' )

        if (nerrroot .ne. 0) then
          write (   6, 8468) ainput, ninput
          write (iout, 8468) ainput, ninput
        endif

        go to 999

      endif                    ! Input line:  big BIGNAME = INT1 root NROOT

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 fact

c.... Find big integer BIGNAME = big integer INT1 factorial.

      if ((nword .eq. 5) .and. (aword(3) .eq. '=') .and.
     &    (aword(5) .eq. 'fact')) then

c....   Find and display big integer BIGNAME.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        if ((ndig(nb1) .gt. 1) .or.
     &      ((ndig(nb1) .eq. 1) .and. (idig(1,nb1) .ne. 0))) then
          write (   6, 8210) ainput, ninput, abig(nb1)
          write (iout, 8210) ainput, ninput, abig(nb1)
        endif                         ! Copying over existing big integer.

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 factorial'
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)

c....   Find and display big integer INT1.  Convert to big integer if not.

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(4)
          write (iout, 8460) ainput, ninput, aword(4)
        endif

        nbwork2 = 0
        if (nb2 .eq. 0) then
          call bigtran (4, nb2, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork2 = nb2
          abigtag(nb2) = 'INT1'
        endif

        call bigout (nb2, nb2, amount)

c....   Find working arrays to store results in.

        call bigwork (nw2)           ! Temporary array for BIGNAME.
        if (nw2 .eq. 0) go to 999
        call bigwork (nw3)                             
        if (nw3 .eq. 0) go to 999
        call bigwork (nw4)                             
        if (nw4 .eq. 0) go to 999
        call bigwork (nw5)                             
        if (nw5 .eq. 0) go to 999

c....   Find abig(nb1), big integer INT1 factorial.

        abigtag(nb1) = '? factorial'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)

        call aptbfac (10, idig(1,nb2), ndig(nb2),
     &                    idig(1,nw2), idig(1,nw3),
     &                    idig(1,nw4), idig(1,nw5), ndigm,
     &                    idig(1,nb1), ndig(nb1), nerrfac)

c....   Display big integer BIGNAME.

        if (nerrfac .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nw2, nerrzero)
        call bigzero (nw3, nerrzero)
        call bigzero (nw4, nerrzero)
        call bigzero (nw5, nerrzero)

        if (nerrfac .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                           ! Input line:  big BIGNAME = INT1 fact

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 perm INT2

c.... Find BIGNAME = the number of permutations of INT1 things
c....   taken INT2 at a time.

      if ((nword .eq. 6) .and. (aword(3) .eq. '=') .and.
     &    (aword(5) .eq. 'perm')) then

c....   Find and display big integers BIGNAME, INT1 and INT2.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        if ((ndig(nb1) .gt. 1) .or.
     &      ((ndig(nb1) .eq. 1) .and. (idig(1,nb1) .ne. 0))) then
          write (   6, 8210) ainput, ninput, abig(nb1)
          write (iout, 8210) ainput, ninput, abig(nb1)
        endif                         ! Copying over existing big integer.

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 perm INT2'
        endif

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(4)
          write (iout, 8460) ainput, ninput, aword(4)
        endif

        nbwork2 = 0
        if (nb2 .eq. 0) then
          call bigtran (4, nb2, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork2 = nb2
          abigtag(nb2) = 'INT1'
        endif

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(6)
          write (iout, 8460) ainput, ninput, aword(6)
        endif

        nbwork3 = 0
        if (nb3 .eq. 0) then
          call bigtran (6, nb3, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork3 = nb3
          abigtag(nb3) = 'INT2'
        endif

        abigtag(nb2) = 'INT1'
        abigtag(nb3) = 'INT2'

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)

c....   Find working arrays to store results in.

        call bigwork (nw2)
        if (nw2 .eq. 0) go to 999
        call bigwork (nw3)
        if (nw3 .eq. 0) go to 999
        call bigwork (nw4)
        if (nw4 .eq. 0) go to 999
        call bigwork (nw5)
        if (nw5 .eq. 0) go to 999
        call bigwork (nw6)
        if (nw6 .eq. 0) go to 999
        call bigwork (nw7)
        if (nw7 .eq. 0) go to 999
        call bigwork (nw8)
        if (nw8 .eq. 0) go to 999
        call bigwork (nw9)
        if (nw9 .eq. 0) go to 999
        call bigwork (nw10)
        if (nw10 .eq. 0) go to 999
        call bigwork (nw11)
        if (nw11 .eq. 0) go to 999
        call bigwork (nw12)
        if (nw12 .eq. 0) go to 999
        call bigwork (nw13)
        if (nw13 .eq. 0) go to 999

c....   Find abig(nb1), permutations of INT1 things taken INT2 at a time.

        abigtag(nb1) = '? perm ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 8, nerrch)

        call aptbpac (10,
     &                idig(1,nb2), ndig(nb2), idig(1,nb3), ndig(nb3),
     &                idig(1,nw2), idig(1,nw3), idig(1,nw4),
     &                idig(1,nw5), idig(1,nw6), idig(1,nw7),
     &                idig(1,nw8), idig(1,nw9), idig(1,nw10),
     &                idig(1,nw11), idig(1,nw12),  ndigm,
     &                idig(1,nb1), ndig(nb1), idig(1,nw13), ndig(nw13),
     &                nerrperm)

c....   Display big integer BIGNAME.

        if (nerrperm .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nw2, nerrzero)
        call bigzero (nw3, nerrzero)
        call bigzero (nw4, nerrzero)
        call bigzero (nw5, nerrzero)
        call bigzero (nw6, nerrzero)
        call bigzero (nw7, nerrzero)
        call bigzero (nw8, nerrzero)
        call bigzero (nw9, nerrzero)
        call bigzero (nw10, nerrzero)
        call bigzero (nw11, nerrzero)
        call bigzero (nw12, nerrzero)
        call bigzero (nw13, nerrzero)

 8410   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  INT2 exceeds INT1.')

        if (nerrperm .eq. 5) then
          write (   6, 8410) ainput, ninput
          write (iout, 8410) ainput, ninput
        elseif (nerrperm .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                         ! Input line:  big BIGNAME = INT1 perm INT2

c=======================================================================********

c.... Input line:  big BIGNAME = INT1 comb INT2

c.... Find BIGNAME = the number of combinations of INT1 things
c....   taken INT2 at a time.

      if ((nword .eq. 6) .and. (aword(3) .eq. '=') .and.
     &    (aword(5) .eq. 'comb')) then

c....   Find and display big integers BIGNAME, INT1 and INT2.

        call bigfind (2, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        if ((ndig(nb1) .gt. 1) .or.
     &      ((ndig(nb1) .eq. 1) .and. (idig(1,nb1) .ne. 0))) then
          write (   6, 8210) ainput, ninput, abig(nb1)
          write (iout, 8210) ainput, ninput, abig(nb1)
        endif                         ! Copying over existing big integer.

        if (abigtag(nb1) .eq. ' ') then
          abigtag(nb1) = 'INT1 comb INT2'
        endif

        call bigfind (4, nb2)

        if (nb2 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(4)
          write (iout, 8460) ainput, ninput, aword(4)
        endif

        nbwork2 = 0
        if (nb2 .eq. 0) then
          call bigtran (4, nb2, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork2 = nb2
          abigtag(nb2) = 'INT1'
        endif

        call bigfind (6, nb3)

        if (nb3 .eq. 0) then
          write (   6, 8460) ainput, ninput, aword(6)
          write (iout, 8460) ainput, ninput, aword(6)
        endif

        nbwork3 = 0
        if (nb3 .eq. 0) then
          call bigtran (6, nb3, nerrtran)
          if (nerrtran .ne. 0) go to 999
          nbwork3 = nb3
          abigtag(nb3) = 'INT2'
        endif

        amount = 'minimum'
        call bigout (nb1, nb1, amount)
        call bigout (nb2, nb2, amount)
        call bigout (nb3, nb3, amount)

c....   Find working arrays to store results in.

        call bigwork (nw2)
        if (nw2 .eq. 0) go to 999
        call bigwork (nw3)
        if (nw3 .eq. 0) go to 999
        call bigwork (nw4)
        if (nw4 .eq. 0) go to 999
        call bigwork (nw5)
        if (nw5 .eq. 0) go to 999
        call bigwork (nw6)
        if (nw6 .eq. 0) go to 999
        call bigwork (nw7)
        if (nw7 .eq. 0) go to 999
        call bigwork (nw8)
        if (nw8 .eq. 0) go to 999
        call bigwork (nw9)
        if (nw9 .eq. 0) go to 999
        call bigwork (nw10)
        if (nw10 .eq. 0) go to 999
        call bigwork (nw11)
        if (nw11 .eq. 0) go to 999
        call bigwork (nw12)
        if (nw12 .eq. 0) go to 999
        call bigwork (nw13)
        if (nw13 .eq. 0) go to 999

c....   Find abig(nb1), combinations of INT1 things taken INT2 at a time.

        abigtag(nb1) = '? comb ?'
        call aptchmv (abig(nb2), 1, 1, abigtag(nb1), 1, nerrch)
        call aptchmv (abig(nb3), 1, 1, abigtag(nb1), 8, nerrch)

        call aptbpac (10,
     &                idig(1,nb2), ndig(nb2),   idig(1,nb3), ndig(nb3),
     &                idig(1,nw2), idig(1,nw3), idig(1,nw4),
     &                idig(1,nw5), idig(1,nw6), idig(1,nw7),
     &                idig(1,nw8), idig(1,nw9), idig(1,nw10),
     &                idig(1,nw11), idig(1,nw12), ndigm,
     &                idig(1,nw13), ndig(nw13), idig(1,nb1), ndig(nb1),
     &                nerrcomb)

c....   Display big integer BIGNAME.

        if (nerrcomb .eq. 0) then
          asign(nb1) = ' '
          abigs      = abig(nb1)
          amount     = 'maximum'
          call bigout (nb1, nb1, amount)
        endif

c....   Zero out working arrays.

        call bigzero (nbwork2, nerrzero)
        call bigzero (nbwork3, nerrzero)
        call bigzero (nw2, nerrzero)
        call bigzero (nw3, nerrzero)
        call bigzero (nw4, nerrzero)
        call bigzero (nw5, nerrzero)
        call bigzero (nw6, nerrzero)
        call bigzero (nw7, nerrzero)
        call bigzero (nw8, nerrzero)
        call bigzero (nw9, nerrzero)
        call bigzero (nw10, nerrzero)
        call bigzero (nw11, nerrzero)
        call bigzero (nw12, nerrzero)
        call bigzero (nw13, nerrzero)

        if (nerrcomb .eq. 5) then
          write (   6, 8410) ainput, ninput
          write (iout, 8410) ainput, ninput
        elseif (nerrcomb .ne. 0) then
          write (   6, 8260) ainput, ninput
          write (iout, 8260) ainput, ninput
        endif

        go to 999

      endif                         ! Input line:  big BIGNAME = INT1 comb INT2

c=======================================================================********

c.... Test for optional integer index.  Use 1 if index not specified.

      ibignew = 0

      if (aword(3) .eq. '=') then
        nword = nword + 1
        ibignew = 1
        do n = nword, 4, -1
          aword(n) = aword(n-1)
          lword(n) = lword(n-1)
          iword(n) = iword(n-1)
          fword(n) = fword(n-1)
          mtype(n) = mtype(n-1)
        enddo
        aword(3) = '1'
        lword(3) = 1
        iword(3) = 1
        fword(3) = 1.0
        mtype(3) = 2

      endif                           ! Added integer index 1.

c=======================================================================********

c.... Input line:  big BIGNAME N = INT1 INT2 INT3 ...

c.... Specify or redefine big integer BIGNAME.

      if ((nword .ge. 5) .and. (aword(4) .eq. '=') .and.
     &    (mtype(3) .eq. 2)) then

cbugc***DEBUG begins.
cbug 8771 format ( / 'bigint DEBUG.  nbug =',i5 /
cbug     &  a / a / a / a / a / a / a /
cbug     &  a / a / a / a / a / a / a / a / a )
cbug      nbug = 1
cbug      write (   6, 8771) nbug, (aword(n), n = 1, nword)
cbug      write (iout, 8771) nbug, (aword(n), n = 1, nword)
cbugc***DEBUG ends.

        call bigfind (2, nb)

        if (nb .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(2)
          write (iout, 8450) ainput, ninput, aword(2)
          go to 999
        endif

        if ((ndig(nb) .gt. 1) .or.
     &      ((ndig(nb) .eq. 1) .and. (idig(1,nb) .ne. 0))) then
          write (   6, 8210) ainput, ninput, abig(nb)
          write (iout, 8210) ainput, ninput, abig(nb)
          amount = 'minimum'
          call bigout (nb, nb, amount)
        endif                         ! Copying over existing big integer.

        abigtag(nb) = 'BIGNAME N = INT1 INT2 INT3 ...'

cbugc***DEBUG begins.
cbug 8772   format ('bigint DEBUG.  nbug =',i5,'  nb =',i3,'  abig = ',a,
cbug     &    '  abigtag =', a )
cbug        nbug = 2
cbug        write (   6, 8772) nbug, nb, abig(nb), abigtag(nb)
cbug        write (iout, 8772) nbug, nb, abig(nb), abigtag(nb)
cbugc***DEBUG ends.

c....   See if big integer is being completely redefined.

        if (ibignew .eq. 1) then
          call bigzero (nb, nerrzero)
          abigtag(nb) = 'BIGNAME 1 = INT1 INT2 INT3 ...'
        endif

        call varfind (3, nword)
cbugc***DEBUG begins.
cbug        abigtag(nb) = 'BIGNAME N = INT1 INT2 INT3 ...'
cbug 8773 format ('bigint DEBUG.  nbug =',i5 /
cbug     &  '  aword(3) = "',a64,'"' /
cbug     &  '  iword(3) =  ',i5 /
cbug     &  '  fword(3) =  ',1pe22.14 /
cbug     &  '  mtype(3) =  ',i5 )
cbug      nbug = 3
cbug      write (   6, 8773) nbug, aword(3), iword(3), fword(3), mtype(3)
cbug      write (iout, 8773) nbug, aword(3), iword(3), fword(3), mtype(3)
cbugc***DEBUG ends.

 8250   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is not a positive integer:  ',a )
 8252   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is negative:  ',a )
 8253   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word must be an integer:  ',a )
 8254   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is not non-negative integer:  ',a )
 8257   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Index N is not positive:  ',a)
 8259   format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Index N exceeds memory space',i6,':  ',a)

        if (mtype(3) .ne. 2) then
          write (   6, 8250) ainput, ninput, aword(3)
          write (iout, 8250) ainput, ninput, aword(3)
          go to 999
        endif

        if (iword(3) .lt. 1) then
          write (   6, 8257) ainput, ninput, aword(3)
          write (iout, 8257) ainput, ninput, aword(3)
          go to 999
        endif

        if (iword(3) .gt. ndigm) then
          write (   6, 8259) ainput, ninput, ndigm, aword(3)
          write (iout, 8259) ainput, ninput, ndigm, aword(3)
          go to 999
        endif

c....   Loop over input line words that replace big integer digits.

        next = iword(3)               ! First digit replaced.
        do n = 5, nword               ! Loop over input words.
cbugc***DEBUG begins.
cbug 8774 format ('bigint DEBUG.  nbug =',i5 /
cbug     &  '  aword(5) = "',a64,'"' /
cbug     &  '  iword(5) =  ',i5 /
cbug     &  '  fword(5) =  ',1pe22.14 /
cbug     &  '  mtype(5) =  ',i5 )
cbug      nbug = 4
cbug      write (   6, 8774) nbug, aword(5), iword(5), fword(5), mtype(5)
cbug      write (iout, 8774) nbug, aword(5), iword(5), fword(5), mtype(5)
cbugc***DEBUG ends.

c....     Translate ASCII word into single digits, one per word.

c....     See if there is room for new digits.

          if ((next + lword(n) - 1) .gt. ndigm) then
            write (   6, 8260) ainput, ninput
            write (iout, 8260) ainput, ninput
            call bigzero (nb, nerrzero)
            go to 999
          endif

c....     Interpret ASCII words.

          call aptatob (aword(n), 1, lword(n), idig(next,nb), ndigm,
     &                  nerra)

          if (nerra .ne. 0) then      ! Can not interpret ASCII word.

c....       Try converting an integer variable to an ASCII word.

            if (mtype(n) .eq. 2) then
              awordx = ' '
              call aptchia (iword(n), 10, awordx, 80,
     &                      ibeg, nchar, nerra)
              awordy = ' '
              call aptchmv (awordx, ibeg, nchar, awordy, 1, nerra)
              aword(n) = awordy
              lword(n) = nchar

c....         See if there is room for new digits.

              if ((next + lword(n) - 1) .gt. ndigm) then
                write (   6, 8260) ainput, ninput
                write (iout, 8260) ainput, ninput
                call bigzero (nb, nerrzero)
                go to 999
              endif

              call aptatob (aword(n), 1, lword(n), idig(next,nb), ndigm,
     &                      nerra)
            endif                     ! Tried to interpret integer variable.

            if (nerra .ne. 0) then    ! Can not interpret integer variable.
              write (   6, 8254) ainput, ninput, aword(n)
              write (iout, 8254) ainput, ninput, aword(n)
              call bigzero (nb, nerrzero)
              go to 999
            endif                     ! Failed to interpret integer variable.
          endif                       ! Failed to interpret ASCII word.

          next = next + lword(n)

        enddo                         ! End of loop over input words.

c....   Finished interpreting input line words.

c....   Set length of big integer.

        ndig(nb) = max (next - 1, ndig(nb))
cbugc***DEBUG begins.
cbug        abigtag(nb) = 'after input'
cbug        amount = 'minimum'
cbug        call bigout (nb, nb, amount)
cbugc***DEBUG ends.

c....   Remove any leading zeros.

cbugc***DEBUG begins.
cbug        abigtag(nb) = 'no leading zero'
cbug        amount = 'minimum'
cbug        call bigout (nb, nb, amount)
cbugc***DEBUG ends.
        nbeg = ndig(nb)
        do n = ndig(nb), 1, -1
          if (idig(n,nb) .gt. 0) nbeg = n
        enddo
cbugc***DEBUG begins.
cbug        call bigout (nb, nb, amount)
cbugc***DEBUG ends.
        ndig(nb) = ndig(nb) - nbeg + 1
        do n = 1, ndig(nb)
          idig(n,nb) = idig(n+nbeg-1,nb)
        enddo
cbugc***DEBUG begins.
cbug        abigtag(nb) = 'no leading zero'
cbug        amount = 'minimum'
cbug        call bigout (nb, nb, amount)
cbugc***DEBUG ends.
        do n = ndig(nb) + 1, ndigm
          idig(n,nb) = 0
        enddo
cbugc***DEBUG begins.
cbug        abigtag(nb) = 'added zeros to end'
cbug        amount = 'minimum'
cbug        call bigout (nb, nb, amount)
cbugc***DEBUG ends.

        if ((ndig(nb) .eq. 1) .and. (idig(1,nb) .eq. 0)) then
          abigtag(nb) = ' '
        endif

        asign(nb1) = ' '
        abigs      = abig(nb)
        amount = 'maximum'
        call bigout (nb, nb, amount)

        go to 999

      endif                   ! Input line:  big BIGNAME N = INT1 INT2 INT3 ...

c=======================================================================********

 8910 format ('  bigint ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8910) ainput, ninput
      write (iout, 8910) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine bigint.       (+1 line)
      end
