      subroutine bigtran (nw, nb, nerrtran)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  bigtran  Updated     2006 April 11 16:20.
c               bigtran  Originated  2005 August 26 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To translate an input line integer or integer variable
c               (word nw) into a big integer, abig(nb).
c
c     Logic:    Entry points:  bigtran.
c bigtran       Called by:  bigint.
c
c bigtran       Calls:  aptatob, aptchia, aptchmv, bigout, bigwork, bigzero,
c               varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     nb        Output   Index of big integer.
c
c     nerrtran  Output   Error indicator:
c                          0 if no error.
c                          1 if nw < 1.
c                          2 if nw > nwordm.
c                          3 if no unused big integer is available.
c                          4 if no big integer can be stored.
c
c     nw        Input    Index of input line word.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8  amount             ! Amount of output from bigout.

      character*80 awordx             ! Temporary command word.
      character*80 awordy             ! Temporary command word.

c.... Initialize.

      nerrtran = 0

      if (nw .lt. 1) then
        nerrtran = 1
        go to 999
      endif

      if (nw .gt. nwordm) then
        nerrtran = 2
        go to 999
      endif

c.... Find an unused big integer.

      call bigwork (nb)
      if (nb .eq. 0) then
        nerrtran = 3
        go to 999
      endif

c.... Translate an ASCII word into a big integer.

      call aptatob (aword(nw), 1, lword(nw), idig(1,nb), ndigm, nerra)

      if (nerra .ne. 0) then          ! Can not interpret ASCII word.

c....   Try converting an integer variable to an ASCII word.

c....   See if the variable exists.

        call varfind (nw, nw)

        if (mtype(nw) .eq. 2) then
          awordx = ' '
          call aptchia (iword(nw), 10, awordx, 80,
     &                  ibeg, nchar, nerra)
          awordy = ' '
          call aptchmv (awordx, ibeg, nchar, awordy, 1, nerra)
          aword(nw) = awordy
          lword(nw) = nchar

          call aptatob (aword(nw), 1, lword(nw), idig(1,nb), ndigm,
     &                  nerra)
        endif                         ! Tried to interpret integer variable.

 8250   format ('  big ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is not a non-negative integer:  ',a )

        if (nerra .ne. 0) then        ! Can not interpret integer variable.
          nerrtran = 4
          write (   6, 8250) ainput, ninput, aword(nw)
          write (iout, 8250) ainput, ninput, aword(nw)
          call bigzero (nb, nerrzero)
          go to 999
        endif                         ! Failed to interpret integer variable.
      endif                           ! Failed to interpret ASCII word.

c.... Finished interpreting input line word.

      ndig(nb) = lword(nw)

c.... Remove any leading zeros.

      nbeg = ndig(nb)
      do n = ndig(nb), 1, -1
        if (idig(n,nb) .gt. 0) nbeg = n
      enddo
      ndig(nb) = ndig(nb) - nbeg + 1
      do n = 1, ndig(nb)
        idig(n,nb) = idig(n+nbeg-1,nb)
      enddo
      do n = ndig(nb) + 1, ndigm
        idig(n,nb) = 0
      enddo

      if ((ndig(nb) .eq. 1) .and. (idig(1,nb) .eq. 0)) then
        abigtag(nb) = ' '
      else
        amount = 'minimum'
        call bigout (nb, nb, amount)
      endif

      aword(nw) = abig(nb)
      lword(nw) = 1
      mtype(nw) = 1

  999 return

c.... End of subroutine bigtran.      (+1 line)
      end
