      subroutine brout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  brout    Updated     2004 November 23 13:30.
c               brout    Originated  1994 February 7 15:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the data for bricks n1 to n2.
c
c     Logic:    Entry points:  brout.
c brout         Called by:  brick, brstore, cluster, copy, delete, distance,
c               mcvol, point, renamea, renamex, search, where.
c
c brout         Calls:  aptbrkv, coord.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8  amount             ! Amount of output to write.
      character*8  asysx              ! Initial coordinate system.
      character*8  angunx             ! Initial angle unit.
      character*24 awordx             ! Initial aword(2).

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. nbricks) nn2 = nbricks

      asysx  = asys
      angunx = angunit
      nwordx = nword
      awordx = aword(2)

      do 180 n = nn1, nn2

        if (abrick(n) .eq. ' ') go to 180

 8100   format ('brick  ',a,2x,'coord',2x,a,2x,a)

        write (   6, 8100) abrick(n), asysbr(n), angbr(n)
        write (iout, 8100) abrick(n), asysbr(n), angbr(n)

c....  Switch to the coordinate system of the brick, if not the current system.

        if ((asysbr(n) .ne. asys)     .or.
     &      (n         .eq. nn1)      .or.
     &     ((asysbr(n) .ne. 'rect')   .and.
     &      (angbr(n)  .ne. angunit))) then
          angunit  = angbr(n)
          nword    = 2
          aword(2) = asysbr(n)
          call coord
        endif

        angunit = angbr(n)

        if (asysbr(n) .eq. 'rect') then
          ksys = 0
        elseif (asysbr(n) .eq. 'cyl') then
          ksys = 1
        elseif (asysbr(n) .eq. 'sph') then
          ksys = 2
        endif

 8120   format ('  brout ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Brick has unrecognized angle units:  ',a)

        if (angbr(n) .eq. 'degrees') then
          iunit = 0
        elseif (angbr(n) .eq. 'radians') then
          iunit = 1
        else
          write (   6, 8120) ainput, ninput, angbr(n)
          write (iout, 8120) ainput, ninput, angbr(n)
        endif
 
        call aptbrkv (ksys, iunit, ubr(n), dubr(n), vbr(n), dvbr(n),
     &                wbr(n), dwbr(n), tol,
     &                umin, vmin, wmin, ucen, vcen, wcen,
     &                umax, vmax, wmax,
     &                du11, dv11, dw11, du12, dv12, dw12,
     &                ducn, dvcn, dwcn, du21, dv21, dw21,
     &                du22, dv22, dw22,
     &                aumin, avmin, awmin, aucen, avcen, awcen,
     &                aumax, avmax, awmax, volbr(n), nerra)

c....   Display the limiting coordinate system values.

        du = abs (dubr(n))
        dv = abs (dvbr(n))
        dw = abs (dwbr(n))

 8150   format ('   u,  v,  w:  min',1p3e20.12 /
     &          '  du, dv, dw      ',1p3e20.12 /
     &          '   u,  v,  w:  max',1p3e20.12 )

        write (   6, 8150) umin, vmin, wmin, du, dv, dw,
     &                     umax, vmax, wmax
        write (iout, 8150) umin, vmin, wmin, du, dv, dw,
     &                     umax, vmax, wmax
    
c....   Display the centroid in the brick.
    
 8172   format ('   u,  v,  w:  cen',1p3e20.12 )

        write (   6, 8172) ucen, vcen, wcen
        write (iout, 8172) ucen, vcen, wcen

c....   Display the volume of the brick.

 8173   format ('  Brick volume    ',1pe20.12 )

        write (   6, 8173) volbr(n)
        write (iout, 8173) volbr(n)

        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

c....     Display the edge lengths.

 8174     format ('  edges           ',1p3e20.12 )
 8175     format ('  edges:  min, min',1p3e20.12 /
     &            '          min, max',1p3e20.12 /
     &            '          max, min',1p3e20.12 /
     &            '          max, max',1p3e20.12 /
     &            '          centroid',1p3e20.12 )

          if (asysbr(n) .eq. 'rect') then
            write (   6, 8174) ducn, dvcn, dwcn
            write (iout, 8174) ducn, dvcn, dwcn
          else
            write (   6, 8175) du11, dv11, dw11,
     &                         du12, dv12, dw12,
     &                         du21, dv21, dw21,
     &                         du22, dv22, dw22,
     &                         ducn, dvcn, dwcn
            write (iout, 8175) du11, dv11, dw11,
     &                         du12, dv12, dw12,
     &                         du21, dv21, dw21,
     &                         du22, dv22, dw22,
     &                         ducn, dvcn, dwcn
          endif

 8178     format ('  areas           ',1p3e20.12 )
 8180     format ('  areas:  min     ',1p3e20.12 /
     &            '          max     ',1p3e20.12 /
     &            '          cen     ',1p3e20.12 )

c....     Display the face areas.

          if (asysbr(n) .eq. 'rect') then
            write (   6, 8178) aucen, avcen, awcen
            write (iout, 8178) aucen, avcen, awcen
          else
            write (   6, 8180) aumin, avmin, awmin, aumax, avmax, awmax,
     &                         aucen, avcen, awcen
            write (iout, 8180) aumin, avmin, awmin, aumax, avmax, awmax,
     &                         aucen, avcen, awcen
          endif

        endif                         ! Tested amount.

  180 continue

c.... Switch back to the original coordinate system, if different.

      if ((asysx  .ne. asys)      .or.
     &   ((asysx  .ne. 'rect')    .and.
     &    (angunx .ne. angunit))) then
        angunit  = angunx
        nword    = 2
        aword(2) = asysx
        call coord
      endif

      angunit  = angunx
      nword    = nwordx
      aword(2) = awordx

      return

c.... End of subroutine brout.        (+1 line)
      end
