      subroutine cluster

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  cluster  Updated     2007 January 23 14:00.
c               cluster  Originated  1994 March 7 14:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create clusters of points, or add points to,
c               or delete points from, existing clusters.
c
c     Logic:    Entry points:  cluster
c cluster       Called by:  helpall, main.
c
c cluster       Calls:  aptsubr, brout,  clout, cloutcl, compsubs, incrsubs,
c               meshchk, namechk, ptbrick, ptfind, pttet, ptzone, tetout, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8  amount             ! Amount of output to write in clout.
      character*24 apointmn           ! Lowest point name to test.
      character*24 apointmx           ! Highest point name to test.
      character*24 aclustmn           ! Lowest cluster name to test.
      character*24 aclustmx           ! Highest cluster name to test.
      character*1  afirst             ! First character of a cluster name.

c.... Input line:  cluster [help]

 8110 format (
     &  '  Command "cluster":  acceptable input lines include:' /
     &  '  "cluster [help]"  Display this message.' /
     &  '  "cluster all"     Display all clusters.' /
     &  '  "cluster list [CLNAME1 CLNAME2 CLNAME3 ...]"' /
     &  '     Display all listed clusters (default is all',
     &  ' clusters).' /
     &  '  "cluster CLNAME"  Display cluster CLNAME.' //
     &  '  "cluster CLNAME all"' /
     &  '     Create new cluster CLNAME, consisting of all points.' //
     &  '  "cluster CLNAME = PNAME1 ... PNAME2 [thru] PNAME3 ..."' /
     &  '     Create new cluster CLNAME, consisting of the points' /
     &  '     PNAME1, ...,  PNAME2 [, through] PNAME3, ...' //
     &  '  "cluster CLNAME & PNAME1 ... PNAME2 [thru] PNAME3 ..."' /
     &  '     Change cluster CLNAME, to add the points' /
     &  '     PNAME1, ... PNAME2 [, through] PNAME3, ...' )

 8112 format (/
     &  '  "cluster CLNAME - PNAME1 ... PNAME2 [thru] PNAME3 ..."' /
     &  '     Change cluster CLNAME, to remove the points' /
     &  '     PNAME1, ..., PNAME2 [, through] PNAME3, ...' //
     &  '  "cluster CLNAME cluster CLNAME1 ... CLNAME2 [thru]',
     &  ' CLNAME3 ..."' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     in existing clusters CLNAME1, ..., CLNAME2 [, through]',
     &  ' CLNAME3, ...' //
     &  '  "cluster CLNAME cluster - CLNAME1 ... CLNAME2 [thru]',
     &  ' CLNAME3 ..."' /
     &  '     Change cluster CLNAME, to remove all points' /
     &  '     in existing clusters CLNAME1, ..., CLNAME2 [, through]',
     &  ' CLNAME3, ...' //
     &  '  "cluster CLNAME cluster comp. CLNAME1"' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     not in cluster CLNAME1.' /
     &  '  "cluster CLNAME cl CLNAME1 .not.  CLNAME2"' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     in cluster CLNAME1 and not in cluster CLNAME2.' //
     &  '  Type <return> for more help.' )

 8113 format (a)

 8114 format (
     &  '  "cluster CLNAME cl CLNAME1 .and.  CLNAME2"' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     in both cluster CLNAME1 and cluster CLNAME2.' /
     &  '  "cluster CLNAME cl CLNAME1 .nand. CLNAME2"' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     not in both cluster CLNAME1 and cluster CLNAME2.' /
     &  '  "cluster CLNAME cl CLNAME1 .or.   CLNAME2"' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     in either cluster CLNAME1 or cluster CLNAME2.' /
     &  '  "cluster CLNAME cl CLNAME1 .nor.  CLNAME2"' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     in neither cluster CLNAME1 nor cluster CLNAME2.' /
     &  '  "cluster CLNAME cl CLNAME1 .xor.  CLNAME2"' /
     &  '     Create new cluster CLNAME, consisting of all points' /
     &  '     in either cluster CLNAME1 or cluster CLNAME2,',
     &  ' but not both.' /
     &  '  "cluster CLNAME cl CLNAME1 .xnor. CLNAME2"' /
     &  '     Create new cluster CLNAME, consisting of all points in' /
     &  '     both or neither cluster CLNAME1 and cluster CLNAME2.' //
     &  '  "cluster CLNAME brick BRNAME"' /
     &  '     Create new cluster CLNAME, consisting of all points',
     &  ' in brick BRNAME.' )

 8116 format (
     &  '  "cluster CLNAME tetrahedron TETNAME"' /
     &  '     Create new cluster CLNAME, consisting of all points',
     &  ' in tetrahedron TETNAME.' /
     &  '  "cluster CLNAME zone ZNAME"' /
     &  '     Create new cluster CLNAME, consisting of all points',
     &  ' in zone ZNAME.' //
     &  '  "cluster CLNAME mesh [all]"' /
     &  '     Create new cluster CLNAME, consisting of all mesh',
     &  ' points.' /
     &  '  "cluster CLNAME nomesh"' /
     &  '     Create new cluster CLNAME, consisting of all non-mesh',
     &  ' points.' /
     &  '  "cluster CLNAME mesh block"' /
     &  '     Create new cluster CLNAME, consisting of all mesh',
     &  ' points' /
     &  '     in the mesh block.' /
     &  '  "cluster CLNAME mesh vacuum"' /
     &  '     Create new cluster CLNAME, consisting of all mesh',
     &  ' points' /
     &  '     with x, y, z coordinates of -1.e99 or less.' //
     &  '   Note:  use +/- for CLNAME to incr/decr the last cluster',
     &  ' name:  ',a //
     &  '   Synonyms:  [.and., .int.], [.or., .un.], [.xnor., .eqv.],'
     &  ' [block, bl],' /
     &  '    [brick, br], [cluster, cl], [help, h],',
     &  ' [tetrahedron, tetra, tet],' /
     &  '    [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8113) ans
        endif
        write (   6, 8114)
        write (iout, 8114)
        write (   6, 8116) aclusts
        write (iout, 8116) aclusts
        go to 999
      endif

c===============================================================================

c.... Input line:  cluster all
c.... Input line:  cluster list

 8120 format (/ 'Number of clusters = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nclustx = 0
        do 120 n = 1, nclusts
          if (aclust(n) .eq. ' ') go to 120
          nclustx = nclustx + 1
  120   continue
        if (nclustx .eq. 1) then
          amount = 'maximum'
        else
          amount = 'minimum'
        endif
        call clout (1, nclusts, amount)
        write (   6, 8120) nclustx
        write (iout, 8120) nclustx
        go to 999
      endif

c===============================================================================

c.... Input line:  cluster list CLNAME1 CLNAME2 CLNAME3 ...

 8130 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 140 nw = 3, nword
          nfind = 0
          do 130 n = 1, nclusts
            call aptsubr (aclust(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aclust(n)) .or.
     &          (aword(nw) .eq. astem))    then
              nfind = nfind + 1
              call clout (n, n, 'min')
            endif
  130     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  140   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  cluster CLNAME

c.... See if user wants one cluster displayed.

      if (nword .eq. 2) then
        do 150 n = 1, nclusts
          if (aword(2) .eq. aclust(n)) then
            amount = 'maximum'
            call clout (n, n, amount)
            go to 999
          endif
  150   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if last cluster name should be incremented and used.

 8128 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last cluster can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = aclusts
        lword(2) = laclusts
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, aclusts
          write (iout, 8128) ainput, ninput, aclusts
          go to 999
        endif
      endif

 8129 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last cluster can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = aclusts
        lword(2) = laclusts
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, aclusts
          write (iout, 8129) ainput, ninput, aclusts
          go to 999
        endif
      endif

 8905 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of cluster too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

      if (aword(2) .eq. 'comp.') then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8160 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of cluster not from 1 to 24 characters:  ',a)
 8165 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of cluster can not begin with ".":  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8160) ainput, ninput, aword(2)
        write (iout, 8160) ainput, ninput, aword(2)
        go to 999
      endif

      afirst = aword(2)
      if (afirst .eq. '.') then
        write (   6, 8165) ainput, ninput, aword(2)
        write (iout, 8165) ainput, ninput, aword(2)
        go to 999
      endif

c.... See if cluster exists.

 8170 format ('  WARNING:  replacing existing cluster:  ',a)
 8180 format ('  Adding points to existing cluster:  ',a)
 8185 format ('  Removing points from existing cluster:  ',a)

      nrep = 0
      do 170 n = 1, nclusts
        if (aword(2) .eq. aclust(n)) then
          nrep = 1
          if (aword(3) .eq. '=') then
            write (   6, 8170) aclust(n)
            write (iout, 8170) aclust(n)
          elseif (aword(3) .eq. '&') then
            write (   6, 8180) aclust(n)
            write (iout, 8180) aclust(n)
          elseif (aword(3) .eq. '-') then
            write (   6, 8185) aclust(n)
            write (iout, 8185) aclust(n)
          endif

          call clout (n, n, 'min')

          go to 190
        endif
  170 continue

c.... Find out where to store this new cluster.

      do 180 n = 1, nclusts
        if (aclust(n) .eq. ' ') go to 190
  180 continue

      nclusts = nclusts + 1

 8190 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of clusters > limit ',i5)

      if (nclusts .gt. nclustm) then
        nclusts = nclustm
        write (   6, 8190) ainput, ninput, nclustm
        write (iout, 8190) ainput, ninput, nclustm
        go to 999
      endif

      n = nclusts

  190 nerr = 0

 8200 format ('  WARNING:  assigning points to new cluster:  ',a)

      if ((nrep .eq. 0) .and. (aword(3) .eq. '&')) then
        write (   6, 8200) aword(2)
        write (iout, 8200) aword(2)
      endif

c===============================================================================

c.... Input line:  cluster CLNAME = PNAME1 ... PNAME2 [thru] PNAME3 ...
c.... Input line:  cluster CLNAME & PNAME1 ... PNAME2 [thru] PNAME3 ...

      if ((aword(3) .eq. '=')  .or.
     &    (aword(3) .eq. '&')) then

c.... Find the points to add to the cluster.

      npairx = npairs

 8210 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No points specified.')

      if (nword .eq. 3) then
        write (   6, 8210) ainput, ninput
        write (iout, 8210) ainput, ninput
        nerr = 1
      endif

 8230 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Exceeded maximum cluster pair table size:  ',i4)
 8232 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Key word "thru" in bad position at word:  ',i4)
 8233 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name > max name:  ',a24,2x,a24)

      do 230 nw = 4, nword          ! Loop over input line words.

c....   Test for option "thru".

        if (aword(nw) .eq. 'thru') then

 8234     format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word "thru" duplicated or misplaced.')

          if ((nw          .eq. 4     )  .or.
     &        (aword(nw-2) .eq. 'thru')  .or.
     &        (aword(nw-1) .eq. 'thru')  .or.
     &        (aword(nw+1) .eq. 'thru')  .or.
     &        (aword(nw+2) .eq. 'thru')  .or.
     &        (nw          .eq. nword )) then
            write (   6, 8234) ainput, ninput
            write (iout, 8234) ainput, ninput
            nerr = 1
            go to 230
          endif

          apointmn = aword(nw-1)
          apointmx = aword(nw+1)

          call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
          if (nsmall .eq. 2) then
            write (   6, 8233) ainput, ninput, apointmn, apointmx
            write (iout, 8233) ainput, ninput, apointmn, apointmx
            nerr = 1
          endif

          do 225 np = 1, npoints      ! Loop over points.
            if (apoint(np) .eq. ' ') go to 225

            call compsubs (apoint(np), apointmn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 225
            call compsubs (apoint(np), apointmx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 225

              npairs = npairs + 1

              if (npairs .gt. npairm) then
                npairs = npairm
                write (   6, 8230) ainput, ninput, npairm
                write (iout, 8230) ainput, ninput, npairm
                nerr = 1
                go to 230
              endif

              apaircl(npairs) = aword(2)
              apairpt(npairs) = apoint(np)

  225     continue                    ! End of loop over points.

          go to 230

        endif                         ! Tested for option "thru".

c....   Test for a point name.

        if (aword(nw-1) .eq. 'thru') go to 230
        if (aword(nw+1) .eq. 'thru') go to 230

        call ptfind (nw, np)

        if (np .gt. 0) then           ! Argument is a point name.

          npairs = npairs + 1

          if (npairs .gt. npairm) then
            npairs = npairm
            write (   6, 8230) ainput, ninput, npairm
            write (iout, 8230) ainput, ninput, npairm
            nerr = 1
            go to 230
          endif

          apaircl(npairs) = aword(2)
          apairpt(npairs) = aword(nw)
          go to 230

        else                          ! Not a point name or option "thru".

          nerr = 1

        endif                         ! Tested for a point name.

  230 continue                        ! End of loop over input line words.

      if (nerr .ne. 0) then
        npairs = npairx
        if (nrep .eq. 0) then
          aclust(n) = ' '
          if (n .eq. nclusts) nclusts = nclusts - 1
        endif
        go to 999
      endif

c.... Blank out old points for a redefined cluster.

      if ((nrep .eq. 1) .and. (aword(3) .eq. '=')) then
        do 240 npr = 1, npairx
          if (apaircl(npr) .eq. aword(2)) then
            apaircl(npr) = ' '
            apairpt(npr) = ' '
          endif
  240   continue
      endif

c.... Store and display the cluster specifications.

      aclust(n)  = aword(2)
      laclust(n) = lword(2)
      aclusts    = aword(2)
      laclusts   = lword(2)
cbugc___DEBUG begins.
cbug 9901 format ('cluster DEBUG.  Cluster ',i3,' is ',a)
cbug 9902 format ('cluster DEBUG.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9901) n, aclust(n)
cbug      write (iout, 9901) n, aclust(n)
cbug      write (   6, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

      amount = 'maximum'
      call clout (n, n, amount)

      go to 999

      endif                           ! Word 3 was "=" or "&".

c===============================================================================

c.... Input line:  cluster CLNAME - PNAME1 ... PNAME2 [thru] PNAME3 ...

      if (aword(3) .eq. '-') then

c.... Find the points to remove from the cluster.

 8410 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No points specified.')

      if (nword .eq. 3) then
        write (   6, 8410) ainput, ninput
        write (iout, 8410) ainput, ninput
        go to 999
      endif

      if (nrep .eq. 0) then
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        aclust(n) = ' '
        if (n .eq. nclusts) nclusts = nclusts - 1
        go to 999
      endif

      do 460 nw = 4, nword          ! Loop over input line words.

c....   Test for option "thru".

        if (aword(nw) .eq. 'thru') then

          if ((nw          .eq. 4     )  .or.
     &        (aword(nw-2) .eq. 'thru')  .or.
     &        (aword(nw-1) .eq. 'thru')  .or.
     &        (aword(nw+1) .eq. 'thru')  .or.
     &        (aword(nw+2) .eq. 'thru')  .or.
     &        (nw          .eq. nword )) then
            write (   6, 8234) ainput, ninput
            write (iout, 8234) ainput, ninput
            go to 999
          endif

          apointmn = aword(nw-1)
          apointmx = aword(nw+1)

          call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
          if (nsmall .eq. 2) then
            write (   6, 8233) ainput, ninput, apointmn, apointmx
            write (iout, 8233) ainput, ninput, apointmn, apointmx
            go to 999
          endif

          do 450 np = 1, npoints      ! Loop over points.
            if (apoint(np) .eq. ' ') go to 450

            call compsubs (apoint(np), apointmn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 450
            call compsubs (apoint(np), apointmx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 450

c....         Blank out specified points.

              do 440 npr = 1, npairs
                if ((apaircl(npr) .eq. aword(2))    .and.
     &              (apairpt(npr) .eq. apoint(np))) then
                  apaircl(npr) = ' '
                  apairpt(npr) = ' '
                endif
  440         continue

  450     continue                    ! End of loop over points.

          go to 460

        endif                         ! Tested for option "thru".

c....   Test for a point name.

        if (aword(nw-1) .eq. 'thru') go to 460
        if (aword(nw+1) .eq. 'thru') go to 460

        call ptfind (nw, np)

        if (np .gt. 0) then           ! Argument is a point name.

c....     Blank out specified points.

          do 420 npr = 1, npairs
            if ((apaircl(npr) .eq. aword(2))    .and.
     &          (apairpt(npr) .eq. apoint(np))) then
              apaircl(npr) = ' '
              apairpt(npr) = ' '
            endif
  420     continue

          go to 460

        else                          ! Not a point name or option "thru".

 8460     format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Not a point name or option "thru":  ',a)

          write (   6, 8460) ainput, ninput, aword(n)
          write (iout, 8460) ainput, ninput, aword(n)
          go to 460

        endif                         ! Tested for point name.

  460 continue                        ! End of loop over input line words.

c.... Display the cluster specifications.

cbugc___DEBUG begins.
cbug      write (   6, 9901) n, aclust(n)
cbug      write (iout, 9901) n, aclust(n)
cbug      write (   6, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

      amount = 'maximum'
      call clout (n, n, amount)

      go to 999

      endif                           ! Word 3 was "-".

c===============================================================================

c.... Input line:  cluster CLNAME all

      if (aword(3) .eq. 'all') then

c.... Put all points into the cluster.

      npairx = npairs

 8310 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  More than 3 words with option "all":  ',i3)

      if (nword .gt. 3) then
        write (   6, 8310) ainput, ninput, nword
        write (iout, 8310) ainput, ninput, nword
        nerr = 1
      endif

 8330 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Exceeded maximum cluster pair table size:  ',i4)

      do 330 np = 1, npoints          ! Loop over points.

        if (apoint(np) .eq. ' ') go to 330

        npairs = npairs + 1

        if (npairs .gt. npairm) then
          npairs = npairm
          write (   6, 8330) ainput, ninput, npairm
          write (iout, 8330) ainput, ninput, npairm
          nerr = 1
          go to 330
        endif

        apaircl(npairs) = aword(2)
        apairpt(npairs) = apoint(np)

  330 continue                        ! End of loop over input line words.

      if (nerr .ne. 0) then
        npairs = npairx
        if (nrep .eq. 0) then
          aclust(n) = ' '
          if (n .eq. nclusts) nclusts = nclusts - 1
        endif
        go to 999
      endif

c.... Blank out old points for a redefined cluster.

      if (nrep .eq. 1) then
        do 340 npr = 1, npairx
          if (apaircl(npr) .eq. aword(2)) then
            apaircl(npr) = ' '
            apairpt(npr) = ' '
          endif
  340   continue
      endif

c.... Store and display the cluster specifications.

      aclust(n)  = aword(2)
      laclust(n) = lword(2)
      aclusts    = aword(2)
      laclusts   = lword(2)
cbugc___DEBUG begins.
cbug      write (   6, 9901) n, aclust(n)
cbug      write (iout, 9901) n, aclust(n)
cbug      write (   6, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

      amount = 'maximum'
      call clout (n, n, amount)

      go to 999

      endif                           ! Word 3 was "all".

c===============================================================================

c.... Input line:  cluster CLNAME mesh [all]
c.... Input line:  cluster CLNAME mesh block

c.... Put all mesh points with indices in the specified range in the cluster.

      if ((aword(3) .eq. 'mesh')    .and.
     &    (aword(4) .ne. 'vacuum')) then

      if (nword .eq. 3) then
        nword = 4
        mtype(4) = 1
        aword(4) = 'all'
      endif

 8505 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words in "cluster CLNAME mesh ..." command not',
     &  ' 3 to 4:  ',i2)

      if ((nword .lt. 3)  .or.
     &    (nword .gt. 4)) then
        write (   6, 8505) ainput, ninput, nword
        write (iout, 8505) ainput, ninput, nword
        go to 999
      endif

      nerr = 0

      if (aword(4) .eq. 'all') then

        k1 = 1
        l1 = 1
        m1 = 1
        k2 = kmax
        l2 = lmax
        m2 = mmax

 8122   format (
     &    'Mesh size:   kmax      =',i6,'   lmax      =',i6,
     &    '   mmax      =',i6 /
     &    '             kmax*lmax =',i6,'   lmax*mmax =',i6,
     &    '   mmax*kmax =',i6 /
     &    '             kmax*lmax*mmax                =',i6)

          klmax  = kmax * lmax
          lmmax  = lmax * mmax
          mkmax  = mmax * kmax
          klmmax = kmax * lmax * mmax
  
          write (   6, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax
          write (iout, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax

      elseif ((aword(4) .eq. 'block') .or.
     &        (aword(4) .eq. 'bl'))   then

        k1 = kbl1
        l1 = lbl1
        m1 = mbl1
        k2 = kbl2
        l2 = lbl2
        m2 = mbl2

 8145   format (
     &    'Mesh block:  kbl1      =',i6,'   lbl1      =',i6,
     &    '   mbl1      =',i6 /
     &    '             kbl2      =',i6,'   lbl2      =',i6,
     &    '   mbl2      =',i6 )

          write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

      else                            ! Unrecognized option.

 8510   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Unrecognized option in "cluster CLNAME mesh ..." command ',
     &    a)

        write (   6, 8510) ainput, ninput, aword(4)
        write (iout, 8510) ainput, ninput, aword(4)
        nerr = 1

      endif                           ! Option "mesh all" or "mesh block".

      if (nerr .ne. 0) go to 598

      call meshchk (k1, k2, l1, l2, m1, m2, nerrchk)

      if (nerrchk .ne. 0) nerr = 1

      if (nerr .ne. 0) go to 598

c.... Put all mesh points with indices in the specified range into the cluster.

      npairx = npairs

      do 595 np = 1, npoints          ! Loop over points.
        if (apoint(np) .eq. ' ') go to 595
        if ((kpoint(np) .lt. k1) .or. (kpoint(np) .gt. k2)) go to 595
        if ((lpoint(np) .lt. l1) .or. (lpoint(np) .gt. l2)) go to 595
        if ((mpoint(np) .lt. m1) .or. (mpoint(np) .gt. m2)) go to 595

        npairs = npairs + 1

        if (npairs .gt. npairm) then
          npairs = npairm
          write (   6, 8330) ainput, ninput, npairm
          write (iout, 8330) ainput, ninput, npairm
          nerr = 1
          go to 595
        endif

        apaircl(npairs) = aword(2)
        apairpt(npairs) = apoint(np)

  595 continue                        ! End of loop over points.

  598 if (nerr .ne. 0) then
        npairs = npairx
        if (nrep .eq. 0) then
          aclust(n) = ' '
          if (n .eq. nclusts) nclusts = nclusts - 1
        endif
        go to 999
      endif

c.... Blank out old points for a redefined cluster.

      if (nrep .eq. 1) then
        do 599 npr = 1, npairx
          if (apaircl(npr) .eq. aword(2)) then
            apaircl(npr) = ' '
            apairpt(npr) = ' '
          endif
  599   continue
      endif

c.... Store and display the cluster specifications.

      aclust(n)  = aword(2)
      laclust(n) = lword(2)
      aclusts    = aword(2)
      laclusts   = lword(2)

      amount = 'maximum'
      call clout (n, n, amount)

      go to 999

      endif                           ! Options "mesh [all]" and "mesh block".

c===============================================================================

c.... Input line:  cluster CLNAME mesh vacuum

c.... Put all mesh points with vacuum coordinates into the cluster.

      if ((aword(3) .eq. 'mesh')    .and.
     &    (aword(4) .eq. 'vacuum')) then

 8360 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words in "cluster CLNAME mesh vacuum" command not',
     &  ' 4:  ',i2)

      if (nword .gt. 4) then
        write (   6, 8360) ainput, ninput, nword
        write (iout, 8360) ainput, ninput, nword
        go to 999
      endif

c.... Put all mesh points with vacuum coordinates into the cluster.

      npairx = npairs

      do 370 np = 1, npoints          ! Loop over points.

        if (apoint(np) .eq. ' ') go to 370
        if ((x(np) .le. -1.e99) .or.
     &      (y(np) .le. -1.e99) .or.
     &      (z(np) .le. -1.e99)) then

          npairs = npairs + 1
  
          if (npairs .gt. npairm) then
            npairs = npairm
            write (   6, 8330) ainput, ninput, npairm
            write (iout, 8330) ainput, ninput, npairm
            nerr = 1
            go to 370
          endif
  
          apaircl(npairs) = aword(2)
          apairpt(npairs) = apoint(np)

        endif

  370 continue                        ! End of loop over points.

  380 if (nerr .ne. 0) then
        npairs = npairx
        if (nrep .eq. 0) then
          aclust(n) = ' '
          if (n .eq. nclusts) nclusts = nclusts - 1
        endif
        go to 999
      endif

c.... Blank out old points for a redefined cluster.

      if (nrep .eq. 1) then
        do 390 npr = 1, npairx
          if (apaircl(npr) .eq. aword(2)) then
            apaircl(npr) = ' '
            apairpt(npr) = ' '
          endif
  390   continue
      endif

c.... Store and display the cluster specifications.

      aclust(n)  = aword(2)
      laclust(n) = lword(2)
      aclusts    = aword(2)
      laclusts   = lword(2)

      amount = 'maximum'
      call clout (n, n, amount)

      go to 999

      endif                           ! Input line "cluster CLNAME mesh vacuum".

c===============================================================================

c.... Input line:  cluster CLNAME nomesh

c.... Put all non-mesh points into the cluster.

      if (aword(3) .eq. 'nomesh') then

 8520   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "cluster CLNAME nomesh" command not',
     &    ' 3:  ',i2)

        if (nword .ne. 3) then
          write (   6, 8520) ainput, ninput, nword
          write (iout, 8520) ainput, ninput, nword
          go to 999
        endif

        nerr = 0

        npairx = npairs

        do 530 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 530
          if (kpoint(np) .ne. 0) go to 530
          if (lpoint(np) .ne. 0) go to 530
          if (mpoint(np) .ne. 0) go to 530

          npairs = npairs + 1

          if (npairs .gt. npairm) then
            npairs = npairm
            write (   6, 8330) ainput, ninput, npairm
            write (iout, 8330) ainput, ninput, npairm
            nerr = 1
            go to 530
          endif

          apaircl(npairs) = aword(2)
          apairpt(npairs) = apoint(np)

  530   continue                      ! End of loop over points.

  540   if (nerr .ne. 0) then
          npairs = npairx
          if (nrep .eq. 0) then
            aclust(n) = ' '
            if (n .eq. nclusts) nclusts = nclusts - 1
          endif
          go to 999
        endif

c....   Blank out old points for a redefined cluster.

        if (nrep .eq. 1) then
          do 550 npr = 1, npairx
            if (apaircl(npr) .eq. aword(2)) then
              apaircl(npr) = ' '
              apairpt(npr) = ' '
            endif
  550     continue
        endif

c....   Store and display the cluster specifications.

        aclust(n)  = aword(2)
        laclust(n) = lword(2)
        aclusts    = aword(2)
        laclusts   = lword(2)

        amount = 'maximum'
        call clout (n, n, amount)

        go to 999

      endif                           ! Input line "cluster CLNAME nomesh".

c===============================================================================

c.... Input line:  cluster CLNAME cluster comp. CLNAME1

c.... Boolean complement of a cluster.

      if ((nword .eq. 5)            .and.
     &   ((aword(3) .eq. 'cluster') .or.
     &    (aword(3) .eq. 'cl'    )) .and.
     &    (aword(4) .eq. 'comp.'))  then

c....   See if CLNAME1 exists.

        do 555 nc = 1, nclusts
          if (aclust(nc) .eq. aword(5)) then
            call clout (nc, nc, 'min')
            go to 558
          endif
  555   continue
        write (   6, 8130) ainput, ninput, aword(4)
        write (iout, 8130) ainput, ninput, aword(4)
        if (nrep .eq. 0) then
          aclust(n) = ' '
          if (n .eq. nclusts) nclusts = nclusts - 1
        endif
        go to 999

c....   For every point, see if in cluster CLNAME1.

  558   npairx = npairs               ! Original number of cluster pairs.

        do 565 np = 1, npoints        ! Loop over all points.

          do 560 npr = 1, npairx      ! Loop over all cluster-point pairs.
            if ((aword(5) .eq. apaircl(npr))    .and.
     &          (apoint(np) .eq. apairpt(npr))) go to 565
  560     continue

c....     Add point to cluster CLNAME, if pair table length not exceeded.

          npairs = npairs + 1
          if (npairs .gt. npairm) then
            npairs = npairm
            write (   6, 8230) ainput, ninput, npairm
            write (iout, 8230) ainput, ninput, npairm
            nerr = 1
            go to 570
          endif

          apaircl(npairs) = aword(2)
          apairpt(npairs) = apoint(np)

  565   continue                      ! End of loop over all points.

c....   See if maximum number of pairs exceeded.

  570   if (nerr .ne. 0) then
          npairs = npairx
          if (nrep .eq. 0) then
            aclust(n) = ' '
            if (n .eq. nclusts) nclusts = nclusts - 1
          endif
          go to 999
        endif

c....   Blank out old points for a redefined cluster.

        if (nrep .eq. 1) then
          do 575 npr = 1, npairx
            if (apaircl(npr) .eq. aword(2)) then
              apaircl(npr) = ' '
              apairpt(npr) = ' '
            endif
  575     continue
        endif

c....   Store and display the cluster specifications.

        aclust(n)  = aword(2)
        laclust(n) = lword(2)
        aclusts    = aword(2)
        laclusts   = lword(2)

        amount = 'maximum'
        call clout (n, n, amount)

        go to 999

      endif                           ! Doing Boolean complement.

c===============================================================================

c.... Input line:  cluster CLNAME cl CLNAME1 .xnor. CLNAME2
c.... Input line:  cluster CLNAME cl CLNAME1 .and.  CLNAME2
c.... Input line:  cluster CLNAME cl CLNAME1 .nor.  CLNAME2
c.... Input line:  cluster CLNAME cl CLNAME1 .not.  CLNAME2
c.... Input line:  cluster CLNAME cl CLNAME1 .or.   CLNAME2
c.... Input line:  cluster CLNAME cl CLNAME1 .xor.  CLNAME2

c.... Boolean operations on pairs of clusters.

      if ((nword .eq. 6)             .and.
     &   ((aword(3) .eq. 'cluster')  .or.
     &    (aword(3) .eq. 'cl'    ))  .and.
     &   ((aword(5) .eq. '.xnor.')   .or.
     &    (aword(5) .eq. '.eqv.' )   .or.
     &    (aword(5) .eq. '.and.' )   .or.
     &    (aword(5) .eq. '.int.' )   .or.
     &    (aword(5) .eq. '.nand.')   .or.
     &    (aword(5) .eq. '.nor.' )   .or.
     &    (aword(5) .eq. '.not.' )   .or.
     &    (aword(5) .eq. '.or.'  )   .or.
     &    (aword(5) .eq. '.un.'  )   .or.
     &    (aword(5) .eq. '.xor.' ))) then

        npairx = npairs               ! Original number of cluster pairs.

c....   See if CLNAME1 and CLNAME2 are different.

 8600   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Two cluster names are the same:  ',a)

        nerr = 0
        if (aword(4) .eq. aword(6)) then
          write (   6, 8600) ainput, ninput, aword(4)
          write (iout, 8600) ainput, ninput, aword(4)
          nerr = 1
        endif

c....   See if CLNAME1 and CLNAME2 exist.

        ncla = 0
        do 600 nc = 1, nclusts
          if (aclust(nc) .eq. aword(4)) then
            ncla = nc
            call clout (nc, nc, 'min')
            go to 601
          endif
  600   continue
        write (   6, 8130) ainput, ninput, aword(4)
        write (iout, 8130) ainput, ninput, aword(4)
        nerr = 1

  601   nclb = 0
        do 602 nc = 1, nclusts
          if (aclust(nc) .eq. aword(6)) then
            nclb = nc
            call clout (nc, nc, 'min')
            go to 603
          endif
  602   continue
        write (   6, 8130) ainput, ninput, aword(6)
        write (iout, 8130) ainput, ninput, aword(6)
        nerr = 1

  603   if (nerr .ne. 0) then
          npairs = npairx
          if (nrep .eq. 0) then
            aclust(n) = ' '
            if (n .eq. nclusts) nclusts = nclusts - 1
          endif
          go to 999
        endif

c....   For every point, see if in each cluster.

        do 616 np = 1, npoints        ! Loop over all points.

          nina = 0
          do 610 npr = 1, npairx      ! Loop over all cluster-point pairs.
            if ((aword(4) .eq. apaircl(npr))    .and.
     &          (apoint(np) .eq. apairpt(npr))) then
              nina = 1
              go to 611
            endif
  610     continue

  611     ninb = 0
          do 612 npr = 1, npairx      ! Loop over all cluster-point pairs.
            if ((aword(6) .eq. apaircl(npr))    .and.
     &          (apoint(np) .eq. apairpt(npr))) then
              ninb = 1
              go to 614
            endif
  612     continue

c....     Add point to cluster if it satisfies Boolean operation.

  614     nadd = 0

          if     (aword(5) .eq. '.and.') then
            if ((nina + ninb) .eq. 2) nadd = 1
          elseif (aword(5) .eq. '.int.') then
            if ((nina + ninb) .eq. 2) nadd = 1
          elseif (aword(5) .eq. '.nand.') then
            if ((nina + ninb) .eq. 2) nadd = 1
          elseif (aword(5) .eq. '.not.') then
            if ((nina + 2 * ninb) .eq. 1) nadd = 1
          elseif (aword(5) .eq. '.or.' ) then
            if ((nina + ninb) .ne. 0) nadd = 1
          elseif (aword(5) .eq. '.un.' ) then
            if ((nina + ninb) .ne. 0) nadd = 1
          elseif (aword(5) .eq. '.nor.') then
            if ((nina + ninb) .eq. 0) nadd = 1
          elseif (aword(5) .eq. '.xor.') then
            if ((nina + ninb) .eq. 1) nadd = 1
          elseif (aword(5) .eq. '.xnor.') then
            if ((nina + ninb) .ne. 1) nadd = 1
          elseif (aword(5) .eq. '.eqv.') then
            if ((nina + ninb) .ne. 1) nadd = 1
          endif

          if (nadd .eq. 1) then
            npairs = npairs + 1
            if (npairs .gt. npairm) then
              npairs = npairm
              write (   6, 8230) ainput, ninput, npairm
              write (iout, 8230) ainput, ninput, npairm
              nerr = 1
              go to 616
            endif

            apaircl(npairs) = aword(2)
            apairpt(npairs) = apoint(np)
          endif

  616   continue                      ! End of loop over all points.

        if (nerr .ne. 0) then
          npairs = npairx
          if (nrep .eq. 0) then
            aclust(n) = ' '
            if (n .eq. nclusts) nclusts = nclusts - 1
          endif
          go to 999
        endif

c....   Blank out old points for a redefined cluster.

        if (nrep .eq. 1) then
          do 618 npr = 1, npairx
            if (apaircl(npr) .eq. aword(2)) then
              apaircl(npr) = ' '
              apairpt(npr) = ' '
            endif
  618     continue
        endif

c....   Store and display the cluster specifications.

        aclust(n)  = aword(2)
        laclust(n) = lword(2)
        aclusts    = aword(2)
        laclusts   = lword(2)

        amount = 'maximum'
        call clout (n, n, amount)

        go to 999

      endif                           ! Doing Boolean operation.

c===============================================================================

c.... Input line:  cluster CLNAME cl CLNAME1 ... CLNAME2 [thru] CLNAME3 ...

      if (((aword(3) .eq. 'cluster') .or.
     &     (aword(3) .eq. 'cl'))     .and.
     &     (aword(4) .ne. '-'))      then

        npairx = npairs               ! Original number of cluster pairs.
cbugc***DEBUG begins.
cbug 9931 format ('DEBUG.  Initial number of pairs is ',i3)
cbug      write (   6, 9931) npairs
cbug      write (iout, 9931) npairs
cbugc***DEBUG ends.

c.... Find the existing clusters to add to the new cluster.

 8610 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No clusters specified.')
 8138 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name > max name:  ',a24,2x,a24)

      if (nword .eq. 3) then
        write (   6, 8610) ainput, ninput
        write (iout, 8610) ainput, ninput
        nerr = 1
cbugc***DEBUG begins.
cbug 9701 format ('DEBUG.  nbug=',i3,'  nw=',i5,'  aword(nw) = "',a24,'"',
cbug     &  '  nbug=',i5)
cbug      nbug = 100
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
        go to 999
      endif

      do 670 nw = 4, nword            ! Loop over input line words.

cbugc***DEBUG begins.
cbug 9932 format ('DEBUG.  Current # if pairs is ',i3 )
cbug      nbug = 110
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9932) npairs
cbug      write (iout, 9932) npairs
cbugc***DEBUG ends.

c....   Test for option "thru".

        if (aword(nw) .eq. 'thru') then  ! Found option "thru".

          if ((aword(nw+1) .eq. 'thru')  .or.
     &        (aword(nw-1) .eq. 'thru')  .or.
     &        (nw .eq. 4)                .or.
     &        (nw .eq. nword)            .or.
     &        (aword(nw-2) .eq. 'thru')) then
            write (   6, 8234) ainput, ninput
            write (iout, 8234) ainput, ninput
            nerr = 1
cbugc***DEBUG begins.
cbug      nbug = 120
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
            go to 675
          endif

          aclustmn = aword(nw-1)
          aclustmx = aword(nw+1)
cbugc***DEBUG begins.
cbug 9703 format ('DEBUG  aclustmn=',a24,'  aclustmx=',a24)
cbug      nbug = 130
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9703) aclustmn, aclustmx
cbug      write (iout, 9703) aclustmn, aclustmx
cbugc***DEBUG ends.

          call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
          if (nsmall .eq. 2) then
            write (   6, 8138) ainput, ninput, aclustmn, aclustmx
            write (iout, 8138) ainput, ninput, aclustmn, aclustmx
            nerr = 1
cbugc***DEBUG begins.
cbug      nbug = 140
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
            go to 675
          endif
cbugc***DEBUG begins.
cbug      nbug = 150
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.

          do 660 ncl = 1, nclusts     ! Loop over all existing clusters.
cbugc***DEBUG begins.
cbug      nbug = 152
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
            if (aclust(ncl) .eq. ' ') go to 660

            call compsubs (aclust(ncl), aclustmn, nsmall, nerrc)
cbugc***DEBUG begins.
cbug      nbug = 154
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
            if (nsmall .eq. 1) go to 660
            call compsubs (aclust(ncl), aclustmx, nsmall, nerrc)
cbugc***DEBUG begins.
cbug      nbug = 156
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
            if (nsmall .eq. 2) go to 660
cbugc***DEBUG begins.
cbug      nbug = 158
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.

            if (nerrc .ne. 0) go to 660

c....       Found an existing cluster to add to the new cluster.
cbugc***DEBUG begins.
cbug      nbug = 160
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.

            write (   6, 8630) aclust(ncl)
            write (iout, 8630) aclust(ncl)
            call cloutcl (ncl, ncl, 'min')

            do 650 npr = 1, npairx      ! Loop over cluster pairs.
              if (apaircl(npr) .eq. aclust(ncl)) then  ! Add this pair.
    
                npairs = npairs + 1
                if (npairs .gt. npairm) then
                  npairs = npairm
                  write (   6, 8230) ainput, ninput, npairm
                  write (iout, 8230) ainput, ninput, npairm
                  nerr = 1
cbugc***DEBUG begins.
cbug      nbug = 170
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
                  go to 675
                endif

                apaircl(npairs) = aword(2)
                apairpt(npairs) = apairpt(npr)

              endif                   ! Found a pair in listed cluster.

  650       continue                  ! End of loop over cluster pairs.

  660     continue                    ! End of loop over existing clusters.

          go to 670

        endif                         ! Tested for option "thru".
cbugc***DEBUG begins.
cbug      nbug = 180
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9932) npairs
cbug      write (iout, 9932) npairs
cbugc***DEBUG ends.

c....   Test for a cluster name.

        if (aword(nw-1) .eq. 'thru') go to 670
        if (aword(nw+1) .eq. 'thru') go to 670

        ncl = 0

        do 620 nc = 1, nclusts
          if (aclust(nc) .eq. aword(nw)) then
            ncl = nc
cbugc***DEBUG begins.
cbug 9933 format ('DEBUG.  Cluster to add is # ',i3,2x,a)
cbug      nbug = 190
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9933) nc, aclust(nc)
cbug      write (iout, 9933) nc, aclust(nc)
cbugc***DEBUG ends.
            go to 630
          endif
  620   continue

  630   if (ncl .gt. 0) then          ! Argument is a cluster name.
cbugc***DEBUG begins.
cbug      nbug = 200
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9932) npairs
cbug      write (iout, 9932) npairs
cbugc***DEBUG ends.

c....     Add the points in the existing cluster to the new cluster.

 8630     format ('  Adding    points in cluster ',a)

          if (nerr .eq. 0) then
            write (   6, 8630) aclust(ncl)
            write (iout, 8630) aclust(ncl)
            call cloutcl (ncl, ncl, 'min')
          endif

          do 640 npr = 1, npairx      ! Loop over cluster pairs.
            if (apaircl(npr) .eq. aclust(ncl)) then  ! Add this pair.

              npairs = npairs + 1
cbugc***DEBUG begins.
cbug      nbug = 210
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9932) npairs
cbug      write (iout, 9932) npairs
cbugc***DEBUG ends.
              if (npairs .gt. npairm) then
                npairs = npairm
                write (   6, 8230) ainput, ninput, npairm
                write (iout, 8230) ainput, ninput, npairm
cbugc***DEBUG begins.
cbug      nbug = 220
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
                nerr = 1
                go to 675
              endif
cbugc***DEBUG begins.
cbug      nbug = 230
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9932) npairs
cbug      write (iout, 9932) npairs
cbugc***DEBUG ends.
    
              apaircl(npairs) = aword(2)
              apairpt(npairs) = apairpt(npr)
cbugc***DEBUG begins.
cbug 9934 format ('DEBUG.  Pair from # ',i3,2x,a24,2x,a24)
cbug      write (   6, 9934) npr, apaircl(npr), apairpt(npr)
cbug      write (iout, 9934) npr, apaircl(npr), apairpt(npr)
cbug 9935 format ('DEBUG.  Pair to   # ',i3,2x,a24,2x,a24)
cbug      write (   6, 9935) npairs, apaircl(npairs), apairpt(npairs)
cbug      write (iout, 9935) npairs, apaircl(npairs), apairpt(npairs)
cbugc***DEBUG ends.
            endif                     ! Found a pair in existing cluster.
  640     continue                    ! End of loop over cluster pairs.

          go to 670

        else                          ! Not a cluster name or option "thru".

          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
cbugc***DEBUG begins.
cbug      nbug = 240
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.
          go to 675

        endif                         ! Tested for a cluster name.

  670 continue                        ! End of loop over input line words.
cbugc***DEBUG begins.
cbug      nbug = 250
cbug      nw = nword
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.

  675 if (nerr .ne. 0) then
        npairs = npairx
        if (nrep .eq. 0) then
          aclust(n) = ' '
          if (n .eq. nclusts) nclusts = nclusts - 1
        endif
cbugc***DEBUG begins.
cbug 9936 format ('DEBUG.  Final number if pairs is ',i3)
cbug      nbug = 260
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9936) npairs
cbug      write (iout, 9936) npairs
cbugc***DEBUG ends.
        go to 999
      endif

c.... Blank out old points for a redefined cluster.
cbugc***DEBUG begins.
cbug      nbug = 270
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.

      if (nrep .eq. 1) then
        do 680 npr = 1, npairx
          if (apaircl(npr) .eq. aword(2)) then
cbugc***DEBUG begins.
cbug 9937 format ('DEBUG.  Pair to blank is # ',i3,2x,a24,2x,a24)
cbug      nbug = 280
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbug      write (   6, 9937) npr, apaircl(npr), apairpt(npr)
cbug      write (iout, 9937) npr, apaircl(npr), apairpt(npr)
cbugc***DEBUG ends.
            apaircl(npr) = ' '
            apairpt(npr) = ' '
          endif
  680   continue
      endif

c.... Store and display the cluster specifications.

      aclust(n)  = aword(2)
      laclust(n) = lword(2)
      aclusts    = aword(2)
      laclusts   = lword(2)
cbugc___DEBUG begins.
cbug 9901 format ('cluster DEBUG.  Cluster ',i3,' is ',a)
cbug 9902 format ('cluster DEBUG.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9901) n, aclust(n)
cbug      write (iout, 9901) n, aclust(n)
cbug      write (   6, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.
cbugc***DEBUG begins.
cbug      nbug = 290
cbug      write (   6, 9701) nbug, nw, aword(nw), nerr
cbug      write (iout, 9701) nbug, nw, aword(nw), nerr
cbugc***DEBUG ends.

      amount = 'maximum'
      call clout (n, n, amount)

      go to 999

      endif                           ! Word 3 was "cluster".

c===============================================================================

c.... Input line:  cluster CLNAME cl - CLNAME1 ... CLNAME2 [thru] CLNAME3 ...

      if (((aword(3) .eq. 'cluster') .or.
     &     (aword(3) .eq. 'cl'))     .and.
     &     (aword(4) .eq. '-'))      then

c.... See if cluster CLNAME exists.

      if (nrep .eq. 0) then
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        aclust(n) = ' '
        if (n .eq. nclusts) nclusts = nclusts - 1
        go to 999
      endif

c.... See if any clusters are listed after "-".

      if (nword .eq. 4) then
        write (   6, 8610) ainput, ninput
        write (iout, 8610) ainput, ninput
        go to 999
      endif

c.... Find the listed clusters to delete from cluster CLNAME.

      do 770 nw = 5, nword            ! Loop over input line words.

c....   Test for option "thru".

        if (aword(nw) .eq. 'thru') then  ! Found option "thru".

          if ((nw          .eq. 5     )  .or.
     &        (aword(nw-2) .eq. 'thru')  .or.
     &        (aword(nw-1) .eq. 'thru')  .or.
     &        (aword(nw+1) .eq. 'thru')  .or.
     &        (aword(nw+2) .eq. 'thru')  .or.
     &        (nw          .eq. nword )) then
            write (   6, 8234) ainput, ninput
            write (iout, 8234) ainput, ninput
            nerr = 1
            go to 770
          endif

          aclustmn = aword(nw-1)
          aclustmx = aword(nw+1)

          call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
          if (nsmall .eq. 2) then
            write (   6, 8138) ainput, ninput, aclustmn, aclustmx
            write (iout, 8138) ainput, ninput, aclustmn, aclustmx
            go to 999
          endif

          do 760 ncl = 1, nclusts     ! Loop over all existing clusters.
            if (aclust(ncl) .eq. ' ') go to 760

            call compsubs (aclust(ncl), aclustmn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 760
            call compsubs (aclust(ncl), aclustmx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 760

c....         Found a listed cluster to remove from cluster CLNAME.

              write (   6, 8730) aclust(ncl)
              write (iout, 8730) aclust(ncl)
              call cloutcl (ncl, ncl, 'min')

              do 750 npr = 1, npairs  ! Loop over cluster pairs.
                if (apaircl(npr) .eq. aclust(ncl)) then  ! Remove this point.

                do 745 nprn = 1, npairs  ! Loop over cluster pairs.
                  if ((apaircl(nprn) .eq. aword(2))      .and.
     &                (apairpt(nprn) .eq. apairpt(npr))) then
                    apaircl(nprn) = ' '
                    apairpt(nprn) = ' '
                  endif               ! Removed a point from CLNAME.
  745           continue

                endif                 ! Found a pair in listed cluster.

  750         continue                ! End of loop over cluster pairs.

  760     continue                    ! End of loop over existing clusters.

          go to 770

        endif                         ! Tested for option "thru".

c....   Test for a cluster name.

        if (aword(nw-1) .eq. 'thru') go to 770
        if (aword(nw+1) .eq. 'thru') go to 770

        ncl = 0

        do 720 nc = 1, nclusts
          if (aclust(nc) .eq. aword(nw)) then
            ncl = nc
            go to 730
          endif
  720   continue

  730   if (ncl .gt. 0) then          ! Argument is a cluster name.

c....     Remove the points in the listed cluster from cluster CLNAME.

 8730     format ('  Removing  points in cluster ',a)

          write (   6, 8730) aclust(ncl)
          write (iout, 8730) aclust(ncl)
          call cloutcl (ncl, ncl, 'min')

          do 740 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .eq. aclust(ncl)) then  ! Remove point from CLNAME.

              do 735 nprn = 1, npairs  ! Loop over cluster pairs.
                if ((apaircl(nprn) .eq. aword(2))      .and.
     &              (apairpt(nprn) .eq. apairpt(npr))) then
                  apaircl(nprn) = ' '
                  apairpt(nprn) = ' '
                endif                 ! Removed a point from CLNAME.
  735         continue

            endif      
  740     continue                    ! End of loop over cluster pairs.

          go to 770

        else                          ! Not a cluster name or option "thru".

          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)

        endif                         ! Argument is a cluster name.

  770 continue                        ! End of loop over input line words.

c.... Store and display the cluster specifications.

      aclust(n)  = aword(2)
      laclust(n) = lword(2)
      aclusts    = aword(2)
      laclusts   = lword(2)

      amount = 'maximum'
      call clout (n, n, amount)

      go to 999

      endif                           ! Words 3-4 were "cluster -".

c===============================================================================

c.... Input line:  cluster CLNAME brick BRNAME

      if ((aword(3) .eq. 'brick') .or.
     &    (aword(3) .eq. 'br'))   then

        nerr = 0

        npairx = npairs

 8682   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "cluster CLNAME brick BRNAME" command',
     &    ' not 4:  ',i2)

        if (nword .ne. 4) then
          write (   6, 8682) ainput, ninput, nword
          write (iout, 8682) ainput, ninput, nword
          nerr = 1
          go to 688
        endif

 8684   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named brick does not exist:  ',a)

        do 684 nb = 1, nbricks
          if (aword(4) .eq. abrick(nb)) then
            call brout (nb, nb, 'min')
            go to 685
          endif
  684   continue
        write (   6, 8684) ainput, ninput, aword(4)
        write (iout, 8684) ainput, ninput, aword(4)
        nerr = 1
        go to 688

  685   continue

        do 686 np = 1, npoints
          if (apoint(np) .eq. ' ') go to 686

          call ptbrick (0, np, nb, nside, fracu, fracv, fracw)

          if (nside .eq. 1) then

            npairs  = npairs + 1
    
            if (npairs .gt. npairm) then
              npairs = npairm
              write (   6, 8330) ainput, ninput, npairm
              write (iout, 8330) ainput, ninput, npairm
              nerr = 1
              go to 688
            endif
    
            apaircl(npairs) = aword(2)
            apairpt(npairs) = apoint(np)

          endif

  686   continue

  688   if (nerr .ne. 0) then
          npairs = npairx
          if (nrep .eq. 0) then
            aclust(n) = ' '
            if (n .eq. nclusts) nclusts = nclusts - 1
          endif
          go to 999
        endif

c....   Blank out old points for a redefined cluster.

        if (nrep .eq. 1) then
          do 689 npr = 1, npairx
            if (apaircl(npr) .eq. aword(2)) then
              apaircl(npr) = ' '
            endif
  689     continue
        endif

c....   Store and display the cluster specifications.

        aclust(n)  = aword(2)
        laclust(n) = lword(2)
        aclusts    = aword(2)
        laclusts   = lword(2)

        amount = 'maximum'
        call clout (n, n, amount)

        go to 999

      endif                           ! Word 3 was "brick".

c===============================================================================

c.... Input line:  cluster CLNAME tetrahedron TETNAME

      if ((aword(3) .eq. 'tetrahedron') .or.
     &    (aword(3) .eq. 'tetra')       .or.
     &    (aword(3) .eq. 'tet'))        then

        nerr = 0

        npairx = npairs

 8692   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "cluster CLNAME tetrahedron',
     &    ' TETNAME" command not 4:  ',i2)

        if (nword .ne. 4) then
          write (   6, 8692) ainput, ninput, nword
          write (iout, 8692) ainput, ninput, nword
          nerr = 1
          go to 698
        endif

 8694   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named tetrahedron does not exist:  ',a)

        do 694 nt = 1, ntets
          if (aword(4) .eq. atetra(nt)) then
            call tetout (nt, nt, 'min')
            go to 695
          endif
  694   continue
        write (   6, 8694) ainput, ninput, aword(4)
        write (iout, 8694) ainput, ninput, aword(4)
        nerr = 1
        go to 698

  695   continue

        do 696 np = 1, npoints
          if (apoint(np) .eq. ' ') go to 696

          call pttet (0, np, nt, nside, wa, wb, wc, wd, nerra)

          if (nside .eq. 1) then

            npairs  = npairs + 1
    
            if (npairs .gt. npairm) then
              npairs = npairm
              write (   6, 8330) ainput, ninput, npairm
              write (iout, 8330) ainput, ninput, npairm
              nerr = 1
              go to 698
            endif
    
            apaircl(npairs) = aword(2)
            apairpt(npairs) = apoint(np)

          endif

  696   continue

  698   if (nerr .ne. 0) then
          npairs = npairx
          if (nrep .eq. 0) then
            aclust(n) = ' '
            if (n .eq. nclusts) nclusts = nclusts - 1
          endif
          go to 999
        endif

c....   Blank out old points for a redefined cluster.

        if (nrep .eq. 1) then
          do 699 npr = 1, npairx
            if (apaircl(npr) .eq. aword(2)) then
              apaircl(npr) = ' '
            endif
  699     continue
        endif

c....   Store and display the cluster specifications.

        aclust(n)  = aword(2)
        laclust(n) = lword(2)
        aclusts    = aword(2)
        laclusts   = lword(2)

        amount = 'maximum'
        call clout (n, n, amount)

        go to 999

      endif                           ! Word 3 was "tetrahedron".

c===============================================================================

c.... Input line:  cluster CLNAME zone ZNAME

      if ((aword(3) .eq. 'zone') .or.
     &    (aword(3) .eq. 'zn')   .or.
     &    (aword(3) .eq. 'z'))   then

        nerr = 0

        npairx = npairs

 8712   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "cluster CLNAME zone',
     &    ' ZNAME" command not 4:  ',i2)

        if (nword .ne. 4) then
          write (   6, 8712) ainput, ninput, nword
          write (iout, 8712) ainput, ninput, nword
          nerr = 1
          go to 718
        endif

 8714   format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named zone does not exist:  ',a)

        do 714 nz = 1, nzones
          if (aword(4) .eq. azone(nz)) then
            call zout (nz, nz, 'min')
            go to 715
          endif
  714   continue
        write (   6, 8714) ainput, ninput, aword(4)
        write (iout, 8714) ainput, ninput, aword(4)
        nerr = 1
        go to 718

  715   continue

        do 716 np = 1, npoints
          if (apoint(np) .eq. ' ') go to 716

          amount = 'minimum'

          call ptzone (0, np, nz, amount, dside, nside, nerra)

          if (nside .eq. 1) then

            npairs  = npairs + 1
    
            if (npairs .gt. npairm) then
              npairs = npairm
              nerr = 1
            endif
    
            apaircl(npairs) = aword(2)
            apairpt(npairs) = apoint(np)

          endif

  716   continue

  718   if (nerr .ne. 0) then
          npairs = npairx
          if (nrep .eq. 0) then
            aclust(n) = ' '
            if (n .eq. nclusts) nclusts = nclusts - 1
          endif
          go to 999
        endif

c....   Blank out old points for a redefined cluster.

        if (nrep .eq. 1) then
          do 719 npr = 1, npairx
            if (apaircl(npr) .eq. aword(2)) then
              apaircl(npr) = ' '
            endif
  719     continue
        endif

c....   Store and display the cluster specifications.

        aclust(n)  = aword(2)
        laclust(n) = lword(2)
        aclusts    = aword(2)
        laclusts   = lword(2)

        amount = 'maximum'
        call clout (n, n, amount)

        go to 999

      endif                           ! Word 3 was "zone".

c===============================================================================

 8900 format ('  cluster ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8900) ainput, ninput
      write (iout, 8900) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine cluster.      (+1 line)
      end
