      subroutine contfr

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  contfr   Updated     2006 April 11 18:00.
c               contfr   Originated  2000 October 16 14:45.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the coefficients of the continued fraction
c               equivalent of a number, and the inverse.
c
c     Logic:    Entry points:  contfr.
c contfr        Called by:  helpall, main.
c
c contfr        Calls:  aptcofr, aptcofv, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension inum(64)              ! Numerator of convergent.
      dimension iden(64)              ! Denominator of convergent.
      dimension kc(64)                ! Coefficients of continued fraction.
      dimension xc(64)                ! Partial sum of continued fraction.
      dimension xnum(64)              ! Numerator of convergent.
      dimension xden(64)              ! Denominator of convergent.

c.... Input line:  contfr [help]

 8110 format (
     &  '  Command "contfr":  acceptable input lines include:' //
     &  '  "contfr [help]"  Display this message.' //
     &  '  "contfr = VALUE"' /
     &  '     Find the coefficients kc1, kc2, kc3, kc4, ...of the',
     &  ' continued fraction' /
     &  '     VALUE = kc1 + 1/(kc2 + 1/(kc3 + 1/(kc4 + ...))).' //
     &  '  "contfr C1 C2 C3 ..."' /
     &  '     Find the value x of the continued fraction' /
     &  '     x = C1 + 1/(C2 + 1/(C3 + 1/(C4 + ...))),' /
     &  '     and the equivalent rational fraction xnum / xden = x.' //
     &  '  Synonyms:  [contfr, cf], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Input line:  contfr = VALUE

      if (aword(2) .eq. '=') then

c.... Test number of words on input line.

 8135 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "contfr" line not 3:  ',i3)

      if (nword .ne. 3) then
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
        go to 999
      endif

c.... Find the variable.

 8276 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 3 not integer or floating point value or variable:  ',a)
 8278 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 3 negative:  ',a)

      call varfind (3, 3)
  
      if ((mtype(3) .ne. 2) .and. (mtype(3) .ne. 3)) then
        write (   6, 8276) ainput, ninput, aword(3)
        write (iout, 8276) ainput, ninput, aword(3)
        go to 999
      endif

c.... Find the coefficients of the continued fraction.

      call aptcofr (fword(3), 64, tol, kc, inum, iden, xc, nc, nerra)

 8202 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  ncmax is not positive.' )
 8203 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Coefficient exceeds largest possible integer.' )
 8204 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Denominator is 0 in aptcofr.' )
 8205 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Unknown error in aptcofr.' )

      if (nerra .ne. 0) then
        if (nerra .eq. 1) then
          write (   6, 8202) ainput, ninput
          write (iout, 8202) ainput, ninput
        elseif (nerra .eq. 2) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        else
          write (   6, 8205) ainput, ninput
          write (iout, 8205) ainput, ninput
        endif
        go to 999
      endif
  
c.... Display the coefficients, approximates, numerators and denominators.

 8210 format ('  kc(',i2,') =',i16 )
 8285 format ('  cf',18i4 / 4x,18i4 / 4x,18i4 / 4x,10i4 )
 8286 format ('  cf',6i12 / 4x,6i12 / 4x,6i12 / 4x,6i12 / 4x,6i12 /
     &               6i12 / 4x,6i12 / 4x,6i12 / 4x,6i12 / 4x,6i12 /
     &               4i12 )

      nlong = 0
      do 270 n = 1, nc
        write (   6, 8210) n, kc(n)
        write (iout, 8210) n, kc(n)
        if (kc(n) .gt. 999) nlong = 1
        if (kc(n) .lt. -99) nlong = 1
  270 continue

      if (nlong .eq. 0) then
        write (   6, 8285) (kc(n), n = 1, nc)
        write (iout, 8285) (kc(n), n = 1, nc)
      else
        write (   6, 8286) (kc(n), n = 1, nc)
        write (iout, 8286) (kc(n), n = 1, nc)
      endif

 8280 format (i4,' x,inum,iden =',1pe20.12,2i20)
 8282 format ('  cf =            ',1pe22.14)

      do 280 n = 1, nc
        write (   6, 8280) n, xc(n), inum(n), iden(n)
        write (iout, 8280) n, xc(n), inum(n), iden(n)
  280 continue

      write (   6, 8282) xc(nc)
      write (iout, 8282) xc(nc)
      go to 999

      endif                           ! Find coefficients of continued fraction.

c=======================================================================********


c.... Input line:  contfr C1 C2 C3 ... CN

      call varfind (2, nword)

 8310 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      nerr = 0

      do 310 nw = 2, nword
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8310) ainput, ninput, aword(nw)
          write (iout, 8310) ainput, ninput, aword(nw)
          nerr = 1
        endif
  310 continue

      if (nerr .ne. 0) go to 999

c.... Find the value of the continued fraction.

      nc = nword - 1

      call aptcofv (fword(2), nc, tol, xnum, xden, xc, nerra)

 8320 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  An infinity occurred.' )

      if (nerra .ne. 0) then
        write (   6, 8320) ainput, ninput
        write (iout, 8320) ainput, ninput
        go to 999
      endif
  
c.... Display the value.

 8330 format (i4,' x,xnum,xden =',1p3e20.12 )
 8332 format ('  cf =            ',1pe22.14 )

      write (   6, 8330) (n, xc(n), xnum(n), xden(n), n = 1, nc)
      write (iout, 8330) (n, xc(n), xnum(n), xden(n), n = 1, nc)
      write (   6, 8332)  xc(nc)
      write (iout, 8332)  xc(nc)

      go to 999

c=======================================================================********

 8810 format ('  contfr ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine contfr.      (+1 line)
      end
