      subroutine cylinder

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  cylinder Updated     2005 April 5 17:45.
c               cylinder Originated  1993 December 16 11:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create circular cylinders
c               (but see option "scale").
c
c     Logic:    Entry points:  cylinder.
c cylinder      Called by:  helpall, main.
c
c cylinder      Calls:  aptcirc, aptclis, aptmopv, aptrois, aptrotv, aptsubr,
c               apttris, incrsubs, namechk, ptfind, qumove, quoper, quout,
c               qustore, varfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lcyl/ rotm(3,3)         ! Principle axis rotation tensor.

      character*8  amount             ! Amount of ouput data.
      character*24 anamex             ! Temporary name of a cylinder.

c.... Input line:  cylinder [help]

 8110 format (
     &  '  Command "cylinder":  acceptable input lines include:' /
     &  '  "cylinder [help]"  Display this message.' /
     &  '  "cylinder all"     Display all circular cylinders.' /
     &  '  "cylinder list [CYLNAME1 CYLNAME2 CYLNAME3 ...]"' /
     &  '     Display all listed circular cylinders',
     &  ' (default is all cylinders).' /
     &  '  "cylinder CYLNAME"  Display circular cylinder CYLNAME.' /
     &  '  "cylinder CYLNAME = PAXIS VAXIS RADIUS"' /
     &  '     Create circular cylinder CYLNAME, with an axis through',
     &  ' point PAXIS' /
     &  '     in the direction of vector VAXIS, and radius RADIUS.' /
     &  '  "cylinder CYLNAME fit VAXIS PNAME1 PNAME2 PNAME3"' /
     &  '     Create circular cylinder CYLNAME with an axis',
     &  ' parallel to vector VAXIS,' /
     &  '     through the three points PNAME1, PNAME2 and PNAME3.' )

 8112 format (/
     &  '  "cylinder CYLNAME concentric NUMCYL INC PAXIS VAXIS RAD1',
     &  ' RAD2"' /
     &  '     Create a family of NUMCYL circular cylinders CYLNAME,',
     &  ' ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     with an axis through point PAXIS in the direction of',
     &  ' vector VAXIS,' /
     &  '     and with radii equally spaced from radius RAD1 to radius',
     &  ' RAD2.' //
     &  '  "cylinder CYL02 NUMCYL INC move CYL01 VMOVE"' /
     &  '     Create a family of NUMCYL circular cylinders CYL02,',
     &  ' ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     and spaced at intervals of vector VMOVE,' /
     &  '     starting from circular cylinder CYL01.' //
     &  '  "cylinder CYL02 rotate NUMCYL INC CYL01 OPNAME PINV"' /
     &  '     Create a family of NUMCYL circular cylinders CYL02,',
     &  ' ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding cylinder with operator OPNAME' /
     &  '     and invariant point PINV, starting from circular',
     &  ' cylinder CYL01.' )

 8114 format (/
     &  '  "cylinder CYL02 scale NUMCYL INC CYL01 OPNAME PINV"' /
     &  '     Create a family of NUMCYL cylinders CYL02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding cylinder with operator OPNAME' /
     &  '     and invariant point PINV, starting from circular',
     &  ' cylinder CYL01.' /
     &  '     WARNING:  a linear or radial scaling axis not parallel' /
     &  '     to that of CYL01 will produce elliptic cylinders.' //
     &  '  Note:  use +/- for CYLNAME or CYL02 to incr/decr the last' /
     &  '    circular cylinder name:  ',a //
     &  '  Synonyms:  [help, h], [concentric, conc], [cylinder, cyl],' /
     &  '    [move, mv, translate, tran], [rotate, rot],',
     &  '  [scale, sc].' )

      if (nword .eq. 0) go to 999

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114) acyls
        write (iout, 8114) acyls
        go to 999
      endif

c=======================================================================********

c.... Input line:  cylinder all
c.... Input line:  cylinder list

 8120 format (/ 'Number of cylinders = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        ncylx = 0
        do 120 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(7)) go to 120
          if (aquad(n) .eq. ' ') go to 120
          call quout (n, n, 'min', nerrq)
          ncylx = ncylx + 1
  120   continue
        write (   6, 8120) ncylx
        write (iout, 8120) ncylx
        go to 999
      endif

c=======================================================================********

c.... Input line:  cylinder list CYLNAME1 CYLNAME2 CYLNAME3 ...

 8130 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cylinder does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            if (aqtype(n) .ne. aqtypes(7)) go to 125
            call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call quout (n, n, 'min', nerrq)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  cylinder CYLNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(7)) go to 130
          if (aword(2) .eq. aquad(n)) then
            amount = 'maximum'
            call quout (n, n, amount, nerrq)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

 8136 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of cylinder not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if the last cylinder name should be incremented and used.

 8128 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last cylinder can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = acyls
        lword(2) = lacyls
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, acyls
          write (iout, 8128) ainput, ninput, acyls
          go to 999
        endif
      endif

 8129 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last cylinder can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = acyls
        lword(2) = lacyls
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, acyls
          write (iout, 8129) ainput, ninput, acyls
          go to 999
        endif
      endif

 8905 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of cylinder too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  cylinder CYLNAME concentric NUMCYL INC PAXIS VAXIS RAD1 RAD2

      if ((aword(3) .eq. 'concentric') .or.
     &    (aword(3) .eq. 'conc'))      then

        nerr = 0

 8505   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 9 words in input line:  ',i3)

        if (nword .ne. 9) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          nerr = 1
        endif

 8520   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMCYL is less than 2:  ',a)
 8525   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMCYL is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numcyl = iword(4)
          if (numcyl .lt. 2) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          nerr = 1
        endif

 8510   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
            nerr = 1
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the point on the axis of the cylinder.

        call ptfind (6, np)

        if (np .eq. 0) nerr = 1

c....   Find the axis vector.

        call vectfind (7, nv)

        if (nv .eq. 0) nerr = 1

c....   Find the inner and outer radii.

        call varfind (8, 9)

        if ((mtype(8) .ne. 2) .and. (mtype(8) .ne. 3)) then
          write (   6, 8260) ainput, ninput, aword(8)
          write (iout, 8260) ainput, ninput, aword(8)
          nerr = 1
        endif

        rad1 = fword(8)

        if ((mtype(9) .ne. 2) .and. (mtype(9) .ne. 3)) then
          write (   6, 8260) ainput, ninput, aword(9)
          write (iout, 8260) ainput, ninput, aword(9)
          nerr = 1
        endif

        rad2 = fword(9)

c....   Find the radial increment between the cylinders.
c....     Use the fuzz factor 1.e-99 in case numcyl = 1 (an error).

        drad = (rad2 - rad1) / (numcyl - 1 + 1.e-99)

c....   See if the cylinder names can be generated uniquely.

 8570 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' cylinder names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numcyl - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8570) ainput, ninput, numcyl, aword(2)
          write (iout, 8570) ainput, ninput, numcyl, aword(2)
          nerr = 1
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

c....   Find the operator to rotate the axis of the cylinder onto the z axis.

        call aptrotv (xcom(nv), ycom(nv), zcom(nv), 0., 0., 1., tol,
     &                rotm, nerra)

c....   See if all radii are positive.

 8575 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not make all radii sufficiently positive.')

        if (np .ne. 0) then
          errsq = tol * (x(np)**2 + y(np)**2 + z(np)**2)
        else                          ! Axis point does not exist.
          errsq = 0.0
        endif

        if ((rad1**2 .le. errsq)  .or.
     &      (rad2**2 .le. errsq)  .or.
     &      (rad1 .le. 0.0)       .or.
     &      (rad2 .le. 0.0))      then
          write (   6, 8575) ainput, ninput
          write (iout, 8575) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) go to 999

c....   Create the concentric cylinders.

        anamex  = aword(2)            ! Initial cylinder name.
        lanamex = lword(2)            ! Initial cylinder name length.

        do 580 ncyl = 1, numcyl       ! Loop over the new cylinders.

c....     Find the index in the quadric surface list for the new cylinder.

          aword(2) = anamex
          lword(2) = lanamex

          call qustore (7, nrep, n)

          if (n .eq. 0) go to 999


          acyls     = anamex
          lacyls    = lanamex
          aquads    = anamex
          laquads   = lanamex
          axisyms   = anamex
          laxisyms  = lanamex

          aquad(n)  = anamex
          laquad(n) = lanamex
          aqtype(n) = aqtypes(7)
          asym(n)   = 'axisymmetric'

          radius = rad1 + (ncyl - 1) * drad
    
          qc(n)  = -radius**2
          qx(n)  = 0.0
          qy(n)  = 0.0
          qz(n)  = 0.0
          qxy(n) = 0.0
          qyz(n) = 0.0
          qzx(n) = 0.0
          qxx(n) = 1.0
          qyy(n) = 1.0
          qzz(n) = 0.0
    
          xcen(n) = x(np)
          ycen(n) = y(np)
          zcen(n) = z(np)

c....     Rotate the cylinder back to the base axis.

          call aptrois (rotm, 1, 0., 0., 0., qc(n), qx(n), qy(n), qz(n),
     &                  qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &                  tol, nerra)

c....     Translate the cylinder back to the base point.

          call apttris (1, xcen(n), ycen(n), zcen(n),
     &                  qc(n), qx(n), qy(n), qz(n),
     &                  qxy(n), qyz(n), qzx(n),
     &                  qxx(n), qyy(n), qzz(n), tol, nerra)

          call quout (n, n, 'min', nerrq)

c....     Find the next cylinder name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  580   continue                      ! End of loop over the new cylinders.

        go to 999

      endif                           ! Created concentric cylinder(s).

c===============================================================================

c.... Input line:  cylinder CYL02 move NUMCYL INC CYL01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(5) .eq. 'trans'))    then

        call qumove

        go to 999

      endif                           ! Created translated cylinder(s).

c===============================================================================

c.... Input line:  cylinder CYL02 rotate NUMCYL INC CYL01 OPNAME PINV
c.... Input line:  cylinder CYL02 scale  NUMCYL INC CYL01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call quoper

        go to 999

      endif                           ! Created rotated or scaled cylinder(s).

c===============================================================================

c.... Find the index in the quadric surface array at which to store this
c....   cylinder.  Set nrep = 1 if replacing an existing quadric surface.

      call qustore (7, nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c=======================================================================********

c.... Input line:  cylinder CYLNAME = PAXIS VAXIS RADIUS

      if ((nword .eq. 6)       .and.
     &    (aword(3) .eq. '=')) then

c....   Find the radius of the cylinder.

 8260   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

        call varfind (6, 6)

        if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
          write (   6, 8260) ainput, ninput, aword(6)
          write (iout, 8260) ainput, ninput, aword(6)
          nerr = 1
        endif

 8270   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Radius of cylinder is not sufficiently positive:  ',
     &    1pe20.12)

  270   if (fword(6) .le. 0.0) then
          write (   6, 8270) ainput, ninput, fword(6)
          write (iout, 8270) ainput, ninput, fword(6)
          nerr = 1
        endif

        radius = fword(6)

c....   Find the point on the axis of the cylinder.

        call ptfind (4, np)

        if (np .eq. 0) then
          nerr = 1
          go to 290
        endif

        errsq = tol * (x(np)**2 + y(np)**2 + z(np)**2)

        if ((radius**2 .le. errsq) .or.
     &      (radius .le. 0.0))     then
          write (   6, 8270) ainput, ninput, radius
          write (iout, 8270) ainput, ninput, radius
          nerr = 1
        endif

c....   Find the axis of the cylinder.

  290   call vectfind (5, nv)

        if (nv .eq. 0) nerr = 1

c....   See if the axis vector has a real length.

 8310   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Axis vector has zero length:  ',a)

        if (vlen(nv) .eq. 0.0) then
          nerr = 1
          write (   6, 8310) ainput, ninput, aword(5)
          write (iout, 8310) ainput, ninput, aword(5)
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            aquad(n) = ' '
            if (n .eq. nquads) nquads = nquads - 1
          endif
          go to 999
        endif

c....   Find the coefficients of the quadric equation for the cylinder.

        acyls     = aword(2)
        lacyls    = lword(2)
        aquads    = aword(2)
        laquads   = lword(2)
        axisyms   = aword(2)
        laxisyms  = lword(2)

        aquad(n)  = aword(2)
        laquad(n) = lword(2)

        call aptclis (x(np), y(np), z(np), xcom(nv), ycom(nv), zcom(nv),
     &                radius, tol, qc(n),
     &                qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &                qxx(n), qyy(n), qzz(n), nerra)

        amount = 'maximum'
        call quout (n, n, amount, nerrq)

        go to 999

      endif                           ! Create cylinder:  point, axis, radius. 

c-----------------------------------------------------------------------********

c.... Input line:  cylinder CYLNAME fit VAXIS PNAME1 PNAME2 PNAME3

      if ((nword .eq. 7)         .and.
     &    (aword(3) .eq. 'fit')) then

c....   Find the axis of the cylinder.

        call vectfind (4, nv)

        if (nv .eq. 0) nerr = 1

c....   See if the axis vector has a real length.

        if (vlen(nv) .eq. 0.0) then
          nerr = 1
          write (   6, 8310) ainput, ninput, aword(4)
          write (iout, 8310) ainput, ninput, aword(4)
        endif

c....   Find the three points.

        call ptfind (5, np1)

        if (np1 .eq. 0) nerr = 1

        call ptfind (6, np2)

        if (np2 .eq. 0) nerr = 1

        call ptfind (7, np3)

        if (np3 .eq. 0) nerr = 1

  380   if (nerr .ne. 0) go to 385

        x1  = x(np1)
        y1  = y(np1)
        z1  = z(np1)

        x2  = x(np2)
        y2  = y(np2)
        z2  = z(np2)

        x3  = x(np3)
        y3  = y(np3)
        z3  = z(np3)

c....   Find the operator to rotate the z axis to the axis of the cylinder.

        call aptrotv (0., 0., 1., xcom(nv), ycom(nv), zcom(nv), tol,
     &                rotm, nerra)

c....   Rotate the three points with the inverse of the operator.

        call aptmopv (rotm, 1, 0., 0., 0., x1, y1, z1, 1, tol, nerra)

        call aptmopv (rotm, 1, 0., 0., 0., x2, y2, z2, 1, tol, nerra)

        call aptmopv (rotm, 1, 0., 0., 0., x3, y3, z3, 1, tol, nerra)

c....   Fit a circle to the three points projected onto the XY plane.

        call aptcirc (x1, y1, x2, y2, x3, y3, 1, tol, xcyl, ycyl,
     &                radius, nerra)

 8380   format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Points are coincident or colinear.')

        if (radius .gt. 1.e99) then
          nerr = 1
          write (   6, 8380) ainput, ninput
          write (iout, 8380) ainput, ninput
        endif

  385   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            aquad(n) = ' '
            if (n .eq. nquads) nquads = nquads - 1
          endif
          go to 999
        endif

c....   Find the coefficients of the cylinder parallel to the z axis.

        acyls     = aword(2)
        lacyls    = lword(2)
        aquads    = aword(2)
        laquads   = lword(2)
        axisyms   = aword(2)
        laxisyms  = lword(2)

        aquad(n)  = aword(2)
        laquad(n) = lword(2)
        qc(n)     = -radius**2 + xcyl**2 + ycyl**2
        qx(n)     = -2.0 * xcyl
        qy(n)     = -2.0 * ycyl
        qz(n)     = 0.0
        qxy(n)    = 0.0
        qyz(n)    = 0.0
        qzx(n)    = 0.0
        qxx(n)    = 1.0
        qyy(n)    = 1.0
        qzz(n)    = 0.0

c....   Rotate the cylinder back to the original axis.

        call aptrois (rotm, 0, 0., 0., 0., qc(n), qx(n), qy(n), qz(n),
     &                qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &                tol, nerra)

        amount = 'maximum'
        call quout (n, n, amount, nerrq)

        go to 999

      endif                           ! Create cylinder:  axis, three points.

c=======================================================================********

 8900 format ('  cylinder ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8900) ainput, ninput
      write (iout, 8900) ainput, ninput

      if (nrep .eq. 0) then
        aquad(n) = ' '
        if (n .eq. nquads) nquads = nquads - 1
      endif

c=======================================================================********

  999 return

c.... End of subroutine cylinder.     (+1 line)
      end
