      subroutine intquad (nq1, nq2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intquad  Updated     1998 June 2 17:00.
c               intquad  Originated  1997 September 29 10:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the distance of separation between two non-intersecting
c               quadric surfaces, or a random point on the intersection curve of
c               two intersecting quadric surfaces.
c
c     Logic:    Entry points:  intquad
c intquad       Called by:  distance.
c
c intquad       Calls:  aptintq, aptvaxb, aptwhis, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*16 aside1
      character*16 aside2

      common /lintquad/ px1(4)        ! Proximal point on surface 1.
      common /lintquad/ py1(4)        ! Proximal point on surface 1.
      common /lintquad/ pz1(4)        ! Proximal point on surface 1.

      common /lintquad/ px2(4)        ! Proximal point on surface 2.
      common /lintquad/ py2(4)        ! Proximal point on surface 2.
      common /lintquad/ pz2(4)        ! Proximal point on surface 2.

      common /lintquad/ dist1(4)      ! Distance point 1 to surface 2.
      common /lintquad/ dist2(4)      ! Distance point 1 to surface 2.

      character*8 aquint              ! Type of proximity or intersection.

c-----------------------------------------------------------------------********

c.... Initialize.

      pdist  = -1.e99
      aside1 = ' '
      aside2 = ' '

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx = -1.e99
      proxvy = -1.e99
      proxvz = -1.e99

c.... For quadric surface nq1 and quadric surface nq2, find the proximal points
c.... and distance of separation if they do not intersect, or find a point on
c.... the intersection curve if they do intersect.

c.... Sample the initial trial point randomly.

      fract = ranf ( )
      px2(1) = xcen(nq1) + fract * (xcen(nq2) - xcen(nq1))
      py2(1) = ycen(nq1) + fract * (ycen(nq2) - ycen(nq1))
      pz2(1) = zcen(nq1) + fract * (zcen(nq2) - zcen(nq1))

      px2(1) = px2(1) + 1.0 - 2.0 * ranf ( )
      py2(1) = py2(1) + 1.0 - 2.0 * ranf ( )
      pz2(1) = pz2(1) + 1.0 - 2.0 * ranf ( )

 9962 format (/ 'Initial px,py,pz =',1p3e20.12)

      write (   6, 9962) px2(1), py2(1), pz2(1)
      write (iout, 9962) px2(1), py2(1), pz2(1)

      pxsave = px2(1)
      pysave = py2(1)
      pzsave = pz2(1)


      ppx1   = -1.e99
      ppy1   = -1.e99
      ppz1   = -1.e99

      ppx1   = -1.e99
      ppy2   = -1.e99
      ppz2   = -1.e99

c.... Initialize the two quadric equations.

      qc1  = qc(nq1)
      qx1  = qx(nq1)
      qy1  = qy(nq1)
      qz1  = qz(nq1)
      qxy1 = qxy(nq1)
      qyz1 = qyz(nq1)
      qzx1 = qzx(nq1)
      qxx1 = qxx(nq1)
      qyy1 = qyy(nq1)
      qzz1 = qzz(nq1)

      qc2  = qc(nq2)
      qx2  = qx(nq2)
      qy2  = qy(nq2)
      qz2  = qz(nq2)
      qxy2 = qxy(nq2)
      qyz2 = qyz(nq2)
      qzx2 = qzx(nq2)
      qxx2 = qxx(nq2)
      qyy2 = qyy(nq2)
      qzz2 = qzz(nq2)


c.... Use apt subroutine aptintq to find intersection or proximal points.

      call aptintq (pxsave, pysave, pzsave,
     &     qc1, qx1, qy1, qz1, qxy1, qyz1, qzx1, qxx1, qyy1, qzz1,
     &     qc2, qx2, qy2, qz2, qxy2, qyz2, qzx2, qxx2, qyy2, qzz2,
     &     tol, aquint, tax, tay, taz, tbx, tby, tbz,
     &     dx, dy, dz, dist12, nerra1)
 
      call aptwhis (tbx, tby, tbz,
     &     qc1, qx1, qy1, qz1, qxy1, qyz1, qzx1, qxx1, qyy1, qzz1,
     &     tol,
     &     dside1, anx, any, anz,
     &     nprox, px1, py1, pz1, adprox, nerra)
 
      call aptwhis (tax, tay, taz,
     &     qc2, qx2, qy2, qz2, qxy2, qyz2, qzx2, qxx2, qyy2, qzz2,
     &     tol,
     &     dside2, bnx, bny, bnz,
     &     nprox, px2, py2, pz2, adprox, nerra)

c.... See if the two normal vectors are parallel or antiparallel,
c....   indicating a proximal point or tangent point.

      call aptvaxb (anx, any, anz, bnx, bny, bnz, 1, tol,
     &              dnx, dny, dnz, vlength, nerra)

 8125 format (/ 'Quadric surface is bad, or no proximal point.')
 8130 format (/ 'Failed to converge in 1000 iterations.')
 8132 format (/ 'Diverged.')
 8135 format (/ 'Converged.')

      if (nerra1 .ne. 0) then
        if (nerra1 .eq. 1) then
          write (   6, 8125)
          write (iout, 8125)
        elseif (nerra1 .eq. 2) then
          write (   6, 8130)
          write (iout, 8130)
        elseif (nerra1 .eq. 3) then
          write (   6, 8132)
          write (iout, 8132)
        endif
        go to 999
      else
        write (   6, 8135)
        write (iout, 8135)
        proxptx = px1(1)
        proxpty = py1(1)
        proxptz = pz1(1)
        proxvx  = anx
        proxvy  = any
        proxvz  = anz
      endif

      if (dside1 .eq. 0.0) then
        aside1 = '**ON SURFACE 2**'
      endif

      if (dside2 .eq. 0.0) then
        aside2 = '**ON SURFACE 1**'
      endif

 8140 format (/ 'Two surfaces overlap.')
 8150 format (/ 'Common point is on intersection curve.')
 8170 format (/ 'Two surfaces do not overlap.')
 8180 format (/ 'Common point is at tangency.')

      if ((dside1 .lt. 0.0) .or. (dside2 .lt. 0.0)) then  ! Overlap
        write (   6, 8140)
        write (iout, 8140)
        dist12 = -abs (dist12)
      elseif ((dside1 .gt. 0.0) .and. (dside2 .gt. 0.0)) then  ! No overlap.
        write (   6, 8170)
        write (iout, 8170)
      else                            ! Common point on intersection curve.
        write (   6, 8150)
        write (iout, 8150)
        if (vlength .eq. 0.0) then
          write (   6, 8180)
          write (iout, 8180)
        endif
      endif

 8185 format (/ 'Relationship of point P1 and P2:  ',a)

      write (   6, 8185) aquint
      write (iout, 8185) aquint

 8250 format (/ 'Point P1 x,y,z   =',1p3e20.12)
 8260 format (  'Point P2 x,y,z   =',1p3e20.12)
 8270 format ('P1 to P2 dx,dy,dz ',1p3e20.12)
 8280 format ('P1 to P2 distance ',1pe20.12)
 8230 format ('Norm2 at P1 (xyz) ',1p3e20.12)
 8240 format ('Norm1 at P2 (xyz) ',1p3e20.12)
 8210 format ('Side, Surf 2 at P1',1pe20.12,2x,a16)
 8220 format ('Side, Surf 1 at P2',1pe20.12,2x,a16)

      write (   6, 8250) tax, tay, taz
      write (iout, 8250) tax, tay, taz
      write (   6, 8260) tbx, tby, tbz
      write (iout, 8260) tbx, tby, tbz
      write (   6, 8270) dx, dy, dz
      write (iout, 8270) dx, dy, dz
      write (   6, 8280) dist12
      write (iout, 8280) dist12
      write (   6, 8230) bnx, bny, bnz
      write (iout, 8230) bnx, bny, bnz
      write (   6, 8240) anx, any, anz
      write (iout, 8240) anx, any, anz
      write (   6, 8210) dside2, aside2
      write (iout, 8210) dside2, aside2
      write (   6, 8220) dside1, aside1
      write (iout, 8220) dside1, aside1

      call plotpt (tax, tay, taz, nerrg)
      call plotpt (tbx, tby, tbz, nerrg)

      pdist   = dist12

  999 continue

      return

c.... End of subroutine intquad.      (+1 line)
      end
