      subroutine line

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  line     Updated     2005 April 5 17:45.
c               line     Originated  1993 November 15 10:00
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create lines by specifying two points.
c
c     Logic:    Entry points:  line.
c line          Called by:  helpall, main.
c
c line          Calls:  aptsubr, incrsubs, lout, lstore, namechk, ptfind,
c               varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /ccline/ np              ! Index in table of points.
      common /ccline/ apointx1        ! Temporary name of a vertex point.
      character*24    apointx1        ! Temporary name of a vertex point.
      common /ccline/ apointx2        ! Temporary name of a vertex point.
      character*24    apointx2        ! Temporary name of a vertex point.

      common /ccline/ anamex          ! Temporary name of a line.
      character*24    anamex          ! Temporary name of a line.

c.... Input line:  line [help]

 8110 format (
     &  '  Command "line":  acceptable input lines include:' /
     &  '  "line [help]"  Display this message.' /
     &  '  "line all"     Display all lines.' /
     &  '  "line list [ALNAME1 ALNAME2 ALNAME3 ...]"' /
     &  '     Display all listed lines (default is all lines).' /
     &  '  "line ALNAME"   Display line ALNAME.' //
     &  '  "line ALNAME = PNAME1 PNAME2"' /
     &  '     Create line ALNAME from point PNAME1 to point PNAME2.' //
     &  '  "line LINE02 incr NUMLINE INC LINE01 [IP1 [IP2]]"' /
     &  '     Create a family of NUMLINE lines LINE02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by incrementing the names of the end points of the' /
     &  '     preceding line by the number of characters indicated' /
     &  '     by the integers IP1, IP2, starting from line LINE01.' //
     &  '  Note:  use +/- for ALNAME to incr/decr the last',
     &  ' line name:  ',a //
     &  '  Synonyms:  [increment, incr], [help, h], [line, ln, l].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110) alines
        write (iout, 8110) alines
        go to 999
      endif

c=======================================================================********

c.... Input line:  line all
c.... Input line:  line list

 8120 format (/ 'Number of lines = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then
        nlinex = 0
        call lout (1, nlines)
        do 120 n = 1, nlines
          if (aline(n) .eq. ' ') go to 120
          nlinex = nlinex + 1
  120   continue
        write (   6, 8120) nlinex
        write (iout, 8120) nlinex
        go to 999
      endif

c=======================================================================********

c.... Input line:  line list ALNAME1 ALNAME2 ALNAME3 ...

 8130 format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nlines        ! Loop over lines.
            call aptsubr (aline(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aline(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call lout (n, n)
            endif
  125     continue                    ! End of loop over lines.
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  line ALNAME

c.... See if user wants one line displayed.

      if (nword .eq. 2) then
        do 130 n = 1, nlines
          if (aword(2) .eq. aline(n)) then
            call lout (n, n)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

 8136 format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of line not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if last line name should be incremented and used.

 8128 format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last line can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = alines
        lword(2) = lalines
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, alines
          write (iout, 8128) ainput, ninput, alines
          go to 999
        endif
      endif

 8129 format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last line can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = alines
        lword(2) = lalines
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, alines
          write (iout, 8129) ainput, ninput, alines
          go to 999
        endif
      endif

 8905 format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of line too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  line LINE02 increment NUMLINE INC LINE01 [IP1 [IP2]]

      if ((aword(3) .eq. 'increment') .or.
     &    (aword(3) .eq. 'incr'))     then

        nerr   = 0

 8505   format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Wrong number of words in input line:  ',i3)

        if ((nword .lt. 6) .or. (nword .gt. 8)) then
          nerr = 1
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
        endif

 8520   format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMLINE is not a positive integer:  ',a)
 8525   format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMLINE is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numline = iword(4)
          if (numline .lt. 1) then
            nerr = 1
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
          endif
        else
          nerr = 1
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
        endif

 8510   format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            nerr = 1
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
          endif
        else
          nerr = 1
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
        endif

c....   Find the base line.

        do 530 nl = 1, nlines
          if (aword(6) .eq. aline(nl)) then
            nbase = nl
            call lout (nbase, nbase)
            go to 540
          endif
  530   continue
        nerr = 1
        write (   6, 8130) ainput, ninput, aword(6)
        write (iout, 8130) ainput, ninput, aword(6)

c....   Find the increments to the vertex point names.

  540   incpt1 = 0

        call varfind (7, nword)

 8560   format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is not an integer or integer variable:  ',a)

        if (mtype(7) .eq. 2) then
          incpt1 = iword(7)
        else
          nerr = 1
          write (   6, 8560) ainput, ninput, aword(7)
          write (iout, 8560) ainput, ninput, aword(7)
        endif

        incpt2 = 0

        if (nword .eq. 8) then
          if (mtype(8) .eq. 2) then
            incpt2 = iword(8)
          else
            nerr = 1
            write (   6, 8560) ainput, ninput, aword(nw)
            write (iout, 8560) ainput, ninput, aword(nw)
          endif
        endif

c....   See if the line names can be generated uniquely.

 8570   format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate',i5,' line names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numline - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8570) ainput, ninput, numline, aword(2)
          write (iout, 8570) ainput, ninput, numline, aword(2)
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) go to 999

c....   Create the new lines.

        anamex  = aword(2)            ! Initial line name.
        lanamex = lword(2)

        do 590 nll = 1, numline        ! Loop over the new lines.

c....     Find the index in the line list for the new line.

          aword(2) = anamex
          lword(2) = lanamex

          call lstore (nrep, n)

          if (n .eq. 0) go to 999

c....     Find the new names of the end points of the line.

          apointx1  = alipt1(nbase)
          lapointx1 = lalipt1(nbase)

          if (incpt1 .eq. 0) go to 580

          call incrsubs (incpt1, apointx1, 1, lapointx1, nerra)

 8580     format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Can not increment point name:  ',a)

          if (nerra .ne. 0) then
            write (   6, 8580) ainput, ninput, alipt1(nbase)
            write (iout, 8580) ainput, ninput, alipt1(nbase)
            go to 999
          endif

          apointx2  = alipt2(nbase)
          lapointx2 = lalipt2(nbase)

          if (incpt2 .eq. 0) go to 580

          call incrsubs (incpt2, apointx2, 1, lapointx2, nerra)

          if (nerra .ne. 0) then
            write (   6, 8580) ainput, ninput, alipt2(nbase)
            write (iout, 8580) ainput, ninput, alipt2(nbase)
            go to 999
          endif

  580     continue

c....     See if the new vertex points exist.

          aword(7) = apointx1
          lword(7) = lapointx1

          call ptfind (7, np)

          if (np .eq. 0) go to 999

          aword(8) = apointx2
          lword(8) = lapointx2

          call ptfind (8, np)

          if (np .eq. 0) go to 999

c....     Store and display the new line data.
    
          aline(n)   = anamex
          alines     = anamex
          lalines    = lanamex

          alipt1(n)  = apointx1
          lalipt1(n) = lapointx1
          alipt2(n)  = apointx2
          lalipt2(n) = lapointx2

          call lout (n, n)

          nbase     = n

c....     Find the next line name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            nerr = 1
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
          endif

  590   continue                      ! End of loop over the new lines.

        go to 999

      endif

c===============================================================================

c.... Find the index in the line array at which to store this line.
c....   Set nrep = 1 if replacing an existing line.

      call lstore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c=======================================================================********

c.... Input line:  line ALNAME = PNAME1 PNAME2

      if (aword(3) .eq. '=') then

        nerr = 0

        if (nword .ne. 5) then
          nerr = 1
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
        endif

c....   Find the points for this line.

        call ptfind (4, np)

        if (np .eq. 0) then
          nerr = 1
          go to 150
        endif

        alipt1(n)  = aword(4)
        lalipt1(n) = lword(4)

  150   call ptfind (5, np)

        if (np .eq. 0) then
          nerr = 1
          go to 170
        endif

        alipt2(n)  = aword(5)
        lalipt2(n) = lword(5)

  170   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            aline(n) = ' '
            if (n .eq. nlines) nlines = nlines - 1
          endif
          go to 999
        endif

c....   Print the line specifications.

        aline(n) = aword(2)
        alines   = aword(2)
        lalines  = lword(2)

        call lout (n, n)

        go to 999

      endif                           ! Created a line.

c=======================================================================********

 8900 format ('  line ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8900) ainput, ninput
      write (iout, 8900) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine line.         (+1 line)
      end
