      subroutine meshline

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshline Updated     2005 March 9 14:40.
c               meshline Originated  1995 September 15 17:45.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To linearly interpolate the mesh points in the mesh block,
c               based on the x, y, z coordinates of the mesh points at the
c               vertices of the mesh block.
c
c     Logic:    Entry points:  meshline.
c meshline      Called by:  mesh.
c
c meshline      Calls:  meshfind, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Initialize.

      nerr = 0

c.... Find the mesh point at each vertex of the mesh block.

      call meshfind (kbl1, lbl1, mbl1, np111)
      call meshfind (kbl1, lbl1, mbl2, np112)
      call meshfind (kbl1, lbl2, mbl1, np121)
      call meshfind (kbl1, lbl2, mbl2, np122)
      call meshfind (kbl2, lbl1, mbl1, np211)
      call meshfind (kbl2, lbl1, mbl2, np212)
      call meshfind (kbl2, lbl2, mbl1, np221)
      call meshfind (kbl2, lbl2, mbl2, np222)

c.... See if each mesh block vertex is assigned to a mesh point.

 8110 format ('  meshline ERROR.  Reading ',a40,' line ',i5,'.' /
     &  'Mesh indices           ',3i6,7x,'! UNASSIGNED')

      if (np111 .le. 0) then
        nerr = 1
        klm = kbl1 + (lbl1 - 1) * kmax + (mbl1 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl1, lbl1, mbl1
        write (iout, 8110) ainput, ninput, kbl1, lbl1, mbl1
      endif

      if (np112 .le. 0) then
        nerr = 1
        klm = kbl1 + (lbl1 - 1) * kmax + (mbl2 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl1, lbl1, mbl2
        write (iout, 8110) ainput, ninput, kbl1, lbl1, mbl2
      endif

      if (np121 .le. 0) then
        nerr = 1
        klm = kbl1 + (lbl2 - 1) * kmax + (mbl1 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl1, lbl2, mbl1
        write (iout, 8110) ainput, ninput, kbl1, lbl2, mbl1
      endif

      if (np122 .le. 0) then
        nerr = 1
        klm = kbl1 + (lbl2 - 1) * kmax + (mbl2 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl1, lbl2, mbl2
        write (iout, 8110) ainput, ninput, kbl1, lbl2, mbl2
      endif

      if (np211 .le. 0) then
        nerr = 1
        klm = kbl2 + (lbl1 - 1) * kmax + (mbl1 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl2, lbl1, mbl1
        write (iout, 8110) ainput, ninput, kbl2, lbl1, mbl1
      endif

      if (np212 .le. 0) then
        nerr = 1
        klm = kbl2 + (lbl1 - 1) * kmax + (mbl2 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl2, lbl1, mbl2
        write (iout, 8110) ainput, ninput, kbl2, lbl1, mbl2
      endif

      if (np221 .le. 0) then
        nerr = 1
        klm = kbl2 + (lbl2 - 1) * kmax + (mbl1 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl2, lbl2, mbl1
        write (iout, 8110) ainput, ninput, kbl2, lbl2, mbl1
      endif

      if (np222 .le. 0) then
        nerr = 1
        klm = kbl2 + (lbl2 - 1) * kmax + (mbl2 - 1) * kmax * lmax
        write (   6, 8110) ainput, ninput, kbl2, lbl2, mbl2
        write (iout, 8110) ainput, ninput, kbl2, lbl2, mbl2
      endif

      if (nerr .ne. 0) go to 999

c.... Interpolate the coordinates of all mesh points in the mesh block.

      delk = kbl2 - kbl1
      if (delk .eq. 0) delk = 1.0
      dell = lbl2 - lbl1
      if (dell .eq. 0) dell = 1.0
      delm = mbl2 - mbl1
      if (delm .eq. 0) delm = 1.0

      npint = 0

      do 120 np = 1, npoints          ! Loop over points.

        if  (apoint(np) .eq. ' ')   go to 120
        if ((kpoint(np) .lt. kbl1) .or.
     &      (kpoint(np) .gt. kbl2)) go to 120
        if ((lpoint(np) .lt. lbl1) .or.
     &      (lpoint(np) .gt. lbl2)) go to 120
        if ((mpoint(np) .lt. mbl1) .or.
     &      (mpoint(np) .gt. mbl2)) go to 120

        npint = npint + 1

        call pout (np, np, 'minimum')

c....   Find the fractional distance of each mesh index across the mesh block.

        fk = (kpoint(np) - kbl1) / delk  ! Fractional distance from k to kbl1.
        fl = (lpoint(np) - lbl1) / dell  ! Fractional distance from l to lbl1.
        fm = (mpoint(np) - mbl1) / delm  ! Fractional distance from m to mbl1.

        bk = 1.0 - fk                    ! Fractional distance from kbl2 to k.
        bl = 1.0 - fl                    ! Fractional distance from lbl2 to l.
        bm = 1.0 - fm                    ! Fractional distance from mbl2 to m.

c....   Interpolate the x, y, z coordinates of this mesh point.

        x(np) = bk * bl * bm * x(np111) + bk * bl * fm * x(np112) +
     &          bk * fl * bm * x(np121) + bk * fl * fm * x(np122) +
     &          fk * bl * bm * x(np211) + fk * bl * fm * x(np212) +
     &          fk * fl * bm * x(np221) + fk * fl * fm * x(np222)

        y(np) = bk * bl * bm * y(np111) + bk * bl * fm * y(np112) +
     &          bk * fl * bm * y(np121) + bk * fl * fm * y(np122) +
     &          fk * bl * bm * y(np211) + fk * bl * fm * y(np212) +
     &          fk * fl * bm * y(np221) + fk * fl * fm * y(np222)

        z(np) = bk * bl * bm * z(np111) + bk * bl * fm * z(np112) +
     &          bk * fl * bm * z(np121) + bk * fl * fm * z(np122) +
     &          fk * bl * bm * z(np211) + fk * bl * fm * z(np212) +
     &          fk * fl * bm * z(np221) + fk * fl * fm * z(np222)

        call pout (np, np, 'minimum')

  120 continue                        ! End of loop over points.

 8120 format ('  Number of mesh points interpolated =',i7)

      write (   6, 8120) npint
      write (iout, 8120) npint

  999 return

c.... End of subroutine meshline.     (+1 line)
      end
