      subroutine meshrlxb

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshrlxb Updated     2005 March 9 14:40.
c               meshrlxb Originated  1995 October 8 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the x, y and z coordinates of all mesh points in the
c               mesh block, except for exterior mesh block points, by a
c               relaxation method, which moves each mesh point to the average
c               coordinates of its nearest neighbors.  Any mesh block point
c               with exactly one neighboring mesh block point in the k, l or m
c               direction is an exterior mesh block point.
c
c     Logic:    Entry points:  meshrlxb.
c meshrlxb      Called by:  mesh.
c
c meshrlxb      Calls:  meshfind, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Initialize.

      nerr  = 0

c.... Test for presence of interior mesh points in the mesh block.

 8105 format ('  meshrlxb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh block has no interior mesh points to relax.')

      if (kbl2 .eq. kbl1) then
        kbeg = kbl1
        kend = kbl2
      elseif (kbl2 .eq. (kbl1 + 1)) then
        nerr = 1
        write (   6, 8105) ainput, ninput
        write (iout, 8105) ainput, ninput
      else
        kbeg = kbl1 + 1
        kend = kbl2 - 1
      endif

      if (lbl2 .eq. lbl1) then
        lbeg = lbl1
        lend = lbl2
      elseif (lbl2 .eq. (lbl1 + 1)) then
        nerr = 1
        write (   6, 8105) ainput, ninput
        write (iout, 8105) ainput, ninput
      else
        lbeg = lbl1 + 1
        lend = lbl2 - 1
      endif

      if (mbl2 .eq. mbl1) then
        mbeg = mbl1
        mend = mbl2
      elseif (mbl2 .eq. (mbl1 + 1)) then
        nerr = 1
        write (   6, 8105) ainput, ninput
        write (iout, 8105) ainput, ninput
      else
        mbeg = mbl1 + 1
        mend = mbl2 - 1
      endif

c.... Test for mesh indices in the mesh block not assigned to mesh points.

 8110 format ('  meshrlxb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  'Mesh indices           ',3i6,7x,'! UNASSIGNED')

      do 130 m = mbl1, mbl2           ! Loop over mesh block.
        do 120 l = lbl1, lbl2
          do 110 k = kbl1, kbl2

c....       Find any mesh point assigned these mesh indices.

            call meshfind (k, l, m, np)
    
            if (np .le. 0) then
              nerr = 1
              klm = k + (l - 1) * kmax + (m - 1) * kmax * lmax
              write (   6, 8110) ainput, ninput, k, l, m
              write (iout, 8110) ainput, ninput, k, l, m
            endif

  110     continue
  120   continue
  130 continue                        ! End of loop over mesh block.

      if (nerr .ne. 0) go to 999

c.... Relax mesh points in the interior of the mesh block.

      kdone = 0
      niter = 0

c.... Return here for another iteration to relax mesh point coordinates.

  140 nch   = 0

      do 170 m = mbeg, mend           ! Loop over interior mesh block points.
        do 160 l = lbeg, lend
          do 150 k = kbeg, kend

c....       Sum the non-vacuum coordinates of any pairs of nearest neighbors.

            ntot = 0
            xtot = 0.0
            ytot = 0.0
            ztot = 0.0

            k1 = k - 1
            k2 = k + 1

            call meshfind (k1, l, m, npk1)
            call meshfind (k2, l, m, npk2)

            if ((npk1 .gt. 0)          .and.
     &          (npk2 .gt. 0)) then
              if( x(npk1) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npk1)
                ytot = ytot + y(npk1)
                ztot = ztot + z(npk1)
              endif
              if( x(npk2) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npk2)
                ytot = ytot + y(npk2)
                ztot = ztot + z(npk2)
              endif
            endif                     ! Mesh point has two neighbors.

            l1 = l - 1
            l2 = l + 1

            call meshfind (k, l1, m, npl1)
            call meshfind (k, l2, m, npl2)

            if ((npl1 .gt. 0)          .and.
     &          (npl2 .gt. 0)) then
              if( x(npl1) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npl1)
                ytot = ytot + y(npl1)
                ztot = ztot + z(npl1)
              endif
              if( x(npl2) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npl2)
                ytot = ytot + y(npl2)
                ztot = ztot + z(npl2)
              endif
            endif                     ! Mesh point has two neighbors.

            m1 = m - 1
            m2 = m + 1

            call meshfind (k, l, m1, npm1)
            call meshfind (k, l, m2, npm2)

            if ((npm1 .gt. 0)          .and.
     &          (npm2 .gt. 0)) then
              if( x(npm1) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npm1)
                ytot = ytot + y(npm1)
                ztot = ztot + z(npm1)
              endif
              if( x(npm2) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npm2)
                ytot = ytot + y(npm2)
                ztot = ztot + z(npm2)
              endif
            endif                     ! Mesh point has two neighbors.

c....       Change the coordinates to the average of the pairs of nearest
c....         neighbors and test for convergence.

            if (ntot .gt. 0) then     ! Recalculate coordinates.

              call meshfind (k, l, m, np)

              if (kdone .eq. 0) then  ! Not converged.
  
                xold  = x(np)
                yold  = y(np)
                zold  = z(np)

                x(np) = xtot / ntot
                y(np) = ytot / ntot
                z(np) = ztot / ntot

                if ((abs (x(np) - xold) .gt. tol * abs (xold))  .or.
     &              (abs (y(np) - yold) .gt. tol * abs (yold))  .or.
     &              (abs (z(np) - zold) .gt. tol * abs (zold))) then
                  nch = nch + 1
                endif

              else                    ! Relaxation method converged.

                call pout (np, np, 'minimum')

              endif                   ! Tested for convergence.

            endif                     ! Recalculated coordinates.

  150     continue
  160   continue
  170 continue                 ! End of loop over interior mesh block points.

      if (kdone .eq. 0) then
        if (nch .eq. 0) kdone = 1
        niter = niter + 1
        go to 140
      endif

c.... Relaxation method converged.

 8410 format ('  Number of mesh points relaxed =',i7)
 8420 format ('  Number of iterations          =',i7)

      nrlax = (kend - kbeg + 1) * (lend - lbeg + 1) * (mend - mbeg + 1)

      write (   6, 8410) nrlax
      write (iout, 8410) nrlax
      write (   6, 8420) niter
      write (iout, 8420) niter

  999 return

c.... End of subroutine meshrlxb.     (+1 line)
      end
