      subroutine plane

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  plane    Updated     2005 April 5 17:45.
c               plane    Originated  1993 November 17 18:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create planes.
c
c     Logic:    Entry points:  plane.
c plane         Called by:  helpall, main.
c
c plane         Calls:  aptcsyv, aptpfit, aptptpl, aptqnor, aptrois, aptrota,
c               aptsubr, aptvdis, aptvdot, aptvusz, dkout, incrsubs, namechk,
c               plout, ptfind, qumove, quoper, quout, qustore, trout, varfind,
c               vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lplane/ rotm(3,3)       ! Rotation tensor.

      character*8  amount             ! Amount of output from quout.
      character*24 anamex             ! Temporary name of a plane.
      character*8  ans                ! User's response.

c.... Input line:  plane [help]

 8110 format (
     &  '  Command "plane":  acceptable input lines include:' /
     &  '  "plane [help]"  Display this message.' /
     &  '  "plane all"     Display all planes.' /
     &  '  "plane list [PLNAME1 PLNAME2 PLNAME3 ...]"' /
     &  '     Display all listed planes (default is all',
     &  ' planes).' /
     &  '  "plane PLNAME"  Display plane PLNAME.' //
     &  '  "plane PLNAME = U V W DU DV DW"' /
     &  '     Create plane PLNAME through point (U, V, W),' /
     &  '     with normal vector (DU, DV, DW).' /
     &  '  "plane PLNAME vector PNAME VNORM"' /
     &  '     Create plane PLNAME through point PNAME',
     &  ' with normal vector VNORM.' /
     &  '  "plane PLNAME fit PNAME1 PNAME2 PNAME3"' /
     &  '     Create plane PLNAME through the 3 points',
     &  ' PNAME1, PNAME2 and PNAME3.' /
     &  '  "plane PLNAME triangle TRNAME"' /
     &  '     Create plane PLNAME through triangle TRNAME.' )

 8112 format (
     &  '  "plane PLNAME disk DKNAME"' /
     &  '     Create plane PLNAME through annular disk',
     &  ' DKNAME.' /
     &  '  "plane PLNAME tangent PNAME QNAME"' /
     &  '     Create plane PLNAME, tangent at point PNAME to',
     &  ' quadric surface' /
     &  '     family QNAME.' / 
     &  '  "plane PLNAME outline PNAME QNAME"' /
     &  '     Create any plane PLNAME, containing the outline of',
     &  ' quadric surface QNAME,' /
     &  '     as seen from point PNAME.' /
     &  '  "plane PLNAME project VNAME QNAME"' /
     &  '     Create any plane PLNAME, containing the curve on',
     &  ' quadric surface QNAME,' /
     &  '     where the normal vector is perpendicular to vector'
     &  ' VNAME.' //
     &  '  Type <return> for more help.' )

 8113 format (a)

 8114 format (/
     &  '  "plane PLNAME parallel NUMPL INC PNAME1 PNAME2"' /
     &  '     Create a family of NUMPL parallel planes PLNAME, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     equally spaced along the normal axis from' /
     &  '     point PNAME1 to point PNAME2.' //
     &  '  "plane PL02 move NUMPL INC PL01 VMOVE"' /
     &  '     Create a family of NUMPL parallel planes',
     &  ' PL02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     translated by intervals of vector VMOVE, starting from',
     &  ' plane PL01.' )

 8115 format (/
     &  '  "plane PL02 rotate NUMPL INC PL01 PAXIS VAXIS DANGLE"' /
     &  '     Create a family of NUMPL planes PL02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     rotated around an axis through point PAXIS' /
     &  '     in the direction of vector VAXIS, by equal angular',
     &  ' increments DANGLE' /
     &  '     (counterclockwise, with the axis pointed at the',
     &  ' observer),' /
     &  '     starting from plane PL01.' //
     &  '  "plane PL02 scale NUMPL INC PL01 OPNAME PINV"' /
     &  '     Create a family of NUMPL planes PL02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding plane with operator OPNAME' /
     &  '     and invariant point PINV, starting from plane PL01.' )

 8116 format (/
     &  '  Note:  use +/- for PLNAME or PL02 to incr/decr the last',
     &  ' plane name:  ',a //
     &  '  Synonyms:  [disk, disc, dk], [help, h], [parallel, par],',
     &  ' [plane, pl],' /
     &  '    [project, proj], [rotate, rot], [scale, sc],'
     &  ' [tangent, tan],' /
     &  '    [triangle, tri], [vector, vect, v].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8113) ans
        endif
        write (   6, 8114)
        write (iout, 8114)
        write (   6, 8115)
        write (iout, 8115)
        write (   6, 8116) aplanes
        write (iout, 8116) aplanes
        go to 999
      endif

c===============================================================================

c.... Input line:  plane all
c.... Input line:  plane list

 8120 format (/ 'Number of planes = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nplanex = 0
        do 120 n = 1, nquads
          if (aquad(n) .eq. ' ') go to 120
          if (aqtype(n) .ne. aqtypes(0)) go to 120
          call plout (n, n)
          nplanex = nplanex + 1
  120   continue
        write (   6, 8120) nplanex
        write (iout, 8120) nplanex
        go to 999
      endif

c===============================================================================

c.... Input line:  plane list PLNAME1 PLNAME2 PLNAME3 ...

 8130 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named plane does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            if (aqtype(n) .ne. aqtypes(0)) go to 125
            call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call plout (n, n)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  plane PLNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(0)) go to 130
          if (aword(2) .eq. aquad(n)) then
            call plout (n, n)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8136 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of plane not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if last plane name should be incremented and used.

 8128 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last plane can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = aplanes
        lword(2) = laplanes
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, aplanes
          write (iout, 8128) ainput, ninput, aplanes
          go to 999
        endif
      endif

 8129 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last plane can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = aplanes
        lword(2) = laplanes
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, aplanes
          write (iout, 8129) ainput, ninput, aplanes
          go to 999
        endif
      endif

 8905 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of plane too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  plane PL02 move NUMPL INC PL01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(3) .eq. 'trans'))    then

        call qumove

        go to 999

      endif                           ! Created translated plane(s).

c===============================================================================

c.... Input line:  plane PLNAME parallel NUMPL INC PNAME1 PNAME2

      if ((aword(3) .eq. 'parallel') .or.
     &    (aword(3) .eq. 'par'))     then

 8505   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 7 words in input line:  ',i3)

        if (nword .ne. 7) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          go to 999
        endif

 8520   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMPL is less than 2:  ',a)
 8525   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMPL is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numpl = iword(4)
          if (numpl .lt. 2) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            go to 999
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          go to 999
        endif

 8510   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
            go to 999
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          go to 999
        endif

c....   Find the first point.

        call ptfind (6, n1)

        if (n1 .eq. 0) go to 999

c....   Find the second point.

        call ptfind (7, np2)

        if (np2 .eq. 0) go to 999

c....   Find the normal vector of the planes.

        call aptvdis (x(n1), y(n1), z(n1), x(np2), y(np2), z(np2),
     &                1, tol, dx, dy, dz, vlength, nerra)

        if (vlength .eq. 0.0) then
          write (   6, 8240) ainput, ninput, aword(6)
          write (iout, 8240) ainput, ninput, aword(6)
          go to 999
        endif

c....   Find the distance between the planes.

        dist = vlength / (numpl - 1)

c....   See if the plane names can be generated uniquely.

 8570 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' plane names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numpl - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8570) ainput, ninput, numpl, aword(2)
          write (iout, 8570) ainput, ninput, numpl, aword(2)
          go to 999
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

c....   Find the data for the first plane.

        call aptvdot (x(n1), y(n1), z(n1), dx, dy, dz, 1, tol,
     &                vdot, nerra)

c....   Create the parallel planes.

        anamex  = aword(2)            ! Initial plane name.
        lanamex = lword(2)            ! Initial plane name length.

        do 530 npl = 1, numpl         ! Loop over the new planes.

c....     Find the index in the quadric surface list for the new plane.

          aword(2) = anamex
          lword(2) = lanamex

          call qustore (0, nrep, n)

          if (n .eq. 0) go to 999

          aplanes   = anamex
          laplanes  = lanamex
          aquads    = anamex
          laquads   = lanamex
          axisyms   = anamex
          laxisyms  = lanamex

          aquad(n)  = anamex
          laquad(n) = lanamex
          aqtype(n) = aqtypes(0)
          asym(n)   = 'axisymmetric'
    
          qc(n)     = -vdot / vlength - (npl - 1) * dist
          qx(n)     = dx / vlength
          qy(n)     = dy / vlength
          qz(n)     = dz / vlength
          qxy(n)    = 0.0
          qyz(n)    = 0.0
          qzx(n)    = 0.0
          qxx(n)    = 0.0
          qyy(n)    = 0.0
          qzz(n)    = 0.0

          call plout (n, n)

c....     Find the next plane name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  530   continue                      ! End of loop over the new planes.

        go to 999

      endif                           ! Created parallel plane(s).

c===============================================================================

c.... Input line:  plane PL02 rotate NUMPL INC PL01 PAXIS VAXIS DANGLE

      if ((aword(3) .eq. 'rotate') .or.
     &    (aword(3) .eq. 'rot'))   then

        nerr = 0

 8605   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 9 words in input line:  ',i3)

        if (nword .ne. 9) then
          write (   6, 8605) ainput, ninput, nword
          write (iout, 8605) ainput, ninput, nword
          nerr = 1
        endif

 8620   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMPL is not a positive integer:  ',a)
 8625   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMPL is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numpl = iword(4)
          if (numpl .lt. 1) then
            write (   6, 8620) ainput, ninput, aword(4)
            write (iout, 8620) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8625) ainput, ninput, aword(4)
          write (iout, 8625) ainput, ninput, aword(4)
          nerr = 1
        endif

 8610   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8615   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8610) ainput, ninput
            write (iout, 8610) ainput, ninput
            nerr = 1
          endif
        else
          write (   6, 8615) ainput, ninput, aword(5)
          write (iout, 8615) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the base plane.  Assign index nbase.

        do 630 nq = 1, nquads
          if (aqtype(nq) .ne. aqtypes(0)) go to 630
          if (aword(6) .eq. aquad(nq)) then
            nbase = nq
            call plout (nbase, nbase)
            go to 640
          endif
  630   continue
        write (   6, 8130) ainput, ninput, aword(6)
        write (iout, 8130) ainput, ninput, aword(6)
        nerr = 1

c....   Find the invariant point.

  640   call ptfind (7, np)

        if (np .eq. 0) nerr = 1

c....   Find the axis vector.

        call vectfind (8, nv)

        if (nv .eq. 0) nerr = 1

 8680   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Axis vector has zero length:  ',a)

        if (nv .gt. 0) then           ! Axis vector exists.
          if (vlen(nv) .eq. 0.0) then
            write (   6, 8680) ainput, ninput, aword(8)
            write (iout, 8680) ainput, ninput, aword(8)
            nerr = 1
          endif
        endif                         ! Axis vector exists.

c....   Find the angle of rotation.

 8690   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  DANGLE not integer or floating point value or',
     &    ' variable:  ',a)

        call varfind (9, 9)

        if ((mtype(9) .ne. 2) .and. (mtype(9) .ne. 3)) then
          write (   6, 8690) ainput, ninput, aword(9)
          write (iout, 8690) ainput, ninput, aword(9)
          nerr = 1
        endif

  700   angle = fword(9)

 8700   format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  DANGLE is zero.')

        if (angle .eq. 0.0) then
          write (   6, 8700) ainput, ninput
          write (iout, 8700) ainput, ninput
          nerr = 1
        endif

c....   Find the angle units.

        iunit = 0
        if (angunit .eq. 'radians') iunit = 1

c....   See if the plane names can be generated uniquely.

 8710 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' plane names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numpl - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8710) ainput, ninput, numpl, aword(2)
          write (iout, 8710) ainput, ninput, numpl, aword(2)
          nerr = 1
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) go to 999

c....   Initialize the quadric surface coefficients.

        ac  = qc(nbase)
        ax  = qx(nbase)
        ay  = qy(nbase)
        az  = qz(nbase)
        axy = qxy(nbase)
        ayz = qyz(nbase)
        azx = qzx(nbase)
        axx = qxx(nbase)
        ayy = qyy(nbase)
        azz = qzz(nbase)

c....   Find the rotation tensor.

        call aptrota (angle, iunit, xcom(nv), ycom(nv), zcom(nv),
     &                tol, rotm, nerra)

c....   Create the rotated planes.

        nerr    = 0
        anamex  = aword(2)            ! Initial plane name.
        lanamex = lword(2)            ! Initial plane name length.

        do 720 npl = 1, numpl         ! Loop over the new planes.

c....     Find the index in the quadric surface list for the new plane.

          aword(2) = anamex
          lword(2) = lanamex

          call qustore (0, nrep, n)

          if (n .eq. 0) go to 999

          aplanes   = anamex
          laplanes  = lanamex
          aquads    = anamex
          laquads   = lanamex
          axisyms   = anamex
          laxisyms  = lanamex

          aquad(n)  = anamex
          laquad(n) = lanamex
          aqtype(n) = aqtypes(0)
          asym(n)   = 'axisymmetric'

          call aptrois (rotm, 0, x(np), y(np), z(np), ac, ax, ay, az,
     &                  axy, ayz, azx, axx, ayy, azz, tol, nerra)
    
          qc(n)  = ac
          qx(n)  = ax
          qy(n)  = ay
          qz(n)  = az
          qxy(n) = axy
          qyz(n) = ayz
          qzx(n) = azx
          qxx(n) = axx
          qyy(n) = ayy
          qzz(n) = azz
    
          call plout (n, n)

c....     Find the next plane name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  720   continue                      ! End of loop over the new planes.

        go to 999

      endif                           ! Created rotated plane(s).

c===============================================================================

c.... Input line:  plane PL02 scale  NUMPL INC PL01 OPNAME PINV

      if ((aword(3) .eq. 'scale') .or.
     &    (aword(3) .eq. 'sc'))   then

        call quoper

        go to 999

      endif                           ! Created scaled plane(s).

c===============================================================================

c.... Find the index in the quadric surface array at which to store this plane.
c....   Returns nrep = 1 if replacing an existing quadric surface.

      call qustore (0, nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c===============================================================================

c.... Input line:  plane PLNAME fit PNAME1 PNAME2 PNAME3

      if (aword(3) .eq. 'fit') then

c.... Test for wrong number of words on "plane ... fit ..." line.

 8145 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 6 words on input line:  ', i3)

      if (nword .ne. 6) then
        write (   6, 8145) ainput, ninput, nword
        write (iout, 8145) ainput, ninput, nword
        go to 999
      endif

c.... Find the three points defining the plane.

      call ptfind (4, n1)

      if (n1 .eq. 0) then
        nerr = 1
      endif

      call ptfind (5, n2)

      if (n2 .eq. 0) then
        nerr = 1
      endif

      call ptfind (6, n3)

      if (n3 .eq. 0) then
        nerr = 1
      endif

 8190 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are not unique:  ' / a / a / a)

      if ((n1 .eq. n2)  .or.
     &    (n2 .eq. n3)  .or.
     &    (n3 .eq. n1)) then
        nerr = 1
        write (   6, 8190) ainput, ninput, aword(4), aword(5), aword(6)
        write (iout, 8190) ainput, ninput, aword(4), aword(5), aword(6)
      endif

c.... Find the coefficients of the equation of the plane.

      call aptpfit (x(n1), y(n1), z(n1), x(n2), y(n2), z(n2),
     &              x(n3), y(n3), z(n3), tol,
     &              ac, ax, ay, az, nerra)

 8200 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are coincident or colinear:  ' a / a / a)

      if (nerra .ne. 0) then
        nerr = 1
        write (   6, 8200) ainput, ninput, aword(4), aword(5), aword(6)
        write (iout, 8200) ainput, ninput, aword(4), aword(5), aword(6)
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the plane.

      aplanes   = aword(2)
      laplanes  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      aqtype(n) = aqtypes(0)
      asym(n)   = 'axisymmetric'

      qc(n)  = ac
      qx(n)  = ax
      qy(n)  = ay
      qz(n)  = az
      qxy(n) = 0.0
      qyz(n) = 0.0
      qzx(n) = 0.0
      qxx(n) = 0.0
      qyy(n) = 0.0
      qzz(n) = 0.0

      call plout (n, n)

      go to 999

      endif                           ! Specified plane by three points.

c===============================================================================

c.... Input line:  plane PLNAME vector PNAME VNORM

      if ((nword .eq. 5)             .and.
     &   ((aword(3) .eq. 'vector')   .or.
     &    (aword(3) .eq. 'vect')     .or.
     &    (aword(3) .eq. 'v')))      then

c.... Find the point and normal vector defining the plane.

      call ptfind (4, np)

      if (np .eq. 0) nerr = 1

      call vectfind (5, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 250
      endif

 8240 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Normal vector has zero length:  ',a)

      if (vlen(nv) .eq. 0.0) then
        nerr = 1
        write (   6, 8240) ainput, ninput, aword(5)
        write (iout, 8240) ainput, ninput, aword(5)
      endif

  250 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the dot product of the planar point and the normal vector.

      call aptvdot (x(np), y(np), z(np), xcom(nv), ycom(nv), zcom(nv),
     &              1, tol, vdot, nerra)

c.... Find the unit normal vector.

      call aptvusz (xcom(nv), ycom(nv), zcom(nv), tol,
     &              ux, uy, uz, vlength)

c.... Find the coefficients of the quadric equation for the plane.

      aplanes   = aword(2)
      laplanes  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      aqtype(n) = aqtypes(0)
      asym(n)   = 'axisymmetric'

      qc(n)     = -vdot / vlength
      qx(n)     = ux
      qy(n)     = uy
      qz(n)     = uz
      qxy(n)    = 0.0
      qyz(n)    = 0.0
      qzx(n)    = 0.0
      qxx(n)    = 0.0
      qyy(n)    = 0.0
      qzz(n)    = 0.0

      call plout (n, n)

      go to 999

      endif                           ! Specified point and normal vector.

c===============================================================================

c.... Input line:  plane PLNAME = U V W DU DV DW

      if ((nword .ge. 7)       .and.
     &    (nword .le. 9)       .and.
     &    (aword(3) .eq. '=')) then

c.... Find the coordinates of the point, and the components of the normal
c....   vector defining the plane.

c.... Default missing words to zero.

      do 320 nw = 4, 9
        if (nw .gt. nword) then
          mtype(nw) = 3
          fword(nw) = 0.0
        endif
  320 continue
      nword = 9

c.... Find the values of words 4-9.

 8330 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (4, 9)

      do 340 nw = 4, 9
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8330) ainput, ninput, aword(nw)
          write (iout, 8330) ainput, ninput, aword(nw)
          nerr = 1
        endif
  340 continue

c.... Find the point and normal vector.

      u  = fword(4)
      v  = fword(5)
      w  = fword(6)

      du = fword(7)
      dv = fword(8)
      dw = fword(9)

c.... Convert units, if needed.

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1
  
      if     (asys .eq. 'cyl') then
        call aptcsyv (1, 0, iunit, u, v, w, du, dv, dw, 1, tol,
     &                nerra)
        if (nerra .ne. 0) nerr = 1
      elseif (asys .eq. 'sph') then
        call aptcsyv (2, 0, iunit, u, v, w, du, dv, dw, 1, tol,
     &                nerra)
        if (nerra .ne. 0) nerr = 1
      endif

      call aptvusz (du, dv, dw, tol, dx, dy, dz, vlength)

 8340 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Normal vector has zero length.')

      if (vlength .eq. 0.0) then
        nerr = 1
        write (   6, 8340) ainput, ninput
        write (iout, 8340) ainput, ninput
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the dot product of the planar point and the normal vector.

      call aptvdot (u, v, w, du, dv, dw, 1, tol, vdot, nerra)

c.... Find the coefficients of the quadric equation for the plane.

      aplanes   = aword(2)
      laplanes  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      aqtype(n) = aqtypes(0)
      asym(n)   = 'axisymmetric'

      qc(n)  = -vdot / vlength
      qx(n)  = du / vlength
      qy(n)  = dv / vlength
      qz(n)  = dw / vlength
      qxy(n) = 0.0
      qyz(n) = 0.0
      qzx(n) = 0.0
      qxx(n) = 0.0
      qyy(n) = 0.0
      qzz(n) = 0.0

      call plout (n, n)

      go to 999

      endif                           ! Specified 3 coordinates, 3 normal comp.

c===============================================================================

c.... Input line:  plane PLNAME triangle TRNAME

c.... Find the triangle coplanar with the plane.

      if ((nword. eq. 4)             .and.
     &   ((aword(3) .eq. 'triangle') .or.
     &    (aword(3) .eq. 'tri')))    then

 8370 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named triangle does not exist:  ',a)
 8371 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Vertex point does not exist:  ',a)

      do 370 nt = 1, ntrigs
        if (aword(4) .eq. atrig(nt)) then
          n1 = nt

          do 371 np = 1, npoints
            if (apoint(np) .eq. atript1(n1)) then
              xtra = x(np)
              ytra = y(np)
              ztra = z(np)
              go to 372
            endif
  371     continue
          nerr = 1
          write (   6, 8371) ainput, ninput, atript1(n1)
          write (iout, 8371) ainput, ninput, atript1(n1)

  372     do 373 np = 1, npoints
            if (apoint(np) .eq. atript2(n1)) then
              xtrb = x(np)
              ytrb = y(np)
              ztrb = z(np)
              go to 374
            endif
  373     continue
          nerr = 1
          write (   6, 8371) ainput, ninput, atript2(n1)
          write (iout, 8371) ainput, ninput, atript2(n1)

  374     do 375 np = 1, npoints
            if (apoint(np) .eq. atript3(n1)) then
              xtrc = x(np)
              ytrc = y(np)
              ztrc = z(np)
              go to 376
            endif
  375     continue
          nerr = 1
          write (   6, 8371) ainput, ninput, atript3(n1)
          write (iout, 8371) ainput, ninput, atript3(n1)

  376     amount = 'maximum'
          call trout (n1, n1, amount)

          go to 380
        endif
  370 continue
      nerr = 1
      write (   6, 8370) ainput, ninput, aword(4)
      write (iout, 8370) ainput, ninput, aword(4)

  380 continue

      call aptpfit (xtra, ytra, ztra, xtrb, ytrb, ztrb,
     &              xtrc, ytrc, ztrc, tol,
     &              ac, ax, ay, az, nerra)

 8390 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Triangle is indeterminate:  ',a)

      if (nerra .ne. 0) then
        nerr = 1
        write (   6, 8390) ainput, ninput, aword(4)
        write (iout, 8390) ainput, ninput, aword(4)
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the plane.

      aplanes   = aword(2)
      laplanes  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      aqtype(n) = aqtypes(0)
      asym(n)   = 'axisymmetric'

      qc(n)     = ac
      qx(n)     = ax
      qy(n)     = ay
      qz(n)     = az
      qxy(n)    = 0.0
      qyz(n)    = 0.0
      qzx(n)    = 0.0
      qxx(n)    = 0.0
      qyy(n)    = 0.0
      qzz(n)    = 0.0

      call plout (n, n)

      go to 999

      endif                           ! Specified a triangle.

c===============================================================================

c.... Input line:  plane PLNAME disk DKNAME

      if ((nword. eq. 4)         .and.
     &   ((aword(3) .eq. 'disk') .or.
     &    (aword(3) .eq. 'disc') .or.
     &    (aword(3) .eq. 'dk'))) then

c.... Find the disk coplanar with the plane.

 8410 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named disk does not exist:  ',a)

      do 410 nd = 1, ndisks
        if (aword(4) .eq. adisk(nd)) then
          call dkout (nd, nd)
          go to 420
        endif
  410 continue
      nerr = 1
      write (   6, 8410) ainput, ninput, aword(4)
      write (iout, 8410) ainput, ninput, aword(4)

c.... Find the center point of the disk.

 8430 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Center point of disk does not exist:  ',a)

  420 do 430 np = 1, npoints
        if (acendk(nd) .eq. apoint(np)) then
          go to 440
        endif
  430 continue
      nerr = 1
      write (   6, 8430) ainput, ninput, acendk(nd)
      write (iout, 8430) ainput, ninput, acendk(nd)

c.... Find the length of the normal vector.

 8450 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Disk is indeterminate:  ',a)

  440 vlength = sqrt (xnormdk(nd)**2 + ynormdk(nd)**2 + znormdk(nd)**2)

      if (vlength .eq. 0.0) then
        nerr = 1
        write (   6, 8450) ainput, ninput, aword(4)
        write (iout, 8450) ainput, ninput, aword(4)
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the dot product of the center point and the normal vector.

      call aptvdot (x(np), y(np), z(np),
     &              xnormdk(nd), ynormdk(nd), znormdk(nd), 1, tol,
     &              vdot, nerra)

c.... Find the coefficients of the quadric equation for the plane.

      aplanes   = aword(2)
      laplanes  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      aqtype(n) = aqtypes(0)
      asym(n)   = 'axisymmetric'

      qc(n)  = -vdot / vlength
      qx(n)  = xnormdk(nd) / vlength
      qy(n)  = ynormdk(nd) / vlength
      qz(n)  = znormdk(nd) / vlength
      qxy(n) = 0.0
      qyz(n) = 0.0
      qzx(n) = 0.0
      qxx(n) = 0.0
      qyy(n) = 0.0
      qzz(n) = 0.0

      call plout (n, n)

      go to 999

      endif                           ! Specified a disk.

c===============================================================================

c.... Input line:  plane PLNAME tangent PNAME QNAME

      if ((nword .eq. 5)             .and.
     &   ((aword(3) .eq. 'tangent')  .or.
     &    (aword(3) .eq. 'tan')))    then

c....   Find the point.

        call ptfind (4, n1)
  
        if (n1 .eq. 0) then
          nerr = 1
          go to 550
        endif

c....   Find the quadric surface.

        do 540 nq = 1, nquads
          if (aword(5) .eq. aquad(nq)) then
            n2 = nq
            call quout (n2, n2, 'min', nerrq)
            go to 550
          endif
  540   continue
        nerr = 1
        write (   6, 8650) ainput, ninput, aword(5)
        write (iout, 8650) ainput, ninput, aword(5)

  550   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            aquad(n) = ' '
            if (n .eq. nquads) nquads = nquads - 1
          endif
          go to 999
        endif

c....   Find the normal vector of the family of quadric surfaces at the point.
 
        call aptqnor (x(n1), y(n1), z(n1), qc(n2),
     &                qx(n2), qy(n2), qz(n2), qxy(n2), qyz(n2), qzx(n2),
     &                qxx(n2), qyy(n2), qzz(n2), tol,
     &                qc(n), qx(n), qy(n), qz(n), slen, nerra)

        aquads    = aword(2)
        laquads   = lword(2)
        aplanes   = aword(2)
        laplanes  = lword(2)
        axisyms   = aword(2)
        laxisyms  = lword(2)

        aquad(n)  = aword(2)
        laquad(n) = lword(2)
        aqtype(n) = aqtypes(0)
        asym(n)   = 'axisymmetric'

        qxy(n)    = 0.0
        qyz(n)    = 0.0
        qzx(n)    = 0.0
        qxx(n)    = 0.0
        qyy(n)    = 0.0
        qzz(n)    = 0.0

        call plout (n, n)

        go to 999

      endif                           ! Plane tangent to quadric surface.

c===============================================================================

c.... Input line:  plane PLNAME outline PNAME QNAME

      if ((nword. eq. 5)             .and.
     &    (aword(3) .eq. 'outline')) then

c.... Find the point from which the quadric surface is viewed.

      nerr = 0

      call ptfind (4, np)

      if (np .eq. 0) then
        nerr = 1
      endif

c.... Find the quadric surface for which the outline plane is desired.

      do 650 nq = 1, nquads           ! Loop over quadric surfaces.
        if (aword(5) .ne. aquad(nq)) go to 650


        call quout (nq, nq, 'min', nerrq)

        go to 660
  650 continue                        ! End of loop over quadric surfaces.

 8650 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      nerr = 1
      write (   6, 8650) ainput, ninput, aword(5)
      write (iout, 8650) ainput, ninput, aword(5)

 660  if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the plane.

      qc(n)  = 2.0 * qc(nq) +
     &         qx(nq) * x(np) + qy(nq) * y(np) + qz(nq) * z(np)
      qx(n)  = qx(nq) +
     &         2.0 * qxx(nq) * x(np) + qxy(nq) * y(np) + qzx(nq) * z(np)
      qy(n)  = qy(nq) +
     &         qxy(nq) * x(np) + 2.0 * qyy(nq) * y(np) + qyz(nq) * z(np)
      qz(n)  = qz(nq) +
     &         qzx(nq) * x(np) + qyz(nq) * y(np) + 2.0 * qzz(nq) * z(np)

 8660 format ('No such plane exists.')

      if (qx(n)**2 + qy(n)**2 + qz(n)**2 .eq. 0.0) then
        write (   6, 8660)
        write (iout, 8660)
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

      aquads    = aword(2)
      laquads   = lword(2)
      aplanes   = aword(2)
      laplanes  = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      aqtype(n) = aqtypes(0)
      asym(n)   = 'axisymmetric'

      qxy(n)    = 0.0
      qyz(n)    = 0.0
      qzx(n)    = 0.0
      qxx(n)    = 0.0
      qyy(n)    = 0.0
      qzz(n)    = 0.0

      call plout (n, n)

c.... Find the point in the plane nearest the center of the quadric surface.

      call aptptpl (xcen(nq), ycen(nq), zcen(nq),
     &              xcen(n), ycen(n), zcen(n), qx(n), qy(n), qz(n),
     &              1, tol, dpmin, cx, cy, cz, itrun, nerra)

 8670 format ('The point in the plane nearest the center of the',
     &  ' quadric surface is' /
     &        '  the point (xyz) ',1p3e20.12)

      write (   6, 8670) cx, cy, cz
      write (iout, 8670) cx, cy, cz

      go to 999

      endif

c===============================================================================

c.... Input line:  plane PLNAME project VNAME QNAME

      if ((nword. eq. 5)            .and.
     &   ((aword(3) .eq. 'project') .or.
     &    (aword(3) .eq. 'proj')))  then

c.... Find the vector direction in which the quadric surface is viewed.

      nerr = 0

      call vectfind (4, nv)

      if (nv .eq. 0) then
        nerr = 1
      endif

        if (nv .gt. 0) then           ! Axis vector exists.
          if (vlen(nv) .eq. 0.0) then
            write (   6, 8680) ainput, ninput, aword(8)
            write (iout, 8680) ainput, ninput, aword(8)
            nerr = 1
          endif
        endif                         ! Axis vector exists.

c.... Find the quadric surface for which the outline plane is desired.

      do 730 nq = 1, nquads           ! Loop over quadric surfaces.
        if (aword(5) .ne. aquad(nq)) go to 730

        call quout (nq, nq, 'min', nerrq)

        go to 740
  730 continue                        ! End of loop over quadric surfaces.

      nerr = 1
      write (   6, 8650) ainput, ninput, aword(5)
      write (iout, 8650) ainput, ninput, aword(5)

  740 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the plane passing
c....   through the curve on quadric surface QNAME where the normal vector
c....   is perpendicular to vector VNAME.

      qc(n) =       qx (nq) * xcom(nv) +       qy (nq) * ycom(nv) +
     &              qz (nq) * zcom(nv)
      qx(n) = 2.0 * qxx(nq) * xcom(nv) +       qxy(nq) * ycom(nv) +
     &              qzx(nq) * zcom(nv)
      qy(n) =       qxy(nq) * xcom(nv) + 2.0 * qyy(nq) * ycom(nv) +
     &              qyz(nq) * zcom(nv)
      qz(n) =       qzx(nq) * xcom(nv) +       qyz(nq) * ycom(nv) +
     &        2.0 * qzz(nq) * zcom(nv)

      if (qx(n)**2 + qy(n)**2 + qz(n)**2 .eq. 0.0) then
        write (   6, 8660)
        write (iout, 8660)
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

      aplanes   = aword(2)
      laplanes  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      aqtype(n) = aqtypes(0)
      asym(n)   = 'axisymmetric'
      qxy(n)    = 0.0
      qyz(n)    = 0.0
      qzx(n)    = 0.0
      qxx(n)    = 0.0
      qyy(n)    = 0.0
      qzz(n)    = 0.0

      call plout (n, n)

c.... Find the point in the plane nearest the center of the quadric surface.

      call aptptpl (xcen(nq), ycen(nq), zcen(nq),
     &              xcen(n), ycen(n), zcen(n), qx(n), qy(n), qz(n),
     &              1, tol, dpmin, cx, cy, cz, itrun, nerra)

      write (   6, 8670) cx, cy, cz
      write (iout, 8670) cx, cy, cz

      go to 999

      endif

c===============================================================================

 8135 format ('  plane ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8135) ainput, ninput
      write (iout, 8135) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine plane.        (+1 line)
      end
