      subroutine plotmap

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  plotmap  Updated     1996 February 14 13:50.
c               plotmap  Originated  1996 February 7 13:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To map the plot point characters into the 2-D plot array,
c               and display it.
c
c     Logic:    Entry points:  plotmap.
c plotmap       Called by:  plotx.
c
c plotmap       Calls:  markget, plotvar.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Start a new page in the output file.

      write (iout, "('')")

c.... Display the plot title.

 8110 format ('Plot title:' / 2x,a)
 8115 format ('Plot title:                    (blank)')

      call markget (aplmarkt, nmark)

      if (nmark .gt. 0) then
        write (   6, 8110) amarkt(nmark)
        write (iout, 8110) amarkt(nmark)
      else
        write (   6, 8115)
        write (iout, 8115)
      endif

c.... Find the ranges of the plot points.

      spthmin =  1.e99
      spthmax = -1.e99
      sptvmin =  1.e99
      sptvmax = -1.e99

      do 120 np = 1, nplpts            ! Loop over plot points.
        if (aplotm(np) .eq. ' ') go to 120

        call plotvar (xplot(np), yplot(np), zplot(np),
     &                spth, sptv, nerrg)

        if (spth .lt. spthmin) spthmin = spth
        if (spth .gt. spthmax) spthmax = spth
        if (sptv .lt. sptvmin) sptvmin = sptv
        if (sptv .gt. sptvmax) sptvmax = sptv

  120 continue                        ! End of loop over plot points.

c.... Display the horizontal axis variable and label.

 8130 format (/ 'Horizontal axis label:' / 2x,a)
 8135 format (/ 'Horizontal axis label:         (blank)')
 8138 format ('Horizontal axis coordinate:    ',a)

      call markget (aplmarkh, nmark)

      if (nmark .gt. 0) then
        write (   6, 8130) amarkt(nmark)
        write (iout, 8130) amarkt(nmark)
      else
        write (   6, 8135)
        write (iout, 8135)
      endif

      write (   6, 8138) axish
      write (iout, 8138) axish

c.... Find and display the range of the horizontal axis.

 8140 format ('Horizontal axis point range: ',1p2e20.12)
 8145 format ('Horizontal axis limits:      ',1p2e20.12)

      write (   6, 8140) spthmin, spthmax
      write (iout, 8140) spthmin, spthmax

      if (nplimh .eq. 0) then
        splhmin = spthmin
        splhmax = spthmax
      else
        write (   6, 8145) splhmin, splhmax
        write (iout, 8145) splhmin, splhmax
      endif

c.... Find and display the horizontal axis center.

 8150 format ('Horizontal center:   :       ',1pe20.12)
 8155 format ('Horizontal center (input):   ',1pe20.12)

      if (nplcen .eq. 0) then
        splhcen = 0.5 * (splhmin + splhmax)
      endif

      if (nplcen .eq. 0) then
        write (   6, 8150) splhcen
        write (iout, 8150) splhcen
      else
        write (   6, 8155) splhcen
        write (iout, 8155) splhcen
      endif

c.... Find the maximum horizontal distance from the center.

      delh = amax1 (splhcen - splhmin, splhmax - splhcen)

c.... Display any horizontal zoom factor.

 8158 format ('Horizontal axis zoom factor: ',1pe20.12)

      if (zoomplh .ne. 1.0) then
        write (   6, 8158) zoomplh
        write (iout, 8158) zoomplh
      endif

c.... Find and display the centered and zoomed horizontal axis limits.

      shmin = splhcen - delh / zoomplh
      shmax = splhcen + delh / zoomplh

      if ((nplcen .ne. 0) .or.
     &    (zoomplh .ne. 1.0)) then
        write (   6, 8145) shmin, shmax
        write (iout, 8145) shmin, shmax
      endif

c.... Display the vertical axis variable and label.

 8230 format (/ 'Vertical   axis label:' / 2x,a)
 8235 format (/ 'Vertical   axis label:         (blank)')
 8238 format ('Vertical   axis coordinate:    ',a)

      call markget (aplmarkv, nmark)

      if (nmark .gt. 0) then
        write (   6, 8230) amarkt(nmark)
        write (iout, 8230) amarkt(nmark)
      else
        write (   6, 8235)
        write (iout, 8235)
      endif

      write (   6, 8238) axisv
      write (iout, 8238) axisv

c.... Find and display the range of the vertical axis.

 8240 format ('Vertical   axis point range: ',1p2e20.12)
 8245 format ('Vertical   axis limits:      ',1p2e20.12)

      write (   6, 8240) sptvmin, sptvmax
      write (iout, 8240) sptvmin, sptvmax

      if (nplimv .eq. 0) then
        splvmin = sptvmin
        splvmax = sptvmax
      else
        write (   6, 8245) splvmin, splvmax
        write (iout, 8245) splvmin, splvmax
      endif

c.... Find and display the vertical axis center.

 8250 format ('Vertical   center:   :       ',1pe20.12)
 8255 format ('Vertical   center (input):   ',1pe20.12)

      if (nplcen .eq. 0) then
        splvcen = 0.5 * (splvmin + splvmax)
      endif

      if (nplcen .eq. 0) then
        write (   6, 8250) splvcen
        write (iout, 8250) splvcen
      else
        write (   6, 8255) splvcen
        write (iout, 8255) splvcen
      endif

c.... Find the maximum vertical distance from the center.

      delv = amax1 (splvcen - splvmin, splvmax - splvcen)

c.... Display any vertical axis zoom factor.

 8258 format ('Vertical   axis zoom factor: ',1pe20.12)

      if (zoomplv .ne. 1.0) then
        write (   6, 8258) zoomplv
        write (iout, 8258) zoomplv
      endif

c.... Find and display the centered and zoomed vertical axis limits.

      svmin = splvcen - delv / zoomplv
      svmax = splvcen + delv / zoomplv

      if ((nplcen .ne. 0) .or.
     &    (zoomplv .ne. 1.0)) then
        write (   6, 8245) svmin, svmax
        write (iout, 8245) svmin, svmax
      endif

c.... Display the number of plot points.

 8260 format (/ 'Number of plot points:       ',i5 )

      write (   6, 8260) nplpts
      write (iout, 8260) nplpts

c.... See if the axis limits are good.

 8270 format ('  plotmap ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Horizontal or vertical axis maximum < minimum.')

      nerr = 0

      if ((shmax .lt. shmin)  .or.
     &    (svmax .lt. svmin)) then
        write (   6, 8270) ainput, ninput
        write (iout, 8270) ainput, ninput
        nerr = 1
      endif

c.... See if all plot points are outside the axis limits.

 8280 format ('  plotmap ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  All plot points are outside the axis limits.')

      if ((spthmin .gt. shmax)  .or.
     &    (spthmax .lt. shmin)  .or.
     &    (sptvmin .gt. svmax)  .or.
     &    (sptvmax .lt. svmin)) then
        write (   6, 8280) ainput, ninput
        write (iout, 8280) ainput, ninput
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c.... Clear the plot array.

      do 320 nc = 1, nplcols + 2
        do 310 nl = 1, nplines + 2
          aplbuf(nc,nl) = ' '
  310   continue
  320 continue

c.... Outline the plot array with vertical and horizontal dashes.

      do 330 nc = 2, nplcols + 1
        aplbuf(nc,        1) = '|'
        aplbuf(nc,nplines+2) = '|'
  330 continue

      do 340 nl = 2, nplines + 1
        aplbuf(1,        nl) = '-'
        aplbuf(nplcols+2,nl) = '-'
  340 continue

c.... Map the plot points onto the plot array.

      shdiff = shmax - shmin
      shincr = shdiff / (nplcols - 1)

      svdiff = svmax - svmin
      svincr = svdiff / (nplines - 1)

      nplptx = 0

      do 360 np = 1, nplpts
        if (aplotm(np) .eq. ' ') go to 360

        call plotvar (xplot(np), yplot(np), zplot(np),
     &                spth, sptv, nerrg)

        if (shdiff .gt. 0.0) then
          ncol = 2.5 + ((spth - shmin) / shdiff) * (nplcols - 1)
        elseif (spth .eq. shmin) then
          ncol = (3 + nplcols) / 2
        else
          go to 360
        endif

 
        if (svdiff .gt. 0.0) then
          nlin = 2.5 + ((sptv - svmin) / svdiff) * (nplines - 1)
        elseif (sptv .eq. svmin) then
          nlin = (3 + nplines) / 2
        else
          go to 360
        endif

        if ((ncol .lt. 2) .or. (ncol .gt. (nplcols + 1))) go to 360
        if ((nlin .lt. 2) .or. (nlin .gt. (nplines + 1))) go to 360

        nplptx = nplptx + 1

        aplbuf(ncol,nlin) = aplotm(np)

  360 continue

c.... See if any plot points are within the axis limits.

 8370 format ('  plotmap ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No plot points are within the axis limits.')

      if (nplptx .le. 0) then
        write (   6, 8370) ainput, ninput
        write (iout, 8370) ainput, ninput
        go to 999
      endif

c.... Display the plot array.

 8410 format (168a1)

      write (iout, "('')")

      do 410 nl = nplines + 2, 1, -1
        write (   6, 8410) (aplbuf(nc,nl), nc = 1, nplcols + 2)
        write (iout, 8410) (aplbuf(nc,nl), nc = 1, nplcols + 2)
  410 continue

 8420 format ('Horizontal axis limits:      ',
     &  1pe13.5,' (',1pe13.5,') ',1pe13.5)
 8430 format ('Vertical   axis limits:      ',
     &  1pe13.5,' (',1pe13.5,') ',1pe13.5)

      write (   6, 8420) shmin, shincr, shmax
      write (iout, 8420) shmin, shincr, shmax
      write (   6, 8430) svmin, svincr, svmax
      write (iout, 8430) svmin, svincr, svmax

      write (iout, "('')")

  999 return

c.... End of subroutine plotmap.      (+1 line)
      end
