      subroutine plotvar (xpt, ypt, zpt, spth, sptv, nerrg)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  plotvar  Updated     1996 February 2 16:00.
c               plotvar  Originated  1996 February 2 16:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To convert the coordinates of a point from (x, y, z) to the
c               coordinates of the horizontal and vertical plot axes.
c
c     Logic:    Entry points:  plotvar.
c plotvar       Called by:  plotmap, plotx.
c
c plotvar       Calls:  ptcoord.
c
c     Biblio:   ?
c
c     Glossary:
c
c     nerrg     Output   0 if no error, 1 if the point coordinates can not be
c                          converted.
c
c     spth      Output   Horizontal axis coordinate of the plot point.
c
c     sptv      Output   Vertical   axis coordinate of the plot point.
c
c     xpt       Input    X-coordinate of a plot point.
c
c     ypt       Input    Y-coordinate of a plot point.
c
c     zpt       Input    Z-coordinate of a plot point.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8 asysx

      nerrg = 0

c.... Find the horizontal axis coordinate.

      if     (axish .eq. 'x')     then
        spth = xpt
      elseif (axish .eq. 'y')     then
        spth = ypt
      elseif (axish .eq. 'z')     then
        spth = zpt
      elseif (axish .eq. 'rcyl')  then
        asysx = 'cyl'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    spth, v, w, vpos, nerrg)
      elseif (axish .eq. 'rsph')  then
        asysx = 'sph'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    spth, v, w, vpos, nerrg)
      elseif (axish .eq. 'theta') then
        asysx = 'sph'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    u, spth, w, vpos, nerrg)
      elseif (axish .eq. 'phi')   then
        asysx = 'sph'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    u, v, spth, vpos, nerrg)
      else
        nerrg = 1
      endif

c.... Find the vertical axis coordinate.

      if     (axisv .eq. 'x')     then
        sptv = xpt
      elseif (axisv .eq. 'y')     then
        sptv = ypt
      elseif (axisv .eq. 'z')     then
        sptv = zpt
      elseif (axisv .eq. 'rcyl')  then
        asysx = 'cyl'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    sptv, v, w, vpos, nerrg)
      elseif (axisv .eq. 'rsph')  then
        asysx = 'sph'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    sptv, v, w, vpos, nerrg)
      elseif (axisv .eq. 'theta') then
        asysx = 'sph'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    u, sptv, w, vpos, nerrg)
      elseif (axisv .eq. 'phi')   then
        asysx = 'sph'
        call ptcoord (asysx, angunit, xpt, ypt, zpt,
     &                    u, v, sptv, vpos, nerrg)
      else
        nerrg = 1
      endif

  999 return

c.... End of subroutine plotvar.      (+1 line)
      end
