      subroutine plout (n1, n2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  plout    Updated     2004 April 9 15:20.
c               plout    Originated  1993 November 17 18:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the points and normal vectors defining any planes
c               among quadrics n1 to n2.
c
c     Logic:    Entry points:  plout.
c plout         Called by:  accel, copy, delete, distance, operator, plane,
c               project, quadric, renamea, renamex, track, vector.
c
c plout         Calls:  aptcsyv, aptvusz.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. nquads) nn2 = nquads

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

 8110 format ('coord     ',a,2x,a,4x,a,4x,a)

      write (   6, 8110) asys, acoordu, acoordv, acoordw
      write (iout, 8110) asys, acoordu, acoordv, acoordw

 8120 format ('plane     ',a,2x,a16,2x,a)
 8140 format ('  Center (P) (uvw)',1p3e20.12 /
     &        '  qc = -N dot P   ',1pe20.12 /
     &        '    qx,  qy,  qz  ',1p3e20.12 /
     &        '  Norm vect  (uvw)',1p3e20.12 )

      do 120 n = nn1, nn2

        if (aquad(n) .eq. ' ') go to 120
        if (aqtype(n) .ne. aqtypes(0)) go to 120

c....   Find the general form of the plane.

        if ((qy(n) .eq. 0.0)  .and.
     &      (qz(n) .eq. 0.0)) then
          aqform(n) = 'planex'
          go to 130
        elseif ((qz(n) .eq. 0.0)  .and.
     &          (qx(n) .eq. 0.0)) then
          aqform(n) = 'planey'
          go to 130
        elseif ((qx(n) .eq. 0.0)  .and.
     &          (qy(n) .eq. 0.0)) then
          aqform(n) = 'planez'
          go to 130
        else
          aqform(n) = 'planegen'
          go to 130
        endif

  130   write (   6, 8120) aquad(n), aqform(n), asym(n)
        write (iout, 8120) aquad(n), aqform(n), asym(n)

c....   Find the central point of the plane.  Zero out small components.

        call aptvusz (qx(n), qy(n), qz(n), tol, du, dv, dw, vlength)

        qx(n)   = du * vlength
        qy(n)   = dv * vlength
        qz(n)   = dw * vlength

        xcen(n) = -qc(n) * qx(n) / vlength**2
        ycen(n) = -qc(n) * qy(n) / vlength**2
        zcen(n) = -qc(n) * qz(n) / vlength**2

c....   Display the data for the plane in the current coordinate system.

        u  = xcen(n)
        v  = ycen(n)
        w  = zcen(n)

        du = qx(n)
        dv = qy(n)
        dw = qz(n)

        if     (asys .eq. 'cyl') then
          call aptcsyv (0, 1, iunit, u, v, w, du, dv, dw, 1,
     &                  tol, nerr)
        elseif (asys .eq. 'sph') then
          call aptcsyv (0, 2, iunit, u, v, w, du, dv, dw, 1,
     &                  tol, nerr)
        endif

        write (   6, 8140) u, v, w, qc(n), qx(n), qy(n), qz(n),
     &    du, dv, dw
        write (iout, 8140) u, v, w, qc(n), qx(n), qy(n), qz(n),
     &    du, dv, dw

  120 continue

      return

c.... End of subroutine plout.        (+1 line)
      end
