      subroutine ptcoord (asyspt, anglpt, xp, yp, zp,
     &                    u, v, w, vpos, nerrg)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ptcoord  Updated     1995 December 19 17:30.
c               ptcoord  Originated  1995 August 2 16:30
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the coordinates of a point in the current
c               coordinate system and angle units.
c
c     Logic:    Entry points:  ptcoord.
c ptcoord       Called by:  clout, cloutcl, plotvar, pout, search.
c
c ptcoord       Calls:  aptcsys.
c
c     Biblio:   ?
c
c     Glossary:
c
c     anglpt    Input    Angle units:  'degrees' or 'radians'.
c
c     asyspt    Input    Coordinate system:  'rect', 'cyl' or 'sph'.
c
c     nerrg      Output   0 if no input error.  1 if asyspt not recognized.
c                          2 if anglpt not recognized.
c
c     u, v, w   Output   The coordinates of the point in the current coordinate
c                          system and angle units.
c
c     vpos      Output   If the coordinate system is cylindrical or spherical,
c                          and v (angle theta) is negative, vpos is the value
c                          of v in the positive angle theta direction,
c                          vpos = v + 2 * pi radians, or
c                          vpos = v + 360 degrees.
c                          Otherwise, vpos = 0.
c
c     xp        Input    X coordinate of the point.
c
c     yp        Input    Y coordinate of the point.
c
c     zp        Input    Z coordinate of the point.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8     anglpt          ! Angle units:  'degrees' or 'radians'.
      character*8     asyspt          ! Coordinate system:  'rect', 'cyl' or
                                      !   'sph'.

c.... Initialize.

      nerrg = 0

 8110 format ('  ptcoord ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Coordinate system not "rect", "cyl" or "sph":  ',a)

      if ((asyspt .ne. 'rect') .and.
     &    (asyspt .ne. 'cyl')  .and.
     &    (asyspt .ne. 'sph')) then
        write (   6, 8110) ainput, ninput, asyspt
        write (iout, 8110) ainput, ninput, asyspt
        nerrg = 1
      endif

 8120 format ('  ptcoord ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Angle units not "degrees" or "radians":  ',a)

      if (anglpt .eq. 'degrees') then
        iunit = 0
      elseif (anglpt .eq. 'radians') then
        iunit = 1
      else
        write (   6, 8120) ainput, ninput, anglpt
        write (iout, 8120) ainput, ninput, anglpt
        nerrg = 2
      endif

      if (nerrg .ne. 0) go to 999

      u    = xp
      v    = yp
      vpos = 0.0
      w    = zp

c.... Find current coordinate system, and convert coordinates of the point.

      if (asyspt .eq. 'cyl') then       ! Convert to cylindrical coordinates.

        call aptcsys (0, 1, iunit, u, v, w, 1, tol, nerra)

        if (v .lt. 0.0) then
          if (iunit .eq. 0) then
            vpos = v + 360.0
          else
            vpos = v + 2.0 * pi
          endif
        endif

      elseif (asyspt .eq. 'sph') then   ! Convert to spherical coordinates.

        call aptcsys (0, 2, iunit, u, v, w, 1, tol, nerra)

        if (v .lt. 0.0) then
          if (iunit .eq. 0) then
            vpos = v + 360.0
          else
            vpos = v + 2.0 * pi
          endif
        endif

      endif                           ! Found current coordinate system

  999 return

c.... End of subroutine ptcoord.      (+1 line)
      end
