      subroutine repack

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  repack   Updated     2006 January 6 15:40.
c               repack   Originated  2003 October 3 11:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c
c     Purpose:  To repack aliases, probability bins, bricks, clusters, annular
c               disks, lines, markers, operators, probability distribution
c               functions (pdfs), points, regular polygons, regular polyhedrons,
c               quadric surfaces (planes, spheres, circular cylinders, circular
c               cones, hyperbolic paraboloids, ellipsoids, axisymmetric and
c               general), symbols, tetrahedrons, triangles, variables, vectors
c               and zones.
c
c     Logic:    Entry points:  repack.
c repack        Called by:  helpall, list, main, sort, tables.
c               Calls:  (none).
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*24 awordx

c.... Input line:  repack [help]

 8110 format (
     &  '  Command "repack":  acceptable input lines include:' //
     &  '  "repack [help]      "   Display this message.' //
     &  '  "repack all         "   Repack all objects.' /
     &  '  "repack alias       "   Repack aliases.' /
     &  '  "repack axisym      "   Repack all quadric surfaces.' /
     &  '  "repack bin         "   Repack probability bins.' /
     &  '  "repack brick       "   Repack bricks.' /
     &  '  "repack cluster     "   Repack clusters.' /
     &  '  "repack cone        "   Repack all quadric surfaces.' /
     &  '  "repack cylinder    "   Repack all quadric surfaces.' /
     &  '  "repack disk        "   Repack annular disks.' /
     &  '  "repack ellipsoid   "   Repack all quadric surfaces.' /
     &  '  "repack hyperb      "   Repack all quadric surfaces.' /
     &  '  "repack line        "   Repack lines.' /
     &  '  "repack marker      "   Repack markers.' /
     &  '  "repack mesh        "   Repack mesh points.' /
     &  '  "repack operator    "   Repack operators.' )

 8112 format (
     &  '  "repack pdf         "   Repack pdfs.' /
     &  '  "repack plane       "   Repack all quadric surfaces.' /
     &  '  "repack point       "   Repack points.' /
     &  '  "repack polygon     "   Repack regular polygons.' /
     &  '  "repack polyhedron  "   Repack regular polyhedrons.' /
     &  '  "repack quadric     "   Repack all quadric surfaces.' /
     &  '  "repack sphere      "   Repack all quadric surfaces.' /
     &  '  "repack symbol      "   Repack symbols.' /
     &  '  "repack tetrahedron "   Repack tetrahedrons.' /
     &  '  "repack triangle    "   Repack triangles.' /
     &  '  "repack variable    "   Repack variables.' /
     &  '  "repack vector      "   Repack vectors.' /
     &  '  "repack zone        "   Repack zones.' //
     &  '  "repack OBJTYPE1 OBJTYPE2 OBJTYPE3 ..."' /
     &  '          Display the names of all objects of types' /
     &  '          OBJTYPE1, OBJTYPE2, OBJTYPE3, ...  which may be',
     &  ' any of the' /
     &  '          object types listed above.' )

 8114 format (/
     &  '  Synonyms:  [alias, al, a], [brick, br], [cluster, cl],' /
     &  '    [cylinder, cyl], [disk, disc, dk],',
     &  ' [ellipsoid, ellipse, ell, el], [help, h],' /
     &  '    [hyperb, saddle], [line, ln, l], [marker, mark, m],' /
     &  '    [operator, oper, op, o],' /
     &  '    [point, pnt, pt, p], [polygon, polyg, pg],',
     &  ' [polyhedron, polyh, ph],' /
     &  '    [quadric, quad, q], [sphere, sph], [symbol, symb, sym],' /
     &  '    [tetrahedron, tetra, tet],  [triangle, tri],' /
     &  ' [variable, var, set, s], [vector, vect, v], [zone, zn, z]' )

      if (nword .eq. 0) go to 999

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        go to 999
      endif

      nrepack = 0

c=======================================================================********

c.... Test for errors.

 9980 format ('  repack ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Less than 2 words in repack command:  ', i3,'.' )

      if (nword .lt. 2) then
        write (   6, 9980) ainput, ninput, nword
        write (iout, 9980) ainput, ninput, nword
        go to 999
      endif

c=======================================================================********

      do 400 nw = 2, nword            ! Loop over object types.

c=======================================================================********

c.... Input line:  repack alias
c.... Input line:  repack all

      if ((aword(nw) .eq. 'alias') .or.
     &    (aword(nw) .eq. 'all')   .or.
     &    (aword(nw) .eq. 'al')    .or.
     &    (aword(nw) .eq. 'a'))    then

        nrepack = 1
        naliasx = nalias
        awordx  = 'aliases'

c....   Remove blanked aliases.

 8200 format ('Repacked ',a24,' from ',i6,' to ',i6,'.')

        ns = 0
        do 525 nn = 1, nalias
          if (alias1(nn) .eq. ' ') go to 525
          ns          = ns + 1
          alias1(ns)  = alias1(nn)
          alias2(ns)  = alias2(nn)
          lalias2(ns) = lalias2(nn)
  525   continue
        nalias = ns

        if (nalias .ne. naliasx) then
          write (   6, 8200) awordx, naliasx, nalias
          write (iout, 8200) awordx, naliasx, nalias
        endif

      endif                           ! Repacked aliases.


c=======================================================================********

c.... Input line:  repack big
c.... Input line:  repack all

      if ((aword(nw) .eq. 'big')  .or.
     &    (aword(nw) .eq. 'all')) then

c....   Do not need to repack big integers.

        nrepack = 1

      endif                           ! Ignored big integers.

c=======================================================================********

c.... Input line:  repack bin
c.... Input line:  repack all

      if ((aword(nw) .eq. 'bin')  .or.
     &    (aword(nw) .eq. 'all')) then

        nrepack = 1
        nbinx   = nbins
        awordx  = 'bins'

c....   Remove blanked bins.

        nb = 0
        do 465 nn = 1, nbins
          if (abin(nn) .eq. ' ') go to 465
          nb         = nb + 1
          abin(nb)   = abin(nn)
          atypeb(nb) = atypeb(nn)
          probl(nb)  = probl(nn)
          probr(nb)  = probr(nn)
          vranl(nb)  = vranl(nn)
          vranr(nb)  = vranr(nn)
          prbin(nb)  = prbin(nn)
          prbint(nb) = prbint(nn)
  465   continue
        nbins = nb

        if (nbins .ne. nbinx) then
          write (   6, 8200) awordx, nbinx, nbins
          write (iout, 8200) awordx, nbinx, nbins
        endif

c....   Remove blanked pdf/bin pairs.

        npdfxx  = npdfxs
        awordx  = 'pdf/bin pairs'

        nb = 0
        do 468 nn = 1, npdfxs
          if (abinx(nn) .eq. ' ') go to 468
          if (apdfx(nn) .eq. ' ') go to 468
          nb        = nb + 1
          abinx(nb) = abinx(nn)
          apdfx(nb) = apdfx(nn)
  468   continue
        npdfxs = nb

        if (npdfxs .ne. npdfxx) then
          write (   6, 8200) awordx, npdfxx, npdfxs
          write (iout, 8200) awordx, npdfxx, npdfxs
        endif

      endif                           ! Repacked bins.

c=======================================================================********

c.... Input line:  repack brick
c.... Input line:  repack all

      if ((aword(nw) .eq. 'brick') .or.
     &    (aword(nw) .eq. 'all')   .or.
     &    (aword(nw) .eq. 'br'))   then

        nrepack = 1
        nbrickx = nbricks
        awordx  = 'bricks'

c....   Remove blanked bricks.

        nb = 0
        do 285 nn = 1, nbricks
          if (abrick(nn) .eq. ' ') go to 285
          nb         = nb + 1
          abrick(nb) = abrick(nn)
          asysbr(nb) = asysbr(nn)
          angbr(nb)  = angbr(nn)
          ubr(nb)    = ubr(nn)
          dubr(nb)   = dubr(nn)
          vbr(nb)    = vbr(nn)
          dvbr(nb)   = dvbr(nn)
          wbr(nb)    = wbr(nn)
          dwbr(nb)   = dwbr(nn)
          volbr(nb)  = volbr(nn)
  285   continue
        nbricks = nb

        if (nbricks .ne. nbrickx) then
          write (   6, 8200) awordx, nbrickx, nbricks
          write (iout, 8200) awordx, nbrickx, nbricks
        endif

      endif                           ! Repacked regular bricks.

c=======================================================================********

c.... Input line:  repack cluster
c.... Input line:  repack all

      if ((aword(nw) .eq. 'cluster') .or.
     &    (aword(nw) .eq. 'all')     .or.
     &    (aword(nw) .eq. 'cl'))     then

        nrepack = 1
        nclustx   = nclusts
        awordx  = 'clusters'

c....   Remove blanked clusters.

        ncl = 0
        do 175 nn = 1, nclusts
          if (aclust(nn) .eq. ' ') go to 175
          ncl          = ncl + 1
          aclust(ncl)  = aclust(nn)
          laclust(ncl) = laclust(nn)
  175   continue
        nclusts = ncl

        if (nclusts .ne. nclustx) then
          write (   6, 8200) awordx, nclustx, nclusts
          write (iout, 8200) awordx, nclustx, nclusts
        endif

c....   Remove blanked point/cluster pairs.

        npairx   = npairs
        awordx   = 'pt/cl pairs'

        np = 0
        do 180 nn = 1, npairs
          if (apairpt(nn) .eq. ' ') go to 180
          if (apaircl(nn) .eq. ' ') go to 180
          np         = np + 1
          apairpt(np) = apairpt(nn)
          apaircl(np) = apaircl(nn)
  180   continue
        npairs = np

        if (npairs .ne. npairx) then
          write (   6, 8200) awordx, npairx, npairs
          write (iout, 8200) awordx, npairx, npairs
        endif

      endif                           ! Repacked clusters.

c=======================================================================********

c.... Input line:  repack disk
c.... Input line:  repack all

      if ((aword(nw) .eq. 'disk') .or.
     &    (aword(nw) .eq. 'all')  .or.
     &    (aword(nw) .eq. 'disc') .or.
     &    (aword(nw) .eq. 'dk'))  then

        nrepack = 1
        ndiskx  = ndisks
        awordx  = 'disks'

c....   Remove blanked disks.

        nd = 0
        do 425 nn = 1, ndisks
          if (adisk(nn) .eq. ' ') go to 425
          nd          = nd + 1
          adisk(nd)   = adisk(nn)
          radkin(nd)  = radkin(nn)
          radkout(nd) = radkout(nn)
          acendk(nd)  = acendk(nn)
          xnormdk(nd) = xnormdk(nn)
          ynormdk(nd) = ynormdk(nn)
          znormdk(nd) = znormdk(nn)
  425   continue
        ndisks = nd

        if (ndisks .ne. ndiskx) then
          write (   6, 8200) awordx, ndiskx, ndisks
          write (iout, 8200) awordx, ndiskx, ndisks
        endif

      endif                           ! Repacked annular disks.

c=======================================================================********

c.... Input line:  repack line
c.... Input line:  repack all

      if ((aword(nw) .eq. 'line') .or.
     &    (aword(nw) .eq. 'all')  .or.
     &    (aword(nw) .eq. 'ln')   .or.
     &    (aword(nw) .eq. 'l'))   then

        nrepack = 1
        nlinex  = nlines
        awordx  = 'lines'

c....   Remove blanked lines.

        nl = 0
        do 195 nn = 1, nlines
          if (aline(nn) .eq. ' ') go to 195
          nl          = nl + 1
          aline(nl)   = aline(nn)
          alipt1(nl)  = alipt1(nn)
          lalipt1(nl) = lalipt1(nn)
          alipt2(nl)  = alipt2(nn)
          lalipt2(nl) = lalipt2(nn)
  195   continue
        nlines = nl

        if (nlines .ne. nlinex) then
          write (   6, 8200) awordx, nlinex, nlines
          write (iout, 8200) awordx, nlinex, nlines
        endif

      endif                           ! Repacked lines.

c=======================================================================********

c.... Input line:  repack marker
c.... Input line:  repack all

      if ((aword(nw) .eq. 'marker') .or.
     &    (aword(nw) .eq. 'all')    .or.
     &    (aword(nw) .eq. 'mark')   .or.
     &    (aword(nw) .eq. 'm'))     then

        nrepack = 1
        nmarkx  = nmarks
        awordx  = 'markers'

c....   Remove blanked markers.

        nm = 0
        do 545 nn = 1, nmarks
          if (amark(nn) .eq. ' ') go to 545
          nm         = nm + 1
          amark(nm)  = amark(nn)
          amarkt(nm) = amarkt(nn)
  545   continue
        nmarks = nm

        if (nmarks .ne. nmarkx) then
          write (   6, 8200) awordx, nmarkx, nmarks
          write (iout, 8200) awordx, nmarkx, nmarks
        endif

      endif                           ! Repacked markers.

c=======================================================================********

c.... Input line:  repack mesh

      if (aword(nw) .eq. 'mesh') then

        nrepack = 1
        npointx = npoints
        awordx  = 'mesh points'

c....   Remove blanked points.

        np = 0
        do 168 nn = 1, npoints
          if (apoint(nn) .eq. ' ') go to 168
          np          = np + 1
          apoint(np)  = apoint(nn)
          lapoint(np) = lapoint(nn)
          x(np)       = x(nn)
          y(np)       = y(nn)
          z(np)       = z(nn)
          kpoint(np)  = kpoint(nn)
          lpoint(np)  = lpoint(nn)
          mpoint(np)  = mpoint(nn)
  168   continue
        npoints = np

        if (npoints .ne. npointx) then
          write (   6, 8200) awordx, npointx, npoints
          write (iout, 8200) awordx, npointx, npoints
        endif

c....   Remove blanked point/cluster pairs.

        npairx   = npairs
        awordx   = 'pt/cl pairs'

        np = 0
        do 176 nn = 1, npairs
          if (apairpt(nn) .eq. ' ') go to 176
          np         = np + 1
          apairpt(np) = apairpt(nn)
          apaircl(np) = apaircl(nn)
  176   continue
        npairs = np

        if (npairs .ne. npairx) then
          write (   6, 8200) awordx, npairx, npairs
          write (iout, 8200) awordx, npairx, npairs
        endif

      endif                           ! Repacked points by mesh index.

c=======================================================================********

c.... Input line:  repack operator
c.... Input line:  repack all

      if ((aword(nw) .eq. 'operator') .or.
     &    (aword(nw) .eq. 'all')      .or.
     &    (aword(nw) .eq. 'oper')     .or.
     &    (aword(nw) .eq. 'op'))      then

        nrepack = 1
        noperx  = nopers
        awordx  = 'operators'

c....   Remove blanked operators.

        nop = 0
        do 395 nn = 1, nopers
          if (aoper(nn) .eq. ' ') go to 395
          nop         = nop + 1
          aoper(nop) = aoper(nn)
          do 371 i = 1, 3
            do 366 j = 1, 3
              toper(i,j,nop) = toper(i,j,nn)
              topin(i,j,nop) = topin(i,j,nn)
  366       continue
  371     continue
  395   continue
        nopers = nop

        if (nopers .ne. noperx) then
          write (   6, 8200) awordx, noperx, nopers
          write (iout, 8200) awordx, noperx, nopers
        endif

      endif                           ! Repacked operators.

c=======================================================================********

c.... Input line:  repack pdf
c.... Input line:  repack all

      if ((aword(nw) .eq. 'pdf')     .or.
     &    (aword(nw) .eq. 'all'))    then

        nrepack = 1
        npdfx   = npdfs
        awordx  = 'pdfs'

c....   Remove blanked pdfs.

        npx = 0
        do 485 nn = 1, npdfs
          if (apdf(nn) .eq. ' ') go to 485
          npx         = npx + 1
          apdf(npx)   = apdf(nn)
          prpdf(npx)  = prpdf(nn)
          prpdft(npx) = prpdft(nn)
  485   continue
        npdfs = npx

        if (npdfs .ne. npdfx) then
          write (   6, 8200) awordx, npdfx, npdfs
          write (iout, 8200) awordx, npdfx, npdfs
        endif

c....   Remove blanked pdf/bin pairs.

        npdfxx  = npdfxs
        awordx  = 'pdf/bin pairs'

        nb = 0
        do 490 nn = 1, npdfxs
          if (abinx(nn) .eq. ' ') go to 490
          if (apdfx(nn) .eq. ' ') go to 490
          nb        = nb + 1
          abinx(nb) = abinx(nn)
          apdfx(nb) = apdfx(nn)
  490   continue
        npdfxs = nb

        if (npdfxs .ne. npdfxx) then
          write (   6, 8200) awordx, npdfxx, npdfxs
          write (iout, 8200) awordx, npdfxx, npdfxs
        endif

      endif                           ! Repacked pdfs.

c=======================================================================********

c.... Input line:  repack point
c.... Input line:  repack all

      if ((aword(nw) .eq. 'point') .or.
     &    (aword(nw) .eq. 'all')   .or.
     &    (aword(nw) .eq. 'pnt')   .or.
     &    (aword(nw) .eq. 'pt')    .or.
     &    (aword(nw) .eq. 'p'))    then

        nrepack = 1
        npointx = npoints
        awordx  = 'points'

c....   Remove blanked points.

        np = 0
        do 155 nn = 1, npoints
          if (apoint(nn) .eq. ' ') go to 155
          np          = np + 1
          apoint(np)  = apoint(nn)
          lapoint(np) = lapoint(nn)
          x(np)       = x(nn)
          y(np)       = y(nn)
          z(np)       = z(nn)
          kpoint(np)  = kpoint(nn)
          lpoint(np)  = lpoint(nn)
          mpoint(np)  = mpoint(nn)
  155   continue
        npoints = np

        if (npoints .ne. npointx) then
          write (   6, 8200) awordx, npointx, npoints
          write (iout, 8200) awordx, npointx, npoints
        endif

c....   Remove blanked point/cluster pairs.

        npairx   = npairs
        awordx   = 'pt/cl pairs'

        np = 0
        do 159 nn = 1, npairs
          if (apairpt(nn) .eq. ' ') go to 159
          if (apaircl(nn) .eq. ' ') go to 159
          np         = np + 1
          apairpt(np) = apairpt(nn)
          apaircl(np) = apaircl(nn)
  159   continue
        npairs = np

        if (npairs .ne. npairx) then
          write (   6, 8200) awordx, npairx, npairs
          write (iout, 8200) awordx, npairx, npairs
        endif

      endif                           ! Repacked points.

c=======================================================================********

c.... Input line:  repack polygon
c.... Input line:  repack all

      if ((aword(nw) .eq. 'polygon') .or.
     &    (aword(nw) .eq. 'all')     .or.
     &    (aword(nw) .eq. 'polyg')   .or.
     &    (aword(nw) .eq. 'pg'))     then

        nrepack = 1
        npolygx = npolygs
        awordx  = 'reg polygons'

c....   Remove blanked regular polygons.

        ng = 0
        do 265 nn = 1, npolygs
          if (apolyg(nn) .eq. ' ') go to 265
          ng         = ng + 1
          apolyg(ng) = apolyg(nn)
          nverpg(ng) = nverpg(nn)
          acenpg(ng) = acenpg(nn)
          averpg(ng) = averpg(nn)
          acoppg(ng) = acoppg(nn)
  265   continue
        npolygs = ng

        if (npolygs .ne. npolygx) then
          write (   6, 8200) awordx, npolygx, npolygs
          write (iout, 8200) awordx, npolygx, npolygs
        endif

      endif                           ! Repacked regular polygons.

c=======================================================================********

c.... Input line:  repack polyhedron
c.... Input line:  repack all

      if ((aword(nw) .eq. 'polyhedron') .or.
     &    (aword(nw) .eq. 'all')     .or.
     &    (aword(nw) .eq. 'polyh')   .or.
     &    (aword(nw) .eq. 'ph'))     then

        nrepack = 1
        npolyhx = npolyhs
        awordx  = 'reg polyhedrons'

c....   Remove blanked regular polyhedrons.

        nh = 0
        do 305 nn = 1, npolyhs
          if (apolyh(nn) .eq. ' ') go to 305
          nh         = nh + 1
          apolyh(nh) = apolyh(nn)
          apolyh(nh) = apolyh(nn)
          atypeh(nh) = atypeh(nn)
          nverph(nh) = nverph(nn)
          nedgph(nh) = nedgph(nn)
          acenph(nh) = acenph(nn)
          averph(nh) = averph(nn)
          acopph(nh) = acopph(nn)
  305   continue
        npolyhs = nh

        if (npolyhs .ne. npolyhx) then
          write (   6, 8200) awordx, npolyhx, npolyhs
          write (iout, 8200) awordx, npolyhx, npolyhs
        endif

      endif                           ! Repacked regular polyhedrons.

c=======================================================================********

c.... Input line:  repack quadric
c.... Input line:  repack   axisym
c.... Input line:  repack   cone
c.... Input line:  repack   cylinder
c.... Input line:  repack   ellipsoid
c.... Input line:  repack   hyperb
c.... Input line:  repack   plane
c.... Input line:  repack   sphere
c.... Input line:  repack all

      if ((aword(nw) .eq. 'quadric')   .or.
     &    (aword(nw) .eq. 'quad')      .or.
     &    (aword(nw) .eq. 'q')         .or.
     &    (aword(nw) .eq. 'plane')     .or.
     &    (aword(nw) .eq. 'pl')        .or.
     &    (aword(nw) .eq. 'sphere')    .or.
     &    (aword(nw) .eq. 'sph')       .or.
     &    (aword(nw) .eq. 'cylinder')  .or.
     &    (aword(nw) .eq. 'cyl')       .or.
     &    (aword(nw) .eq. 'cone')      .or.
     &    (aword(nw) .eq. 'hyperb')    .or.
     &    (aword(nw) .eq. 'saddle')    .or.
     &    (aword(nw) .eq. 'ellipsoid') .or.
     &    (aword(nw) .eq. 'ellipse')   .or.
     &    (aword(nw) .eq. 'ell')       .or.
     &    (aword(nw) .eq. 'el')        .or.
     &    (aword(nw) .eq. 'axisym')    .or.
     &    (aword(nw) .eq. 'all'))      then

        nrepack = 1
        nquadx  = nquads
        awordx  = 'quadric surfaces'

c....   Remove blanked quadric surfaces.

        nq = 0
        do 328 nn = 1, nquads
          if (aquad(nn) .eq. ' ') go to 328
          nq         = nq + 1
          aquad(nq)  = aquad(nn)
          laquad(nq) = laquad(nn)
          aqtype(nq) = aqtype(nn)
          aqform(nq) = aqform(nn)
          asym(nq)   = asym(nn)
          xcen(nq)   = xcen(nn)
          ycen(nq)   = ycen(nn)
          zcen(nq)   = zcen(nn)
          qc(nq)     = qc(nn)
          qx(nq)     = qx(nn)
          qy(nq)     = qy(nn)
          qz(nq)     = qz(nn)
          qxy(nq)    = qxy(nn)
          qyz(nq)    = qyz(nn)
          qzx(nq)    = qzx(nn)
          qxx(nq)    = qxx(nn)
          qyy(nq)    = qyy(nn)
          qzz(nq)    = qzz(nn)
  328   continue
        nquads = nq

        if (nquads .ne. nquadx) then
          write (   6, 8200) awordx, nquadx, nquads
          write (iout, 8200) awordx, nquadx, nquads
        endif

      endif                           ! Repacked quadric surfaces.

c=======================================================================********

c.... Input line:  repack symbol
c.... Input line:  repack all

      if ((aword(nw) .eq. 'symbol') .or.
     &    (aword(nw) .eq. 'all')    .or.
     &    (aword(nw) .eq. 'symb')   .or.
     &    (aword(nw) .eq. 'sym'))   then

        nrepack = 1
        nsymbx  = nsymbs
        awordx  = 'symbols'

c....   Remove blanked symbols.

        nsymb = 0
        do 565 nn = 1, nsymbs
          if (asymbl(nn) .eq. ' ') go to 565
          nsymb         = nsymb + 1
          asymbl(nsymb) = asymbl(nn)
          asymbr(nsymb) = asymbr(nn)
          fsymb(nsymb)  = fsymb(nn)
          isymb(nsymb)  = isymb(nn)
          lsymb(nsymb)  = lsymb(nn)
          msymb(nsymb)  = msymb(nn)
  565   continue
        nsymbs = nsymb

        if (nsymbs .ne. nsymbx) then
          write (   6, 8200) awordx, nsymbx, nsymbs
          write (iout, 8200) awordx, nsymbx, nsymbs
        endif

      endif                           ! Repacked symbols.

c=======================================================================********

c.... Input line:  repack tetrahedron
c.... Input line:  repack all

      if ((aword(nw) .eq. 'tetrahedron') .or.
     &    (aword(nw) .eq. 'all')         .or.
     &    (aword(nw) .eq. 'tetra')       .or.
     &    (aword(nw) .eq. 'tet'))        then

        nrepack = 1
        ntetx   = ntets
        awordx  = 'tetrahedrons'

c....   Remove blanked tetrahedrons.

        nt = 0
        do 245 nn = 1, ntets
          if (atetra(nn) .eq. ' ') go to 245
          nt          = nt + 1
          atetra(nt)  = atetra(nn)
          acenpg(nt)  = acenpg(nn)
          voltet(nt)  = voltet(nn)
          do 242 nv = 1, 4
            atetpt(nv,nt)  = atetpt(nv,nn)
            latetpt(nv,nt) = latetpt(nv,nn)
  242     continue
  245   continue
        ntets = nt

        if (ntets .ne. ntetx) then
          write (   6, 8200) awordx, ntetx, ntets
          write (iout, 8200) awordx, ntetx, ntets
        endif

      endif                           ! Repacked tetrahedrons.

c=======================================================================********

c.... Input line:  repack triangle
c.... Input line:  repack all

      if ((aword(nw) .eq. 'triangle') .or.
     &    (aword(nw) .eq. 'all')      .or.
     &    (aword(nw) .eq. 'tri'))     then

        nrepack = 1
        ntrigx  = ntrigs
        awordx  = 'triangles'

c....   Remove blanked triangles.

        nt = 0
        do 222 nn = 1, ntrigs
          if (atrig(nn) .eq. ' ') go to 222
          nt           = nt + 1
          atrig(nt)    = atrig(nn)
          atript1(nt)  = atript1(nn)
          atript2(nt)  = atript2(nn)
          atript3(nt)  = atript3(nn)
          latript1(nt) = latript1(nn)
          latript2(nt) = latript2(nn)
          latript3(nt) = latript3(nn)
  222   continue
        ntrigs = nt

        if (ntrigs .ne. ntrigx) then
          write (   6, 8200) awordx, ntrigx, ntrigs
          write (iout, 8200) awordx, ntrigx, ntrigs
        endif

      endif                           ! Repacked triangles.

c=======================================================================********

c.... Input line:  repack variable
c.... Input line:  repack all

      if ((aword(nw) .eq. 'variable') .or.
     &    (aword(nw) .eq. 'all')      .or.
     &    (aword(nw) .eq. 'var')      .or.
     &    (aword(nw) .eq. 'set')      .or.
     &    (aword(nw) .eq. 's'))       then

        nrepack = 1
        nvarx   = nvars
        awordx  = 'variables'

c....   Remove blanked variables.

        nvar = 0
        do 135 nn = 1, nvars
          if (avarbl(nn) .eq. ' ') go to 135
          nvar          = nvar + 1
          avarbl(nvar)  = avarbl(nn)
          avar(nvar)    = avar(nn)
          fvar(nvar)    = fvar(nn)
          ivar(nvar)    = ivar(nn)
          lvar(nvar)    = lvar(nn)
          mvar(nvar)    = mvar(nn)
          avartag(nvar) = avartag(nn)
  135   continue
        nvars = nvar

        if (nvars .ne. nvarx) then
          write (   6, 8200) awordx, nvarx, nvars
          write (iout, 8200) awordx, nvarx, nvars
        endif

      endif                           ! Repacked variables.

c=======================================================================********

c.... Input line:  repack vector
c.... Input line:  repack all

      if ((aword(nw) .eq. 'vector') .or.
     &    (aword(nw) .eq. 'all')    .or.
     &    (aword(nw) .eq. 'vect')   .or.
     &    (aword(nw) .eq. 'v'))     then

        nrepack  = 1
        nvectorx = nvectors
        awordx   = 'vectors'

c....   Remove blanked vectors.

        nv = 0
        do 368 nn = 1, nvectors
          if (avector(nn) .eq. ' ') go to 368
          nv           = nv + 1
          avector(nv)  = avector(nn)
          lavector(nv) = lavector(nn)
          xcom(nv)     = xcom(nn)
          ycom(nv)     = ycom(nn)
          zcom(nv)     = zcom(nn)
          xbnd(nv)     = xbnd(nn)
          ybnd(nv)     = ybnd(nn)
          zbnd(nv)     = zbnd(nn)
          vlen(nv)     = vlen(nn)
  368   continue
        nvectors = nv

        if (nvectors .ne. nvectorx) then
          write (   6, 8200) awordx, nvectorx, nvectors
          write (iout, 8200) awordx, nvectorx, nvectors
        endif

      endif                           ! Repacked vectors.

c=======================================================================********

c.... Input line:  repack zone
c.... Input line:  repack all

      if ((aword(nw) .eq. 'zone') .or.
     &    (aword(nw) .eq. 'all')     .or.
     &    (aword(nw) .eq. 'zn')    .or.
     &    (aword(nw) .eq. 'z'))       then

        nrepack = 1
        nzonex  = nzones
        awordx  = 'zones'

c....   Remove blanked zones.

        nz = 0
        do 345 nn = 1, nzones
          if (azone(nn) .eq. ' ') go to 345
          nz          = nz + 1
          azone(nz)   = azone(nn)
          nzquads(nz) = nzquads(nn)

          do 342 nsq = 1, nzquadm
             azquad(nsq,nz)  = azquad(nsq,nn)
             lazquad(nsq,nz) = lazquad(nsq,nn)
             azsign(nsq,nz)  = azsign(nsq,nn)
  342     continue

          volz(nz)    = volz(nn)
          volzerr(nz) = volzerr(nn)
  345   continue
        nzones = nz

        if (nzones .ne. nzonex) then
          write (   6, 8200) awordx, nzonex, nzones
          write (iout, 8200) awordx, nzonex, nzones
        endif

      endif                           ! Repacked zones.

c=======================================================================********

  400 continue                        ! End of loop over object types.

c=======================================================================********

 9990 format ('  repack ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Object type not recognized:  'a,20(/ a))

      if (nrepack .eq. 0) then
        write (   6, 9990) ainput, ninput, (aword(nw), nw = 2, nword)
        write (iout, 9990) ainput, ninput, (aword(nw), nw = 2, nword)
        go to 999
      endif

c=======================================================================********

  999 return

c.... End of subroutine repack.       (+1 line)
      end
