      subroutine solve

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  solve    Updated     1995 November 29 16:50.
c               solve    Originated  1995 November 29 16:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To solve for the three unknowns x, y and z in the three
c               simultaneous linear equations:
c               A1 * x + A2 * y + A3 * z = D1
c               B1 * x + B2 * y + B3 * z = D2
c               C1 * x + C2 * y + C3 * z = D3
c               where VA, VB, VC and VD are the vectors VA = (A1, A2, A3),
c               VB = (B1, B2, B3), VC = (C1, C2, C3), VD = (D1, D2, D3).
c
c     Logic:    Entry points:  solve.
c solve         Called by:  helpall, main.
c
c solve         Calls:  aptsolv, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /lsolve/ abc(3,3)        ! Coefficients of equations (row, column).
      common /lsolve/ cba(3,3)        ! Inverse of matrix abc     (row, column).
      common /lsolve/ d(3)            ! Right-hand side constants.
      common /lsolve/ dm(3)           ! Right-hand side constants check.
      common /lsolve/ dr(3)           ! Right-hand side constants residuals.
      common /lsolve/ work(3,4)       ! Working matrix.
      common /lsolve/ xyz(3)          ! Solution.
      common /lsolve/ xyzm(3)         ! Solution check.
      common /lsolve/ xyzr(3)         ! Solution residuals.

      common /lsolve/ alabq(3)        ! Names of unknowns
      character*8     alabq

      data alabq / 'x', 'y', 'z' /

c=======================================================================********

c.... Input line:  solve [help]

 8110 format ('  Command "solve":  acceptable input lines include:' /
     &  '  "solve [help]"  Display this message.' //
     &  '  "solve VA VB VC VD"' /
     &  '     Solve for the three unknowns x, y and z in the three',
     &  ' equations' /
     &  '     A1 * x + A2 * y + A3 * z = D1' /
     &  '     B1 * x + B2 * y + B3 * z = D2' /
     &  '     C1 * x + C2 * y + C3 * z = D3' /
     &  '     where VA, VB, VC and VD are the vectors' /
     &  '     (A1, A2, A3), (B1, B2, B3), (C1, C2, C3), and',
     &  ' (D1, D2, D3), respectively.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  solve VA VB VC VD

c.... Test for wrong number of words on "solve" line.

 8120 format ('  solve ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 5 words on "solve" line:  ', i3)
      if (nword .ne. 5) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      nerr = 0

c.... Find vector VA, the coefficients of the first equation.

      call vectfind (2, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 120
      endif

      abc(1,1) = xcom(nv)
      abc(1,2) = ycom(nv)
      abc(1,3) = zcom(nv)

c.... Find vector VB, the coefficients of the second equation.

  120 call vectfind (3, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 130
      endif

      abc(2,1) = xcom(nv)
      abc(2,2) = ycom(nv)
      abc(2,3) = zcom(nv)

c.... Find vector VC, the coefficients of the third equation.

  130 call vectfind (4, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 140
      endif

      abc(3,1) = xcom(nv)
      abc(3,2) = ycom(nv)
      abc(3,3) = zcom(nv)

c.... Find vector D, the right-hand side constants.

  140 call vectfind (5, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 150
      endif

      d(1) = xcom(nv)
      d(2) = ycom(nv)
      d(3) = zcom(nv)

  150 if (nerr .ne. 0) go to 999

c.... Solve the three simultaneous equations.

      call aptsolv (abc, d, 3, work, 3, tol, xyz, cba, xyzm, xyzr,
     &              dm, dr, nerra)

 8210 format ('  solve ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Singular matrix.' )
 8220 format ('  solve WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Large x, y, z residuals.' )
 8230 format ('  solve WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Large d residuals.' )
 8240 format ('Unknowns,  recalculated values, residues:' /
     &        '  x,  recal, resid',1p3e20.12 /
     &        '  y,  recal, resid',1p3e20.12 /
     &        '  z,  recal, resid',1p3e20.12 )
 8250 format ('Constants, recalculated values, residues:' /
     &        '  D1, recal, resid',1p3e20.12 /
     &        '  D2, recal, resid',1p3e20.12 /
     &        '  D3, recal, resid',1p3e20.12 )

      if (nerra .eq. 1) then
        write (   6, 8210) ainput, ninput
        write (iout, 8210) ainput, ninput
        go to 999
      elseif (nerra .eq. 2) then
        write (   6, 8220) ainput, ninput
        write (iout, 8220) ainput, ninput
      elseif (nerra .eq. 3) then
        write (   6, 8230) ainput, ninput
        write (iout, 8230) ainput, ninput
      endif

      write (   6, 8240) (xyz(nq), xyzm(nq), xyzr(nq), nq = 1, 3)
      write (iout, 8240) (xyz(nq), xyzm(nq), xyzr(nq), nq = 1, 3)

      write (   6, 8250) (d(nd), dm(nd), dr(nd), nd = 1, 3)
      write (iout, 8250) (d(nd), dm(nd), dr(nd), nd = 1, 3)

  999 return

c.... End of subroutine solve.        (+1 line)
      end
