      subroutine symbol

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  symbol   Updated     2005 April 5 17:45.
c               symbol   Originated  2002 August 21 14:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create named symbols by specifying a symbol name
c               and a replacement name.
c
c     Logic:    Entry points:  symbol.
c symbol        Called by:  main, synonym.
c
c symbol        Calls:  alout, aptchtp, aptsubr, bigout, compsubs, incrsubs,
c               namechk, symbdisp, symstore, synout, varfind, varout.
c
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*24 anamex             ! Temporary name of a symbol.
      character*80 asymbs             ! Original replacement of symbol.
      character*80 awords             ! User message field.  ASCII (1).
      character*8  amount             ! Amount of output.

c===============================================================================

c.... Input line:  symbol [help]

 8110 format ('  Command "symbol":  acceptable input lines include:' /
     &  '  "symbol [help]"            Display this message.' //
     &  '  "symbol SYMBNAME"           Display named symbol',
     &  ' SYMBNAME.' //
     &  '  "symbol all"               Display all named symbols.'/
     &  '  "symbol list [SYMBNAME1 SYMBNAME2 SYMBNAME3 ...]"' /
     &  '     Display all listed symbols (default is all',
     &  ' symbols).' /
     &  '  "symbol list SYMBNAME1 thru SYMBNAME2"' /
     &  '     Display named symbols SYMBNAME1 thru SYMBNAME2.' //
     &  '  "symbol SYMBNAME [=] REPLNAME"' /
     &  '     Create symbol SYMBNAME with replacement REPLNAME.' /
     &  '     REPLNAME may be an ASCII string, an integer or' /
     &  '     floating point value,  or another symbol.' //
     &  '  "symbol SYMBNAME increment [INCR|1]"' /
     &  '     Increment the ASCII replacement value of',
     &  ' symbol SYMBNAME' /
     &  '     by INCR (default 1) characters.' //
     &  '  "symbol SYMBNAME [=] STRING increment [INCR|1]"' /
     &  '     Create symbol SYMBNAME with ASCII replacement value',
     &  ' STRING' /
     &  '     incremented by INCR (default 1) characters.' //
     &  '  In the commands above, "=" is optional if the',
     &  ' next argument is numerical.' //
     &  '  Note:  symbols will not be replaced in commands "symbol,"' /
     &  '  "copy symbol", "delete symbol" and "rename symbol".' //
     &  '  Note:  use +/- for SYMBNAME to incr/decr the last',
     &  ' symbol name:  ',a //
     &  '  Synonyms:  [increment, incr], [help, h],',
     &  ' [symbol, symb, sym].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110) asymbls
        write (iout, 8110) asymbls
        go to 999
      endif

c===============================================================================

c.... Initialize.

      if (nword .eq. 0) then

        asymbl(1) = 'pi'
        asymbr(1) = '3.14159265358979324'
        fsymb(1)  = pi
        isymb(1)  = 0
        lsymb(1)  = 2
        msymb(1)  = 3

        asymbl(2) = 'deg/rad'
        asymbr(2) = '57.29577951308232'
        fsymb(2)  = 180.0 / pi
        isymb(2)  = 0
        lsymb(2)  = 7
        msymb(2)  = 3


        asymbl(3) = 'rad/deg'
        asymbr(3) = '0.01745329251994330'
        fsymb(3)  = pi / 180.0
        isymb(3)  = 0
        lsymb(3)  = 7
        msymb(3)  = 3

        asymbl(4) = 'ebase'
        asymbr(4) = '2.71828182845904523'
        fsymb(4)  = ebase
        isymb(4)  = 0
        lsymb(4)  = 5
        msymb(4)  = 3

        asymbl(5) = 'rgold'
        asymbr(5) = '1.61803398874989485'
        fsymb(5)  = rgold
        isymb(5)  = 0
        lsymb(5)  = 5
        msymb(5)  = 3

        asymbl(6) = 'euler'
        asymbr(6) = '0.577215664901533861'
        fsymb(6)  = euler
        isymb(6)  = 0
        lsymb(6)  = 5
        msymb(6)  = 3

        nsymbs    = 6
        asymbls   = 'euler'
        lasymbls  = 5

        go to 999

      endif                           ! Initialize when nword = 0.

c=======================================================================********

c.... Input line:  symbol all
c.... Input line:  symbol list

 8120 format (/ 'Number of symbols = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nsymbx = 0

        do 120 n = 1, nsymbs
          if (asymbl(n) .eq. ' ') go to 120
          nsymbx = nsymbx + 1

          call symbdisp (n)

  120   continue

        write (   6, 8120) nsymbx
        write (iout, 8120) nsymbx

        go to 999

      endif                           ! Listed all symbols.

c===============================================================================

c.... Input line:  symbol list SYMBNAME.

      if (aword(2) .eq. 'list') then  ! List one or more symbols.

 8150   format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  Named symbol does not exist:  ',a24)

        if (nword .eq. 3) then        ! List one symbol.

          nfind = 0

          do 142 nsymb = 1, nsymbs    ! Loop over symbols.
            call aptsubr (asymbl(nsymb), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(3) .eq. asymbl(nsymb)) .or.
     &          (aword(3) .eq. astem))  then
              nfind = nfind + 1
              call symbdisp (nsymb)
            endif
  142     continue                    ! End of loop over symbols.

          if(nfind .eq. 0) then
            write (   6, 8150) ainput, ninput, aword(3)
            write (iout, 8150) ainput, ninput, aword(3)
          endif

          go to 999
        endif                         ! Listed one symbol.

c===============================================================================

c.... Input line:  symbol list SYMBNAME1 thru SYMBNAME2

 8136 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of symbol not from 1 to 24 characters:  ',a)
 8138 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name > max name:  ',a24,2x,a24)
 8139 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word "thru" is duplicated.')

      if ((nword .eq. 5) .and.
     &    (aword(4) .eq. 'thru')) then

        if ((lword(3) .lt. 1) .or. (lword(3) .gt. 24)) then
          write (   6, 8136) ainput, ninput, aword(3)
          write (iout, 8136) ainput, ninput, aword(3)
          go to 999
        endif

        if ((lword(5) .lt. 1) .or. (lword(5) .gt. 24)) then
          write (   6, 8136) ainput, ninput, aword(5)
          write (iout, 8136) ainput, ninput, aword(5)
          go to 999
        endif

        call compsubs (aword(3), aword(5), nsmall, nerrc)
        if (nsmall .eq. 2) then
          write (   6, 8138) ainput, ninput, aword(3), aword(5)
          write (iout, 8138) ainput, ninput, aword(3), aword(5)
          go to 999
        endif

        if ((aword(3) .eq. 'thru')  .or.
     &      (aword(5) .eq. 'thru')) then
          write (   6, 8139) ainput, ninput
          write (iout, 8139) ainput, ninput
          go to 999
        endif

 8156   format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  No symbols between ',a24,' and ',a24)

        nfind = 0
        do 132 nsymb = 1, nsymbs
          if (asymbl(nsymb) .eq. ' ') go to 132
          call compsubs (asymbl(nsymb), aword(3), nsmall, nerrc)
          if (nsmall .eq. 1) go to 132
          call compsubs (asymbl(nsymb), aword(5), nsmall, nerrc)
          call aptsubr (asymbl(nsymb), 1, 24, 80, astem, lstem,
     &                  asub, lsub, nerra)
          if ((nsmall .eq. 2) .and.
     &        (astem .ne. aword(5))) go to 132
          nfind = nfind + 1
          call symbdisp (nsymb)
  132   continue                      ! End of loop over symbols.

        if (nfind .eq. 0) then
          write (   6, 8156) ainput, ninput, aword(3), aword(5)
          write (iout, 8156) ainput, ninput, aword(3), aword(5)
        endif

       go to 999

      else                            ! Word 4 was not "thru".

c===============================================================================

c....   Input line:  symbol list SYMBNAME1 SYMBNAME2 SYMBNAME3 ...

        do 140 nw = 3, nword

          do 130 n = 1, nsymbs
            call aptsubr (asymbl(n), 1, 24, 80, astem, lstem,
     &                      asub, lsub, nerra)
            if ((aword(nw) .eq. asymbl(n)) .or.
     &          (aword(nw) .eq. astem))    then
              call symbdisp (n)
              go to 140
            endif
  130     continue

          write (   6, 8150) ainput, ninput, aword(nw)
          write (iout, 8150) ainput, ninput, aword(nw)
  140   continue

        go to 999

      endif                           ! Tested word 4.

      endif                           ! Word 2 was "list"

c===============================================================================

c.... Input line:  symbol SYMBNAME

      if (nword .eq. 2) then

        do 150 n = 1, nsymbs
          if (aword(2) .eq. asymbl(n)) then

            call symbdisp (n)

            go to 999
          endif
  150   continue

        write (   6, 8150) ainput, ninput, aword(2)
        write (iout, 8150) ainput, ninput, aword(2)

        go to 999

      endif                           ! Displayed one symbol.

c===============================================================================

c.... Test for acceptable symbol name.

 8130 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of symbol not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if last symbol name should be incremented and used.

 8140 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last symbol can not be incremented:  ',a24)

      if (aword(2) .eq. '+') then
        aword(2) = asymbls
        lword(2) = lasymbls
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8140) ainput, ninput, asymbls
          write (iout, 8140) ainput, ninput, asymbls
          go to 999
        endif
      endif

 8141 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last symbol can not be decremented:  ',a24)

      if (aword(2) .eq. '-') then
        aword(2) = asymbls
        lword(2) = lasymbls
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8141) ainput, ninput, asymbls
          write (iout, 8141) ainput, ninput, asymbls
          go to 999
        endif
      endif

 8905 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of symbol too dangerous to use:  ',a24)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

      if (aword(2) .eq. 'none') then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  symbol SYMBNAME increment INCR

      if (((nword .eq. 3)              .or.
     &     (nword .eq. 4))             .and.
     &     (aword(3) .eq. 'increment') .or.
     &     (aword(3) .eq. 'incr'))     then

c....   Find the symbol.

        n = 0
        do 160 ns = 1, nsymbs
          if (aword(2) .eq. asymbl(ns)) then
            n = ns

            call symbdisp (n)

          endif
  160   continue

 8155 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named symbol does not exist:  ',a)

        if (n .eq. 0) then
          write (   6, 8155) ainput, ninput, aword(2)
          write (iout, 8155) ainput, ninput, aword(2)
          go to 999
        endif

c....   Find the increment in the replacement value of SYMBNAME.

        if (nword .eq. 3) then
          nword    = 4
          aword(4) = '1'
          iword(4) = 1
          fword(4) = 1.0
          lword(4) = 1
          mtype(4) = 2
        endif

        call varfind (4, 4)

 8162   format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Increment is not an integer:  ',a)

        if (mtype(4) .ne. 2) then
          write (   6, 8162) ainput, ninput, aword(4)
          write (iout, 8162) ainput, ninput, aword(4)
          go to 999
        endif

        incr = iword(4)

c....   Try to increment the replacement value of the symbol.

        asymbs = asymbr(n)

        call incrsubs (incr, asymbr(n), 1, lsymb(n), nerra)

 8164   format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not increment ',a,' by ',i8,'.')

        if (nerra .ne. 0) then
          asymbr(n) = asymbs
          write (   6, 8164) ainput, ninput, asymbr(n), incr
          write (iout, 8164) ainput, ninput, asymbr(n), incr
          go to 999
        endif

        if (asymbl(n) .eq. asymbr(n)) then
          asymbr(n) = asymbs
          write (   6, 8132) ainput, ninput, aword(2)
          write (iout, 8132) ainput, ninput, aword(2)
          go to 999
        endif

c....   Evaluate the new replacement value.

        asymbs = asymbr(n)

        call aptchtp (asymbs, 1, lsymb(n), iamax, idmax, iemax,
     &                asymbr(n), isymb(n), fsymb(n), msymb(n), nerra)

        call symbdisp (n)

        go to 610

      endif                           ! Incrementing symbol replacement.

c===============================================================================

c.... Find the index in the symbol array at which to store this symbol.
c....   Returns nrep = 1 if replacing an existing symbol.

      call symstore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c===============================================================================

c.... Test for optional equal sign.

      if ((nword .ge. 3)      .and.
     &   ((mtype(3) .eq. 2)   .or.
     &    (mtype(3) .eq. 3))) then
        nword  = nword + 1

        do 802 nn = nword, 4, -1
          aword(nn) = aword(nn-1)
          mtype(nn) = mtype(nn-1)
          iword(nn) = iword(nn-1)
          fword(nn) = fword(nn-1)
          lword(nn) = lword(nn-1)
  802   continue

        aword(3) = '='
        mtype(3) = 1
        iword(3) = 0
        fword(3) = 0.0
        lword(3) = 1
      endif

c===============================================================================

c.... Input line:  symbol SYMBNAME = REPLNAME

      if ((nword .eq. 4)       .and.
     &    (aword(3) .eq. '=')) then

 8132   format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  ASCII symbol and replacement are the same:  ',a24)

        if (aword(2) .eq. aword(4)) then
          write (   6, 8132) ainput, ninput, aword(2)
          write (iout, 8132) ainput, ninput, aword(2)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nsymbs) nsymbs = nsymbs - 1
          endif
          go to 999
        endif

        asymbls    = aword(2)
        lasymbls   = lword(2)

        asymbl(n)  = aword(2)
        asymbr(n)  = aword(4)
        lsymb(n)   = lword(4)
        fsymb(n)   = fword(4)
        isymb(n)   = iword(4)
        msymb(n)   = mtype(4)

        call symbdisp (n)

        go to 610

      endif                           ! Created one symbol.

c=======================================================================********

c.... Input line:  symbol SYMBNAME = STRING increment INCR

      if (((nword .eq. 5)              .or.
     &     (nword .eq. 6))             .and.
     &     (aword(3) .eq. '=')         .and.
     &    ((aword(5) .eq. 'increment') .or.
     &     (aword(5) .eq. 'incr')))    then

c....   Find the increment in the value of REPLNAME.

        if (nword .eq. 5) then
          nword    = 6
          aword(6) = '1'
          iword(6) = 1
          fword(6) = 1.0
          lword(6) = 1
          mtype(6) = 2
        endif

        call varfind (6, 6)

        if (mtype(6) .ne. 2) then
          write (   6, 8162) ainput, ninput, aword(6)
          write (iout, 8162) ainput, ninput, aword(6)
          nerr = 1
        endif

        incr = iword(6)

c....   Try to increment STRING.

        call incrsubs (incr, aword(4), 1, lword(4), nerra)

        if (nerra .ne. 0) then
          write (   6, 8164) ainput, ninput, aword(4), incr
          write (iout, 8164) ainput, ninput, aword(4), incr
          nerr = 1
        endif

        if (aword(2) .eq. aword(4)) then
          write (   6, 8132) ainput, ninput, aword(2)
          write (iout, 8132) ainput, ninput, aword(2)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nsymbs) nsymbs = nsymbs - 1
          endif
          go to 999
        endif

c....   Evaluate the replacement value, store the new symbol.

        asymbls   = aword(2)
        lasymbls  = lword(2)

        asymbl(n) = aword(2)
        asymbs    = aword(4)
        lsymb(n)  = lword(4)

        call aptchtp (asymbs, 1, lsymb(n), iamax, idmax, iemax,
     &                asymbr(n), isymb(n), fsymb(n), msymb(n), nerra)

        call symbdisp (n)

        go to 610

      endif                           ! Created one symbol.

c=======================================================================********

c.... Input line is not recognized.

 8134 format ('  symbol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8134) ainput, ninput
      write (iout, 8134) ainput, ninput

      write (   6, 8110)
      write (iout, 8110)

c=======================================================================********

c....   See if a synonym already exists with this symbol name.

 8210 format ('  symbol WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  A preset synonym has the same name.' )

  610   do 210 ns = 1, nsyns
          if ((asymbl(n) .eq. asyn1(ns))  .or.
     &        (asymbr(n) .eq. asyn2(ns))) then
            write (   6, 8210) ainput, ninput
            write (iout, 8210) ainput, ninput
            call synout (ns, ns)
          endif
  210   continue

c....   See if an alias already exists with this symbol name.

 8220   format ('  symbol WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  An alias or its replacement has the same name.' )

        do ns = 1, nalias
          if ((asymbl(n) .eq. alias1(ns))  .or.
     &        (asymbl(n) .eq. alias2(ns))  .or.
     &        (asymbr(n) .eq. alias1(ns))  .or.
     &        (asymbr(n) .eq. alias2(ns))) then
            write (   6, 8220) ainput, ninput
            write (iout, 8220) ainput, ninput
            call alout (ns, ns)
          endif
        enddo

c....  See if a variable already exists with this symbol name.

 8230   format ('  symbol WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  A variable has the same name.' )

        do nv = 1, nvars
          if ((asymbl(n) .eq. avarbl(nv))  .or.
     &        (asymbr(n) .eq. avarbl(nv))) then
            write (   6, 8230) ainput, ninput
            write (iout, 8230) ainput, ninput
            call varout (nv, nv)
          endif
        enddo

c....   See if a big integer already exists with this symbol name.

 8240   format ('  symbol WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  A big integer has the same name.' )

        do nb = 1, nbigm
          if ((asymbl(n) .eq. abig(nb))  .or.
     &        (asymbr(n) .eq. abig(nb))) then
            write (   6, 8240) ainput, ninput
            write (iout, 8240) ainput, ninput
            amount = 'maximum'
            call bigout (nb, nb, amount)
          endif
        enddo

c=======================================================================********

  999 return

c.... End of subroutine symbol.       (+1 line)
      end
