      subroutine volume

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  volume   Updated     1994 August 30 17:00.
c               volume   Originated  1994 March 4 17:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the volume specified by a polygon with 3-36 vertex
c                 points, rotated around a specified axis.  Also finds the
c                 arithmetic average vertex point and edge length, the area
c                 projected in the angular direction around the axis into a
c                 plane through the axis, and the average radius from the axis.
c                 The polygon vertex points need not be coplanar.
c
c     Note:     Do not use a cluster of points as input, because the points may
c                 be reordered by a sort command.
c
c     Logic:    Entry points:  volume.
c volume        Called by:  helpall, main.
c
c volume        Calls:  aptcsys, aptmopv, aptrotv, apttran, aptvusz, ptfind,
c               vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lvolume/ xp(36)
      common /lvolume/ yp(36)
      common /lvolume/ zp(36)
      common /lvolume/ rotm(3,3)

c.... Input line:  volume [help]

 8110 format (
     &  '  Command "volume":  acceptable input lines include:' /
     &  '  "volume [help]"  Display this message.' //
     &  '  "volume PAXIS VAXIS PNAME1 PNAME2 PNAME3 ..."' /
     &  '     Find the volume of revolution around the axis',
     &  ' through' /
     &  '     point PAXIS with direction vector VAXIS, of the polygon' /
     &  '     with vertices PNAME1, PNAME2, PNAME3, ...' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Test input line for validity.

      nerr = 0

 8115 format ('  volume ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of vertex points not 3-36:  ',i3)

      nver = nword - 3

      if ((nver .lt. 3) .or. (nver .gt. 36)) then
        nerr = 1
        write (   6, 8115) ainput, ninput, nver
        write (iout, 8115) ainput, ninput, nver
      endif

c=======================================================================********

c.... Find the point on the axis.

      call ptfind (2, np)

      if (np .eq. 0) then
        nerr = 1
        go to 115
      endif

      xaxis = x(np)
      yaxis = y(np)
      zaxis = z(np)

  115 continue

c=======================================================================********

c.... Find the axis vector, and the corresponding unit vector.

 8130 format ('  volume ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Axis vector has zero length:  ',a)

      call vectfind (3, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 130
      endif

      call aptvusz (xcom(nv), ycom(nv), zcom(nv), tol,
     &              ux, uy, uz, vlength)

      if (vlen(nv) .le. 0.0) then
        nerr = 1
        write (   6, 8130) ainput, ninput, aword(3)
        write (iout, 8130) ainput, ninput, aword(3)
      endif

c=======================================================================********

c.... Input line:  volume PAXIS VAXIS PNAME1 PNAME2 PNAME3 ...

c.... Note:  cannot use cluster as input, because points may be reordered.

c.... Find the points defining the polygon, and the average point.

  130 npver = 0

      puavg = 0.0
      pvavg = 0.0
      pwavg = 0.0

      do 150 nw = 4, nword

        call ptfind (nw, np)

        if (np .eq. 0) then
          nerr = 1
          go to 150
        endif

        npver     = npver + 1
        xp(npver) = x(np)
        yp(npver) = y(np)
        zp(npver) = z(np)
        puavg     = puavg + x(np)
        pvavg     = pvavg + y(np)
        pwavg     = pwavg + z(np)

  150 continue

      if (nerr .ne. 0) go to 999

      puavg = puavg / npver
      pvavg = pvavg / npver
      pwavg = pwavg / npver

c=======================================================================********

c.... Convert the average point into the current coordinate system.

      if     (asys .eq. 'rect') then
        ksys = 0
      elseif (asys .eq. 'cyl')  then
        ksys = 1
      elseif (asys .eq. 'sph')  then
        ksys = 2
      endif

      if (ksys .ne. 0) then

        if     (angunit .eq. 'degrees') then
          iunit = 0
        elseif (angunit .eq. 'radians') then
          iunit = 1
        endif

        call aptcsys (0, ksys, iunit, puavg, pvavg, pwavg, 1, tol,
     &                nerra)

      endif

 8180 format ('  Average vertex  ',1p3e20.12)

      write (iout, 8180) puavg, pvavg, pwavg
      write (   6, 8180) puavg, pvavg, pwavg

c=======================================================================********

c.... Find the average edge.

      edgem = 0.0
  
      do 160 np = 1, npver
        next = mod (np, npver) + 1
        edgem = edgem + sqrt ((xp(np) - xp(next))**2 +
     &                        (yp(np) - yp(next))**2 +
     &                        (zp(np) - zp(next))**2)
  160 continue

      edgem = edgem / npver

 8185 format ('  Average edge    ',1pe20.12)

      write (   6, 8185) edgem
      write (iout, 8185) edgem

c=======================================================================********

c.... Find the rotation operator to make vector VAXIS parallel to the z axis.

      call aptrotv (ux, uy, uz, 0.0, 0.0, 1.0, tol, rotm, nerrpg)

c.... Rotate the points to make vector VAXIS parallel to the z axis.

      call aptmopv (rotm, 0, 0.0, 0.0, 0.0, xp, yp, zp, npver, tol,
     &              nerra)

c.... Translate the points to put point PAXIS at the origin.

      call apttran (xaxis, yaxis, zaxis, xp, yp, zp, npver, tol, nerra)

c=======================================================================********

c.... Find the area of the polygon and the average radius from the axis.

      areapg  = 0.0
      arerr   = 0.0
      volpg   = 0.0
      volerr  = 0.0
  
      do 170 np = 1, npver

        next = mod (np, npver) + 1
        rcyl1 = sqrt (xp(np)**2 + yp(np)**2)
        rcyl2 = sqrt (xp(next)**2 + yp(next)**2)

        darea   = 0.5 * (rcyl1 + rcyl2) * (zp(next) - zp(np))
        areapg  = areapg + darea
        darerr  = 0.5 * tol * (rcyl1 + rcyl2) *
     &                        (abs (zp(np)) + abs (zp(next)))
        arerr   = arerr + darerr

        rcyla   = 0.25 * (rcyl1 + rcyl2)
        dvol    = rcyla * darea
        volpg   = volpg + dvol
        volerr  = volerr + rcyla * darerr

  170 continue

      areapg = abs (areapg)
      if (areapg .le. arerr) areapg = 0.0

      volpg = abs (volpg)
      if (volpg .le. volerr) volpg = 0.0

      rcylpg  = 1.e99
      rcylerr = 1.e99
      if ((areapg .ne. 0.0) .and. (volpg .ne. 0.0)) then
        rcylpg  = volpg / areapg
        rcylerr = rcylpg * (volerr / volpg + arerr / areapg)
      endif

      volpg = 2.0 * pi * volpg

 8190 format ('  Projected area  ',1pe20.12)
 8192 format ('  Average radius  ',1pe20.12)
 8194 format ('  Toroidal volume ',1pe20.12)

      write (   6, 8190) areapg
      write (iout, 8190) areapg
      write (   6, 8192) rcylpg
      write (iout, 8192) rcylpg
      write (   6, 8194) volpg
      write (iout, 8194) volpg

c=======================================================================********

  999 return

c.... End of subroutine volume.         (+1 line)
      end
