      subroutine vout (n1, n2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  vout     Updated     2004 November 23 13:30.
c               vout     Originated  1993 November 3 13:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the components, length and bound point of vectors
c               n1 to n2.
c
c     Logic:    Entry points:  vout.
c vout          Called by:  copy, delete, matrix, movex, quadric, renamea,
c               renamex, search, vectfind, vector, voper, vstore.
c
c vout          Calls:  aptcsyv, aptvusz.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables
      common /ccvout/ atype           ! Vector type, absolute or relative.
      character*8     atype           ! Vector type, absolute or relative.

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. nvectors) nn2 = nvectors

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

      do 120 n = nn1, nn2             ! Loop over vectors to display.

        if (avector(n) .eq. ' ') go to 120

c....   Find the vector length, based on the current value of tol.

        du = xcom(n)
        dv = ycom(n)
        dw = zcom(n)

        call aptvusz (du, dv, dw, tol, u, v, w, vlen(n))

        u  = xbnd(n)
        v  = ybnd(n)
        w  = zbnd(n)

c....   Convert units, if necessary, for output.

        if     (asys .eq. 'cyl') then
          call aptcsyv (0, 1, iunit, u, v, w, du, dv, dw, 1, tol, nerra)
        elseif (asys .eq. 'sph') then
          call aptcsyv (0, 2, iunit, u, v, w, du, dv, dw, 1, tol, nerra)
        endif

c....   Print the vector data.

c????  8120   format ('coord  ',a,5x,a,4x,a,4x,a)
 8122   format ('v ',a,1p3e17.9)
 8125   format ('v ',a,1p3e20.12)
 8225   format ('vector ',a8,' = ',1p3e20.12)

c????         write (   6, 8120) asys, acoordu, acoordv, acoordw
c????         write (iout, 8120) asys, acoordu, acoordv, acoordw
        if (lavector(n) .gt. 16) then
          write (   6, 8122) avector(n), du, dv, dw
          write (iout, 8122) avector(n), du, dv, dw
        elseif (lavector(n) .gt. 8) then
          write (   6, 8125) avector(n), du, dv, dw
          write (iout, 8125) avector(n), du, dv, dw
        else
          write (   6, 8225) avector(n), du, dv, dw
          write (iout, 8225) avector(n), du, dv, dw
        endif

 8126   format ('  bound at',8x,1p3e20.12 )
 8127   format ('  length',10x,1pe20.12 )

        if ((u**2 + v**2 + w**2) .gt. 0.0) then
          write (   6, 8126) u, v, w
          write (iout, 8126) u, v, w
        endif
        write (   6, 8127) vlen(n)
        write (iout, 8127) vlen(n)

 8128   format ('  WARNING:  all components of vector ',a,
     &    ' < tol =',1pe13.5) 

        if (vlen(n) .eq. 0.0) then
          write (   6, 8128) avector(n), tol
          write (iout, 8128) avector(n), tol
        endif

  120 continue                        ! End of loop over vectors to display.

  999 return

c.... End of subroutine vout.         (+1 line)
      end
