      subroutine zone

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  zone     Updated     2005 April 5 17:45.
c               zone     Originated  1994 January 14 13:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create zones, bounded by quadric surfaces in
c               3-D space.
c
c     Logic:    Entry points:  zone.
c zone          Called by:  helpall, main.
c
c zone          Calls:  aptsubr, incrsubs, namechk, varfind, znmake, znstore,
c               zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /llzone/ incq(nzquadm)   ! Increment in a quadric surface name.
      common /llzone/ lazquadx(nzquadm) ! Number of characters in azquadx.
      common /llzone/ nzquadx         ! Number of quadric surfaces around zone.

      common /llzone/ azquadx(nzquadm)! Name of a quadric surface.
      character*24    azquadx         ! Name of a quadric surface.
      common /llzone/ azsignx(nzquadm)! Exit side of a quadric surface (+ or -).
      character*1     azsignx         ! Exit side of a quadric surface (+ or -).

      character*8  akey               ! Key word option in command "zone".
      character*8  amount             ! Amount of output data.
      character*8  ans                ! User's response.

      character*80 aqlast             ! Temporary name of a quadric surface.
      character*80 aqu                ! Temporary name of a quadric surface.
      character*80 aqun               ! Temporary name of a quadric surface.
      character*80 aqv                ! Temporary name of a quadric surface.
      character*80 aqvn               ! Temporary name of a quadric surface.
      character*80 aqw                ! Temporary name of a quadric surface.
      character*80 aqwn               ! Temporary name of a quadric surface.

      character*80 azlast             ! Temporary name of a zone.
      character*80 azonex             ! Temporary name of a zone.
      character*80 azu                ! Temporary name of a zone.
      character*80 azun               ! Temporary name of a zone.
      character*80 azuv               ! Temporary name of a zone.
      character*80 azuvw              ! Temporary name of a zone.
      character*80 azv                ! Temporary name of a zone.
      character*80 azvn               ! Temporary name of a zone.
      character*80 azvw               ! Temporary name of a zone.
      character*80 azw                ! Temporary name of a zone.
      character*80 azwn               ! Temporary name of a zone.
      character*80 azwu               ! Temporary name of a zone.

c=======================================================================********

c.... Input line:  zone [help]

 8110 format (
     &  '  Command "zone":  acceptable input lines include:' //
     &  '  "zone [help]"  Display this message.' /
     &  '  "zone all"     Display all zones.' /
     &  '  "zone ZNAME"   Display zone ZNAME.' /
     &  '  "zone list [ZNAME1 ZNAME2 ZNAME3 ...]"' /
     &  '     Display all listed zones',
     &  ' (default is all zones).' //
     &  '  "zone ZNAME [=,&] [+,-|+] Q1 [+,-|+] Q2 [+,-|+] Q3 ..."' /
     &  '     Create [=] or add surfaces to [&] zone ZNAME, which is' /
     &  '     opposite the [+,-] side of quadric surface Q1,' /
     &  '     opposite the [+,-] side of quadric surface Q2, etc.,' /
     &  '     where the + side is in the direction of the',
     &  ' normal vector.' //
     &  '  "zone ZNAME volume ZVOL"' /
     &  '     Assign zone ZNAME volume ZVOL.' )

 8112 format (/
     &  '  "zone ZNAME02 increment NUMZN INC ZNAME01 IQ1 IQ2 ..."' /
     &  '     Create a family of NUMZN zones ZNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by incrementing the names of the bounding',
     &  ' surfaces of' /
     &  '     the preceding zone by the number of characters',
     &  ' indicated' /
     &  '     by the integers IQ1, IQ2, ..., starting from',
     &  ' zone ZNAME01.' //
     &  '  Type <return> for more help.' )

 8113 format (a)

 8114 format (/
     &  '  "zone ZNAME family IZU AQU IQU NQU"' /
     &  '     Create a family of NQU-1 zones ZNAME, ...,' /
     &  '     with names incremented by IZU characters,' /
     &  '     bounded by the NQU quadric surfaces AQU, ...,' /
     &  '     with names incremented by IQU characters, and' /
     &  '     ordered in the direction of their normal vectors.' /
     &  '     Note that the normal vector of a circular cone reverses',
     &  ' direction' /
     &  '     at a vertex angle of 90 degrees.' )

 8115 format (/
     &  '  "zone ZNAME family IZU AQU IQU NQU family IZV AQV IQV NQV"' /
     &  '     Like the preceding command, but' /
     &  '     with each zone expanded to a family of NQV-1 zones,' /
     &  '     with names incremented by IZV characters,' /
     &  '     bounded by the NQV quadric surfaces AQV, ...,' /
     &  '     with names incremented by IQV characters, and' /
     &  '     ordered in the direction of their normal vectors.' //
     &  '  "zone ZNAME family IZU AQU IQU NQU family IZV AQV IQV NQV',
     &  ' family IZW AQW IQW NQW"' /
     &  '     Like the preceding command, but' /
     &  '     with each zone expanded to a family of NQW-1 zones,' /
     &  '     with names incremented by IZW characters,' /
     &  '     bounded by the NQW quadric surfaces AQW, ...,' /
     &  '     with names incremented by IQW characters, and' /
     &  '     ordered in the direction of their normal vectors.' )

 8116 format (/
     &  '  Note:  use +/- for ZNAME or ZNAME02 to incr/decr the last',
     &  ' zone name:  ',a //
     &  '  Synonyms:  [help, h], [increment, incr], [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8113) ans
        endif
        write (   6, 8114)
        write (iout, 8114)
        write (   6, 8115)
        write (iout, 8115)
        write (   6, 8116) azones
        write (iout, 8116) azones
        go to 999
      endif

c=======================================================================********

c.... Input line:  zone all
c.... Input line:  zone list

 8120 format (/ 'Number of zones = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nzonex = 0
        call zout (1, nzones, 'min')
        do 120 n = 1, nzones
          if (azone(n) .eq. ' ') go to 120
          nzonex = nzonex + 1
  120   continue
        write (   6, 8120) nzonex
        write (iout, 8120) nzonex
        go to 999
      endif

c=======================================================================********

c.... Input line:  zone list ZNAME1 ZNAME2 ZNAME3 ...

 8130 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 140 nw = 3, nword
          nfind = 0
          do 130 n = 1, nzones
            call aptsubr (azone(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. azone(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call zout (n, n, 'min')
            endif
  130     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  140   continue
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

 8150 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of zone not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8150) ainput, ninput, aword(2)
        write (iout, 8150) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Input line:  zone ZNAME

      if (nword .eq. 2) then
        do 150 n = 1, nzones
          if (aword(2) .eq. azone(n)) then
            amount = 'maximum'
            call zout (n, n, amount)
            go to 999
          endif
  150   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Input line:  zone ZNAME volume ZVOL

      if ((aword(3) .eq. 'volume') .or.
     &    (aword(3) .eq. 'vol'))   then

 8152 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on "zone ZNAME volume ZVOL line:  ',i3)

        if (nword .ne. 4) then
          write (   6, 8152) ainput, ninput, nword
          write (iout, 8152) ainput, ninput, nword
          go to 999
        endif

        do 154 nz = 1, nzones
          if (aword(2) .eq. azone(nz)) then
            n = nz
            go to 155
          endif
  154   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999

 8156 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 4 not integer or floating point value or variable:  ',a)

  155   call varfind (4, 4)

        if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
          write (   6, 8156) ainput, ninput, aword(4)
          write (iout, 8156) ainput, ninput, aword(4)
          go to 999
        endif

        volz(n)    = fword(4)
        volzerr(n) = 0.0

        call zout (nz, nz, 'min')

        go to 999

      endif

c=======================================================================********

c.... See if last zone name should be incremented and used.

 8128 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last zone can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = azones
        lword(2) = lazones
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, azones
          write (iout, 8128) ainput, ninput, azones
          go to 999
        endif
      endif

 8129 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last zone can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = azones
        lword(2) = lazones
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, azones
          write (iout, 8129) ainput, ninput, azones
          go to 999
        endif
      endif

 8905 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of zone too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  zone ZNAME02 increment NUMZN INC ZNAME01 IQ1 IQ2 ...

      if ((aword(3) .eq. 'increment') .or.
     &    (aword(3) .eq. 'incr'))     then

        nerr = 0

 8505   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Too few words in input line:  ',i3)

        if (nword .lt. 7) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          go to 999
        endif

 8520   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMZN is not a positive integer:  ',a)
 8525   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMZN is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numzn = iword(4)
          if (numzn .lt. 1) then
            go to 999
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          go to 999
        endif

 8510   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero:  ',a)
 8515   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput, aword(5)
            write (iout, 8510) ainput, ninput, aword(5)
            go to 999
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          go to 999
        endif

c....   Find the base zone.

        do 530 nz = 1, nzones
          if (aword(6) .eq. azone(nz)) then
            nbase = nz
            call zout (nbase, nbase, 'min')
            go to 540
          endif
  530   continue
        write (   6, 8130) ainput, ninput, aword(6)
        write (iout, 8130) ainput, ninput, aword(6)
        go to 999

c....   Find the increments to the bounding surface names.

  540   do 550 nzq = 1, nzquadm
          incq(nzq) = 0
  550   continue

 8560   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Surface name increment not integer:  ',a)

        call varfind (7, nword)

        do 560 nw = 7, nword
          if (mtype(nw) .eq. 2) then
            incq(nw-6) = iword(nw)
          else
            write (   6, 8560) ainput, ninput, aword(nw)
            write (iout, 8560) ainput, ninput, aword(nw)
            go to 999
          endif
  560   continue

c....   See if the zone names can be generated uniquely.

 8570   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate',i5,' zone names from:  ',a)

        azonex  = aword(2)
        lazonex = lword(2)
        namex   = inc * (numzn - 1)
cbugc***DEBUG begins.
cbug 9701 format ('DEBUG #1 namex=',i3,'  azonex=',a /
cbug     &  'lazonex=',i3,'  nerra=',i3)
cbug      write (iout, 9701) namex, azonex, lazonex, nerra
cbugc***DEBUG ends.

        call incrsubs (namex, azonex, 1, lazonex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8570) ainput, ninput, numzn, aword(2)
          write (iout, 8570) ainput, ninput, numzn, aword(2)
cbugc***DEBUG begins.
cbug 9702 format ('DEBUG #2 namex=',i3,'  azonex=',a /
cbug     &  'lazonex=',i3,'  nerra=',i3)
cbug      write (iout, 9702) namex, azonex, lazonex, nerra
cbugc***DEBUG ends.
          go to 999
        endif

        call namechk (azonex, lazonex, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, azonex
          write (iout, 8905) ainput, ninput, azonex
          go to 999
        endif

        if (nerr .ne. 0) go to 999

c....   Create the new zones.

        azonex  = aword(2)            ! Initial zone name.
        lazonex = lword(2)            ! Initial zone name.

        do 590 nzn = 1, numzn         ! Loop over the new zones.

c....     Find the index in the zone list for the new zone.

          aword(2) = azonex
          lword(2) = lazonex

          call znstore (nrep, n)

          if (n .eq. 0) go to 999

          nzquadx = nzquads(nbase)

c....     Increment the names of the bounding surfaces.

          do 580 nq = 1, nzquadx

            azsignx(nq)  = azsign(nq,nbase)
            azquadx(nq)  = azquad(nq,nbase)
            lazquadx(nq) = lazquad(nq,nbase)

            if (incq(nq) .eq. 0) go to 580

cbugc***DEBUG begins.
cbug 9703 format ('DEBUG #3 incq=',i3,'  azquadx=',a /
cbug     &  'lazquadx=',i3,'  nerra=',i3)
cbug      write (iout, 9703) incq(nq), azquadx(nq), lazquadx(nq), nerra
cbugc***DEBUG ends.
            call incrsubs (incq(nq), azquadx(nq), 1, lazquadx(nq),
     &                     nerra)

 8572       format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Can not increment quadric surface name:  ',a)

            if (nerra .ne. 0) then
              write (   6, 8572) ainput, ninput, azquad(nq,nbase)
              write (iout, 8572) ainput, ninput, azquad(nq,nbase)
              go to 999
            endif

 8574       format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Duplicate bounding surface name:  ',a)

            do 574 nnn = 1, nq - 1
              if (azquadx(nq) .eq. azquadx(nnn)) then
                write (   6, 8574) ainput, ninput, azquadx(nq)
                write (iout, 8574) ainput, ninput, azquadx(nq)
                go to 999
              endif
  574       continue

 8210       format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named quadric surface does not exist:  ',a)

            do 575 nn = 1, nquads
              if (azquadx(nq) .eq. aquad(nn)) go to 580
  575       continue
cbugc***DEBUG begins.
cbug      nbug = 9
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
            write (   6, 8210) ainput, ninput, azquadx(nq)
            write (iout, 8210) ainput, ninput, azquadx(nq)
            go to 999

  580     continue
    
          azones     = azonex
          lazones    = lazonex

          azone(n)   = azonex
          nzquads(n) = nzquadx

          do 585 nq = 1, nzquads(n)
            azsign(nq,n)  = azsignx(nq)
            azquad(nq,n)  = azquadx(nq)
            lazquad(nq,n) = lazquadx(nq)
  585     continue

          volz(n)    = 0.0
          volzerr(n) = 0.0

          call zout (n, n, 'min')

          nbase      = n

c....     Find the next zone name, by incrementing by inc.

          call incrsubs (inc, azonex, 1, lazonex, nerra)

          call namechk (azonex, lazonex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, azonex
            write (iout, 8905) ainput, ninput, azonex
            go to 999
          endif

  590   continue                      ! End of loop over the new zones.

        go to 999

      endif                           ! Create a family of zones, option "incr".

c===============================================================================

c.... Input line:  zone ZNAME family IZU AQU IQU NQU
c.... Input line:  zone ZNAME family IZU AQU IQU NQU family IZV AQV IQV NQV
c.... Input line:  zone ZNAME family IZU AQU IQU NQU family IZV AQV IQV NQV
c....              family IZW AQW IQW NQW                     ! All on one line.

 8310 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7, 12 or 17 words with "family" option:  ',i3)

      if ((aword(3) .eq. 'family') .or.
     &    (aword(3) .eq. 'fam'))   then

        akey    = 'family'            ! 1-D, 2-D or 3-D zoning.
        nerr    = 0
        nwordx  = nword
        azonex  = aword(2)
        lazonex = lword(2)

c....   Verify the total number of arguments.

        if ((nword. ne.  7)  .and.
     &      (nword .ne. 12)  .and.
     &      (nword .ne. 17)) then
          write (   6, 8310) ainput, ninput, nword
          write (iout, 8310) ainput, ninput, nword
          go to 999
        endif

c....   Verify arguments 4-7 of the input line.

c....   Find the zone name increment in direction U.

 8320   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Increment in zone name (arg ',i2,') is zero:  ',a)
 8330   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Increment in zone name (arg ',i2,') is not an integer' /
     &    '  or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          izu = iword(4)
          if (izu .eq. 0) then
            nw = 4
            write (   6, 8320) ainput, ninput, nw, aword(4)
            write (iout, 8320) ainput, ninput, nw, aword(4)
            go to 999
          endif
        else
          nw = 4
          write (   6, 8330) ainput, ninput, nw, aword(4)
          write (iout, 8330) ainput, ninput, nw, aword(4)
          go to 999
        endif

c....   Find the quadric surface name, and its increment in direction U.

 8340   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Increment in quadric surface name (arg ',i2,') is zero.')
 8350   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Increment in quadric surface name (arg ',i2,') is not',
     &    ' an integer' /
     &    '  or integer variable:  ',a)

        aqu  = aword(5)
        laqu = lword(5)

        call varfind (6, 6)

        if (mtype(6) .eq. 2) then
          iqu = iword(6)
          if (iqu .eq. 0) then
            nw = 6
            write (   6, 8340) ainput, ninput, nw
            write (iout, 8340) ainput, ninput, nw
            go to 999
          endif
        else
          nw = 6
          write (   6, 8350) ainput, ninput, nw, aword(6)
          write (iout, 8350) ainput, ninput, nw, aword(6)
          go to 999
        endif

c....   Find the number of quadric surfaces in direction U.

 8360   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of quadric surfaces (arg ',i2,') is less',
     &    ' than 2:  ',i5)
 8370   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of quadric surfaces (arg ',i2,') is not an',
     &    ' integer' /
     &    '  or integer variable:  ',a)

        call varfind (7, 7)

        if (mtype(7) .eq. 2) then
          nqu = iword(7)
          if (nqu .lt. 2) then
            nw = 7
            write (   6, 8360) ainput, ninput, nw, nqu
            write (iout, 8360) ainput, ninput, nw, nqu
            go to 999
          endif
        else
          nw = 7
          write (   6, 8370) ainput, ninput, nw, aword(7)
          write (iout, 8370) ainput, ninput, nw, aword(7)
          go to 999
        endif

c....   See if the zone name can be incremented in U direction.

 8410   format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not increment zone name by ',i5,':  ',a)

        iztotu = 0
        if ((izu .ne. 0)  .and.
     &      (nqu .ge. 2)) then

          iztotu  = izu * (nqu - 1)
          azu     = azonex
          lazu    = lazonex
          azlast  = azonex
          lazlast = lazonex

          call incrsubs (iztotu, azu, 1, lazu, nerra)
  
          if (nerra .ne. 0) then
            write (   6, 8410) ainput, ninput, iztotu, aword(2)
            write (iout, 8410) ainput, ninput, iztotu, aword(2)
            go to 999
          endif

          call namechk (azu, lazu, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, azu
            write (iout, 8905) ainput, ninput, azu
            go to 999
            azu  = azlast
            lazu = lazlast
          endif

        endif                         ! Tested zone increment in U direction.

c....   See if all of the quadric surfaces exist in direction U.

        aqun  = aqu
        laqun = laqu

        do 377 n = 1, nqu
          do 375 nq = 1, nquads
cbugc***DEBUG begins.
cbug 8702 format ('zone DEBUG nbug=',i3,'  n=',i3,'  nq=',i3)
cbug 8701 format ('  aqun=',a8,' aquad(nq)=',a8)
cbug      nbug = 1
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
            if (aqun .eq. aquad(nq)) go to 376
cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
  375     continue
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
          write (   6, 8210) ainput, ninput, aqun
          write (iout, 8210) ainput, ninput, aqun
          go to 999
  376     continue
cbugc***DEBUG begins.
cbug      nbug = 4
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
          if (n .lt. nqu) then
            aqlast  = aqun
            laqlast = laqun

            call incrsubs (iqu, aqun, 1, laqun, nerra)
    
            if (nerra .ne. 0) then
              write (   6, 8572) ainput, ninput, aqlast
              write (iout, 8572) ainput, ninput, aqlast
              go to 999
            endif
          endif
cbugc***DEBUG begins.
cbug      nbug = 5
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
  377   continue
cbugc***DEBUG begins.
cbug      nbug = 6
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
  378   continue

c-------------------------------------------------------------------------------

c....   Verify arguments 9-12 of the input line.

        if ((aword(8) .eq. 'family') .or.
     &      (aword(8) .eq. 'fam'))   then

c....     Find the zone name increment in direction V.

          call varfind (9, 9)

          if (mtype(9) .eq. 2) then
            izv = iword(9)
            if (izv .eq. 0) then
              nw = 9
              write (   6, 8320) ainput, ninput, nw
              write (iout, 8320) ainput, ninput, nw
              go to 999
            endif
          else
            nw = 9
            write (   6, 8330) ainput, ninput, nw, aword(9)
            write (iout, 8330) ainput, ninput, nw, aword(9)
            go to 999
          endif

c....     Find the quadric surface name, and its increment in direction V.

          aqv  = aword(10)
          laqv = lword(10)

          call varfind (11, 11)

          if (mtype(11) .eq. 2) then
            iqv = iword(11)
            if (iqv .eq. 0) then
              nw = 11
              write (   6, 8340) ainput, ninput, nw
              write (iout, 8340) ainput, ninput, nw
              go to 999
            endif
          else
            nw = 11
            write (   6, 8350) ainput, ninput, nw, aword(11)
            write (iout, 8350) ainput, ninput, nw, aword(11)
            go to 999
          endif

c....     Find the number of quadric surfaces in direction V.

          call varfind (12, 12)

          if (mtype(12) .eq. 2) then
            nqv = iword(12)
            if (nqv .lt. 2) then
              nw = 12
              write (   6, 8360) ainput, ninput, nw, nqv
              write (iout, 8360) ainput, ninput, nw, nqv
              go to 999
            endif
          else
            nw = 12
            write (   6, 8370) ainput, ninput, nw, aword(12)
            write (iout, 8370) ainput, ninput, nw, aword(12)
            go to 999
          endif

c....     See if the zone name can be incremented in V direction.

          iztotv = 0
          if ((izv .ne. 0)  .and.
     &        (nqv .ge. 2)) then

            iztotv = izv * (nqv - 1)
            azv     = azonex
            lazv    = lazonex
            azlast  = azonex
            lazlast = lazonex

            call incrsubs (iztotv, azv, 1, lazv, nerra)
  
            if (nerra .ne. 0) then
              write (   6, 8410) ainput, ninput, iztotv, aword(2)
              write (iout, 8410) ainput, ninput, iztotv, aword(2)
              go to 999
            endif

            call namechk (azv, lazv, nerrchk)

            if (nerrchk .ne. 0) then
              write (   6, 8905) ainput, ninput, azv
              write (iout, 8905) ainput, ninput, azv
              go to 999
            endif

            iztotuv = iztotu + iztotv
            azuv    = azu
            lazuv   = lazu
            azlast  = azu
            lazlast = lazu

            call incrsubs (iztotv, azuv, 1, lazuv, nerra)
  
            if (nerra .ne. 0) then
              write (   6, 8410) ainput, ninput, iztotuv, aword(2)
              write (iout, 8410) ainput, ninput, iztotuv, aword(2)
              go to 999
            endif

            call namechk (azuv, lazuv, nerrchk)

            if (nerrchk .ne. 0) then
              write (   6, 8905) ainput, ninput, azuv
              write (iout, 8905) ainput, ninput, azuv
              go to 999
            endif

          endif                         ! Tested zone increment in V direction.

c....     See if all of the quadric surfaces exist in direction V.

          aqvn  = aqv
          laqvn = laqv

          do 387 n = 1, nqv
            do 385 nq = 1, nquads
              if (aqvn .eq. aquad(nq)) go to 386
  385       continue
cbugc***DEBUG begins.
cbug      nbug = 7
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
            write (   6, 8210) ainput, ninput, aqvn
            write (iout, 8210) ainput, ninput, aqvn
            go to 999
  386       if (n .lt. nqv) then
              aqlast  = aqvn
              laqlast = laqvn

              call incrsubs (iqv, aqvn, 1, laqvn, nerra)
    
              if (nerra .ne. 0) then
                write (   6, 8572) ainput, ninput, aqlast
                write (iout, 8572) ainput, ninput, aqlast
                go to 999
              endif
            endif
  387     continue
  388     continue

        endif                         ! Verified arguments 9-12.

c-------------------------------------------------------------------------------

c....   Verify arguments 14-17 of the input line.

        if ((aword(13) .eq. 'family') .or.
     &      (aword(13) .eq. 'fam'))   then

c....     Find the zone name increment in direction W.

          call varfind (14, 14)

          if (mtype(14) .eq. 2) then
            izw = iword(14)
            if (izw .eq. 0) then
              nw = 14
              write (   6, 8320) ainput, ninput, nw
              write (iout, 8320) ainput, ninput, nw
              go to 999
            endif
          else
            nw = 14
            write (   6, 8330) ainput, ninput, nw, aword(14)
            write (iout, 8330) ainput, ninput, nw, aword(14)
            go to 999
          endif

c....     Find the quadric surface name, and its increment in direction W.

          aqw  = aword(15)
          laqw = lword(15)

          call varfind (16, 16)

          if (mtype(16) .eq. 2) then
            iqw = iword(16)
            if (iqw .eq. 0) then
              nw = 16
              write (   6, 8340) ainput, ninput, nw
              write (iout, 8340) ainput, ninput, nw
              go to 999
            endif
          else
            nw = 16
            write (   6, 8350) ainput, ninput, nw, aword(16)
            write (iout, 8350) ainput, ninput, nw, aword(16)
            go to 999
          endif

c....     Find the number of quadric surfaces in direction W.

          call varfind (17, 17)

          if (mtype(17) .eq. 2) then
            nqw = iword(17)
            if (nqw .lt. 2) then
              nw = 17
              write (   6, 8360) ainput, ninput, nw, nqw
              write (iout, 8360) ainput, ninput, nw, nqw
              go to 999
            endif
          else
            nw = 17
            write (   6, 8370) ainput, ninput, nw, aword(17)
            write (iout, 8370) ainput, ninput, nw, aword(17)
            go to 999
          endif

c....     See if the zone name can be incremented in w direction.

          iztotw = 0
          if ((izw .ne. 0)  .and.
     &        (nqw .ge. 2)) then

            iztotw  = izw * (nqw - 1)
            azw     = azonex
            lazw    = lazonex
            azlast  = azonex
            lazlast = lazonex

            call incrsubs (iztotw, azw, 1, lazw, nerra)
  
            if (nerra .ne. 0) then
              write (   6, 8410) ainput, ninput, iztotw, aword(2)
              write (iout, 8410) ainput, ninput, iztotw, aword(2)
              go to 999
              azw  = azlast
              lazw = lazlast
            endif

            call namechk (azw, lazw, nerrchk)

            if (nerrchk .ne. 0) then
              write (   6, 8905) ainput, ninput, azw
              write (iout, 8905) ainput, ninput, azw
              go to 999
            endif

            iztotwu = iztotw + iztotu
            azwu    = azu
            lazwu   = lazu
            azlast  = azu
            lazlast = lazu

            call incrsubs (iztotw, azwu, 1, lazwu, nerra)
  
            if (nerra .ne. 0) then
              write (   6, 8410) ainput, ninput, iztotwu, aword(2)
              write (iout, 8410) ainput, ninput, iztotwu, aword(2)
              go to 999
            endif

            call namechk (azwu, lazwu, nerrchk)

            if (nerrchk .ne. 0) then
              write (   6, 8905) ainput, ninput, azwu
              write (iout, 8905) ainput, ninput, azwu
              go to 999
              azwu  = azlast
              lazwu = lazlast
            endif

            iztotvw = iztotv + iztotw
            azvw    = azv
            lazvw   = lazv
            azlast  = azv
            lazlast = lazv

            call incrsubs (iztotw, azvw, 1, lazvw, nerra)
  
            if (nerra .ne. 0) then
              write (   6, 8410) ainput, ninput, iztotvw, aword(2)
              write (iout, 8410) ainput, ninput, iztotvw, aword(2)
              go to 999
              azvw  = azlast
              lazvw = lazlast
            endif

            call namechk (azvw, lazvw, nerrchk)

            if (nerrchk .ne. 0) then
              write (   6, 8905) ainput, ninput, azvw
              write (iout, 8905) ainput, ninput, azvw
              go to 999
              azvw  = azlast
              lazvw = lazlast
            endif

            iztotuvw = iztotu + iztotv + iztotw
            azuvw    = azuv
            lazuvw   = lazuv
            azlast   = azuv
            lazlast  = lazuv

            call incrsubs (iztotw, azuvw, 1, lazuvw, nerra)
  
            if (nerra .ne. 0) then
              write (   6, 8410) ainput, ninput, iztotuvw, aword(2)
              write (iout, 8410) ainput, ninput, iztotuvw, aword(2)
              go to 999
            endif

            call namechk (azuvw, lazuvw, nerrchk)

            if (nerrchk .ne. 0) then
              write (   6, 8905) ainput, ninput, azuvw
              write (iout, 8905) ainput, ninput, azuvw
              go to 999
            endif

          endif                         ! Tested zone increment in w direction.

c....     See if all of the quadric surfaces exist in direction V.

          aqwn  = aqw
          laqwn = laqw

          do 397 n = 1, nqw
            do 395 nq = 1, nquads
              if (aqwn .eq. aquad(nq)) go to 396
  395       continue
cbugc***DEBUG begins.
cbug      nbug = 8
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
            write (   6, 8210) ainput, ninput, aqwn
            write (iout, 8210) ainput, ninput, aqwn
            go to 999
  396       if (n .lt. nqw) then
              aqlast  = aqwn
              laqlast = laqwn

              call incrsubs (iqw, aqwn, 1, laqwn, nerra)
    
              if (nerra .ne. 0) then
                write (   6, 8572) ainput, ninput, aqlast
                write (iout, 8572) ainput, ninput, aqlast
                go to 999
                go to 398
              endif
            endif
  397     continue
  398     continue

        endif                         ! Verified arguments 14-17.

        if (nerr .ne. 0) go to 999

c=======================================================================********

c....   Loop over zones.

        if (nwordx .eq.  7) nword =  6
        if (nwordx .eq. 12) nword =  9
        if (nwordx .eq. 17) nword = 12

        azun     = azonex
        lazun    = lazonex
        aqun     = aqu
        laqun    = laqu

        aword(3)  = '='
        aword(4)  = '-'
        aword(7)  = '-'
        aword(10) = '-'

        do 430 nnqu = 1, nqu - 1      ! Loop in U direction.

          aword(2) = azun
          lword(2) = lazun
          aword(5) = aqun
          lword(5) = laqun

          call incrsubs (iqu, aqun, 1, laqun, nerra)

          aword(6) = aqun
          lword(6) = laqun

          if (nword .eq. 6) then      ! 1-D.

cbugc***DEBUG begins.
cbug      nbug = 10
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
            call znmake (akey)

          else                        ! 2-D or 3-D.

            azvn  = azun
            lazvn = lazun
            aqvn  = aqv
            laqvn = laqv

            do 440 nnqv = 1, nqv - 1  ! Loop in V direction

              aword(2) = azvn
              lword(2) = lazvn
              aword(8) = aqvn
              lword(8) = laqvn

              call incrsubs (iqv, aqvn, 1, laqvn, nerra)

              aword(9) = aqvn
              lword(9) = laqvn

              if (nword .eq. 9) then  ! 2-D 
  
cbugc***DEBUG begins.
cbug      nbug = 11
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
                call znmake (akey)
  
              else                    ! 3-D

                azwn  = azvn
                lazwn = lazvn
                aqwn  = aqw
                laqwn = laqw

                do 450 nnqw = 1, nqw - 1  ! Loop in W direction.

                  aword(2)  = azwn
                  lword(2)  = lazwn
                  aword(11) = aqwn
                  lword(11) = laqwn
  
                  call incrsubs (iqw, aqwn, 1, laqwn, nerra)
  
                  aword(12) = aqwn
                  lword(12) = laqwn
  
cbugc***DEBUG begins.
cbug      nbug = 12
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
                  call znmake (akey)
  
                  call incrsubs (izw, azwn, 1, lazwn, nerra)

                  call namechk (azwn, lazwn, nerrchk)

                  if (nerrchk .ne. 0) then
                    write (   6, 8905) ainput, ninput, azwn
                    write (iout, 8905) ainput, ninput, azwn
                    go to 999
                  endif

  450           continue              ! End of loop in W direction.
  
              endif                   ! 3-D.

              call incrsubs (izv, azvn, 1, lazvn, nerra)

              call namechk (azvn, lazvn, nerrchk)

              if (nerrchk .ne. 0) then
                write (   6, 8905) ainput, ninput, azvn
                write (iout, 8905) ainput, ninput, azvn
                go to 999
              endif

  440       continue                  ! End of loop in V direction.

          endif                       ! 2-D or 3-D.

          call incrsubs (izu, azun, 1, lazun, nerra)

          call namechk (azun, lazun, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, azun
            write (iout, 8905) ainput, ninput, azun
            go to 999
          endif

  430   continue                      ! End of loop in U direction.

        go to 999

      endif                           ! Word 3 is "family".

c===============================================================================

c.... Input line:  zone ZNAME = [+,-|+] Q1 [+,-|+] Q2 [+,-|+] Q3 ...
c.... Input line:  zone ZNAME & [+,-|+] Q1 [+,-|+] Q2 [+,-|+] Q3 ...

c.... Make a zone.

      if ((aword(3) .eq. '=')  .or.
     &    (aword(3) .eq. '&')) then

        akey = ' '

cbugc***DEBUG begins.
cbug      nbug = 13
cbug      write (   6, 8702) nbug, n, nq
cbug      write (iout, 8702) nbug, n, nq
cbug      write (   6, 8701) aqun, aquad(nq)
cbug      write (iout, 8701) aqun, aquad(nq)
cbugc***DEBUG ends.
        call znmake (akey)

        go to 999

      endif

c=======================================================================********

 8440 format ('  zone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8440) ainput, ninput
      write (iout, 8440) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine zone.         (+1 line)
      end
